@@ -77,17 +77,10 @@ public:
     virtual void Render()
     {
     }
-
-    int typeAsTarget()
-    {
-        return TARGET_STACKACTION;
-    }
     
-    Interruptible(bool hasFocus = false) : PlayGuiObject(40,x,y,hasFocus)
+    Interruptible(GameObserver* observer, int inID = 0, bool hasFocus = false)
+        : PlayGuiObject(40, x, y, inID, hasFocus), Targetable(observer), state(NOT_RESOLVED), display(0), source(NULL)
     {
-        state = NOT_RESOLVED;
-        display = 0;
-        source = NULL;
     }
     
     virtual const string getDisplayName() const;
@@ -114,7 +107,7 @@ public:
     void Render();
     virtual ostream& toString(ostream& out) const;
     virtual const string getDisplayName() const;
-    NextGamePhase(int id);
+    NextGamePhase(GameObserver* observer, int id);
 };
 
 class Spell: public Interruptible
@@ -127,9 +120,8 @@ public:
     ManaCost * cost;
     int payResult;
     int computeX(MTGCardInstance * card);
-    int computeXX(MTGCardInstance * card);
-    Spell(MTGCardInstance* _source);
-    Spell(int id, MTGCardInstance* _source, TargetChooser *_tc, ManaCost * _cost, int payResult);
+    Spell(GameObserver* observer, MTGCardInstance* _source);
+    Spell(GameObserver* observer, int id, MTGCardInstance* _source, TargetChooser *_tc, ManaCost * _cost, int payResult);
     ~Spell();
     int resolve();
     void Render();
@@ -142,7 +134,7 @@ public:
     Interruptible * getNextInterruptible(Interruptible * previous, int type);
     Spell * getNextSpellTarget(Spell * previous = 0);
     Damage * getNextDamageTarget(Damage * previous = 0);
-    Targetable * getNextTarget(Targetable * previous = 0, int type = -1);
+    Targetable * getNextTarget(Targetable * previous = 0);
     int getNbTargets();
 };
 
@@ -154,7 +146,7 @@ public:
     void Render();
     virtual ostream& toString(ostream& out) const;
     virtual const string getDisplayName() const;
-    StackAbility(int id, MTGAbility * _ability);
+    StackAbility(GameObserver* observer, int id, MTGAbility * _ability);
 };
 
 class PutInGraveyard: public Interruptible {
@@ -164,7 +156,7 @@ public:
     int resolve();
     void Render();
     virtual ostream& toString(ostream& out) const;
-    PutInGraveyard(int id, MTGCardInstance * _card);
+    PutInGraveyard(GameObserver* observer, int id, MTGCardInstance * _card);
 };
 
 
@@ -176,9 +168,10 @@ public:
     int resolve();
     void Render();
     virtual ostream& toString(ostream& out) const;
-    DrawAction(int id, Player *  _player, int _nbcards);
+    DrawAction(GameObserver* observer, int id, Player *  _player, int _nbcards);
 };
 
+class ATutorialMessage;
 class LifeAction: public Interruptible
 {
 public:
@@ -187,40 +180,46 @@ public:
     int resolve();
     void Render();
     virtual ostream& toString(ostream& out) const;
-    LifeAction(int id, Damageable * _target, int amount);
+    LifeAction(GameObserver* observer, int id, Damageable * _target, int amount);
 };
 
 class ActionStack :public GuiLayer
 {
-protected:
-    JQuadPtr pspIcons[8];
-    GameObserver* game;
-    int interruptDecision[2];
-    float timer;
-    int currentState;
-    int mode;
-    int checked;
-
 public:
-
-    enum
+    typedef enum
     {
         NOT_DECIDED = 0,
         INTERRUPT = -1,
         DONT_INTERRUPT = 1,
-        DONT_INTERRUPT_ALL = 2,
-    };
+        DONT_INTERRUPT_ALL = 2
+    } InterruptDecision;
 
-    int setIsInterrupting(Player * player);
+protected:
+    JQuadPtr pspIcons[8];
+    InterruptDecision interruptDecision[2];
+    float timer;
+    int currentState;
+    int mode;
+    int checked;
+    ATutorialMessage* currentTutorial;
+    int interruptBtnXOffset, noBtnXOffset, noToAllBtnXOffset, interruptDialogWidth;
+
+    JButton handleInterruptRequest( JButton inputKey, int& x, int& y );
+
+public:
+
+    Player * lastActionController;
+    int setIsInterrupting(Player * player, bool log = true);
     int count( int type = 0 , int state = 0 , int display = -1);
+    Interruptible * getActionElementFromCard(MTGCardInstance * card);
     Interruptible * getPrevious(Interruptible * next, int type = 0, int state = 0 , int display = -1);
     int getPreviousIndex(Interruptible * next, int type = 0, int state = 0 , int display = -1);
     Interruptible * getNext(Interruptible * previous, int type = 0, int state = 0 , int display = -1);
     int getNextIndex(Interruptible * previous, int type = 0, int state = 0 , int display = -1);
     void Fizzle(Interruptible * action);
     Interruptible * getAt(int id);
-    void cancelInterruptOffer(int cancelMode = 1);
-    void endOfInterruption();
+    void cancelInterruptOffer(InterruptDecision cancelMode = DONT_INTERRUPT, bool log = true);
+    void endOfInterruption(bool log = true);
     Interruptible * getLatest(int state);
     Player * askIfWishesToInterrupt;
     int garbageCollect();
@@ -244,7 +243,9 @@ public:
 #if defined (WIN32) || defined (LINUX) || defined (IOS)
     void Dump();
 #endif
-
+    void setCurrentTutorial(ATutorialMessage* message) {currentTutorial = message;};
+    ATutorialMessage* getCurrentTutorial() {return currentTutorial;};
+    bool isCalm() {return interruptDecision[0] == NOT_DECIDED && interruptDecision[1] == NOT_DECIDED;};
 };
 
 #endif
