@@ -12,6 +12,13 @@ using std::vector;
 class PlayGuiObject;
 class DuelLayers;
 
+// The X lib annoyingly defines True to be 1, leading to
+// hard to understand syntax errors. Not using it, so it's
+// safe to undefine it.
+#ifdef True
+#undef True
+#endif
+
 template<typename T>
 struct LimitorFunctor
 {
@@ -24,15 +31,15 @@ class CardSelectorBase: public GuiLayer
 {
 public:
 
-    CardSelectorBase(int inDrawMode = DrawMode::kNormal) :
-        mDrawMode(inDrawMode)
+    CardSelectorBase(GameObserver *observer, int inDrawMode = DrawMode::kNormal) :
+        GuiLayer(observer), mDrawMode(inDrawMode)
+
     {
     }
     ;
     virtual void Add(PlayGuiObject*) = 0;
     virtual void Remove(PlayGuiObject*) = 0;
     virtual bool CheckUserInput(JButton key) = 0;
-    virtual bool CheckUserInput(int x, int y) = 0;
     virtual void PushLimitor() = 0;
     virtual void PopLimitor() = 0;
     virtual void Limit(LimitorFunctor<PlayGuiObject>* inLimitor, CardView::SelectorZone inZone) = 0;
@@ -71,11 +78,10 @@ protected:
     PlayGuiObject* fetchMemory(SelectorMemory&);
 
 public:
-    CardSelector(DuelLayers*);
+    CardSelector(GameObserver *observer, DuelLayers*);
     void Add(PlayGuiObject*);
     void Remove(PlayGuiObject*);
     bool CheckUserInput(JButton key);
-    bool CheckUserInput(int x, int y);
     void Update(float dt);
     void Render();
     void Push();
