@@ -11,7 +11,6 @@
 #include "SimpleMenu.h"
 #include "WResourceManager.h"
 #include "CardGui.h"
-#include "GameOptions.h"
 #include "PriceList.h"
 #include "PlayerData.h"
 #include "DeckDataWrapper.h"
@@ -32,7 +31,8 @@ enum
     STAGE_ONSCREEN_MENU = 5,
     STAGE_WELCOME = 6,
     STAGE_MENU = 7,
-    STAGE_FILTERS = 8
+    STAGE_FILTERS = 8,
+    STAGE_TRANSITION_SELECTED = 9
 };
 
 // TODO: need a better name for MENU_FIRST_MENU, this is reused for the 1st submenu of
@@ -75,13 +75,13 @@ enum DECK_VIEWER_MENU_ITEMS
 #define MED_SPEED 5.0f
 #define LOW_SPEED 1.5
 
-#define MAX_SAVED_FILTERS 8
-#define CARDS_DISPLAYED 7
+#define MAX_SAVED_FILTERS Constants::NB_Colors + 1
+#define CARDS_DISPLAYED 10
 
 class GameStateDeckViewer: public GameState, public JGuiListener
 {
 private:
-    JQuadPtr mIcons[CARDS_DISPLAYED];
+    vector<JQuadPtr> mIcons;
     JQuadPtr pspIcons[8];
     JTexture * pspIconsTexture;
     float last_user_activity;
@@ -94,6 +94,7 @@ private:
     JMusic * bgMusic;
     int lastPos;
     int lastTotal;
+    int mSelected;
 
     WGuiFilters * filterMenu;
     WSrcDeckViewer * source;
@@ -142,6 +143,8 @@ public:
     int loadDeck(int deckid);
     void LoadDeckStatistics(int deckId);
 
+    void OnScroll(int inXVelocity, int inYVelocity);
+
     void buildEditorMenu();
     virtual void ButtonPressed(int controllerId, int controlId);
 };
