@@ -1,7 +1,6 @@
 #ifndef _MTGCARD_H_
 #define _MTGCARD_H_
 
-#define MTGCARD_NAME_SIZE 16
 
 #define MTG_IMAGE_WIDTH 200
 #define MTG_IMAGE_HEIGHT 285
@@ -13,17 +12,21 @@
 #include <vector>
 #include <map>
 
+#include "ObjectAnalytics.h"
+
 class CardPrimitive;
 
 using namespace std;
 
 class MTGCard
+#ifdef TRACK_OBJECT_USAGE
+    : public InstanceCounter<MTGCard>
+#endif
 {
 protected:
     friend class MTGSetInfo;
     int mtgid;
     char rarity;
-    char image_name[MTGCARD_NAME_SIZE];
     int init();
 
 public:
@@ -33,6 +36,7 @@ public:
     MTGCard();
     MTGCard(int set_id);
     MTGCard(MTGCard * source);
+    virtual ~MTGCard();
 
     void setMTGId(int id);
     void setRarity(char _rarity);
@@ -42,7 +46,7 @@ public:
     int getMTGId() const;
     int getId() const;
     char getRarity() const;
-    char * getImageName();
+    const string getImageName();
 };
 
 #endif
