@@ -99,7 +99,7 @@ private:
 
 protected:
     int conf_read_mode;
-    int colorsCount[Constants::MTG_NB_COLORS];
+    vector <int> colorsCount;
     int total_cards;
     void init();
     void initCounters();
@@ -119,6 +119,11 @@ public:
     map<string, CardPrimitive *> primitives;
     MTGCard * _(int id);
     MTGCard * getCardById(int id);
+
+#ifdef TESTSUITE
+    void prefetchCardNameCache();
+#endif
+
     MTGCard * getCardByName(string name);
     int load(const char * config_file, const char * setName = NULL, int autoload = 1);
     int countByType(const char * _type);
@@ -127,8 +132,6 @@ public:
     int totalCards();
     int randomCardId();
 
-    static void loadPrimitives(const char *root_directory);
-    static void loadSets(const char *root_directory, const char* filename);
     static void loadInstance();
     static void unloadAll();
     static inline MTGAllCards* getInstance() { return instance; };
@@ -156,6 +159,9 @@ public:
     map<int, int> cards;
     string meta_desc;
     string meta_name;
+    vector<string> meta_AIHints;
+    string meta_unlockRequirements;
+
     int meta_id;
     int totalCards();
     int totalPrice();
