@@ -10,45 +10,107 @@
 #include "CardSelector.h"
 #include "ManaCost.h"
 
-class OtherAbilitiesEventReceiver: public MTGAbility
+class PermanentAbility: public MTGAbility
+{
+public:
+    int testDestroy() {return 0;};
+    PermanentAbility(GameObserver* observer, int _id);
+};
+
+class OtherAbilitiesEventReceiver: public PermanentAbility
 {
 public:
-    int testDestroy();
     int receiveEvent(WEvent * event);
-    OtherAbilitiesEventReceiver(int _id);
+    OtherAbilitiesEventReceiver(GameObserver* observer, int _id);
     OtherAbilitiesEventReceiver * clone() const;
 };
-class MTGPutInPlayRule: public MTGAbility
+
+class MTGEventBonus: public PermanentAbility
+{
+public:
+    int textAlpha;
+    string text;
+    int army[2];
+    bool army1[2];
+    bool army2[2];
+    bool army3[2];
+    int toys[2];
+    bool toybonusgranted[2];
+    int chain[2];
+    int highestChain[2];
+    bool beastbonusgranted[2];
+    int beast[2];
+    bool zombiebonusgranted[2];
+    int zombie[2];
+    bool knightbonusgranted[2];
+    int knight[2];
+    bool insectbonusgranted[2];
+    int insect[2];
+    bool elementalbonusgranted[2];
+    int elemental[2];
+    bool vampirebonusgranted[2];
+    int vampire[2];
+    bool clericbonusgranted[2];
+    int cleric[2];
+    bool elfbonusgranted[2];
+    int elf[2];
+    bool Angelbonusgranted[2];
+    int Angel[2];
+    bool dragonbonusgranted[2];
+    int dragon[2];
+
+    int receiveEvent(WEvent * event);
+    void grantAward(string awardName,int amount);
+    void Update(float dt);
+    void Render();
+    MTGEventBonus(GameObserver* observer, int _id);
+    virtual MTGEventBonus * clone() const;
+};
+
+class MTGPutInPlayRule: public PermanentAbility
 {
 public:
     int isReactingToClick(MTGCardInstance * card, ManaCost * mana = NULL);
     int reactToClick(MTGCardInstance * card);
-    int testDestroy();
     virtual ostream& toString(ostream& out) const;
-    MTGPutInPlayRule(int _id);
+    MTGPutInPlayRule(GameObserver* observer, int _id);
     const char * getMenuText()
     {
-        return "Play Card Normally";
+        return "cast card normally";
     }
     virtual MTGPutInPlayRule * clone() const;
 };
 
-class MTGAlternativeCostRule: public MTGAbility
+class MTGKickerRule: public MTGPutInPlayRule
+{
+public:
+    int isReactingToClick(MTGCardInstance * card, ManaCost * mana = NULL);
+    int reactToClick(MTGCardInstance * card);
+    virtual ostream& toString(ostream& out) const;
+    MTGKickerRule(GameObserver* observer, int _id);
+    const char * getMenuText()
+    {
+        return "pay kicker";
+    }
+    virtual MTGKickerRule * clone() const;
+};
+
+class MTGAlternativeCostRule: public PermanentAbility
 {
 protected:
     int isReactingToClick(MTGCardInstance * card, ManaCost * mana, ManaCost *alternateManaCost);
     int reactToClick(MTGCardInstance * card, ManaCost * alternateManaCost, int paymentType = ManaCost::MANA_PAID);
-
+    string alternativeName;
 public:
     int isReactingToClick(MTGCardInstance * card, ManaCost * mana = NULL);
     int reactToClick(MTGCardInstance * card);
-
-    int testDestroy();
     virtual ostream& toString(ostream& out) const;
-    MTGAlternativeCostRule(int _id);
+    MTGAlternativeCostRule(GameObserver* observer, int _id);
     const char * getMenuText()
     {
-        return "Pay Alternative Cost";
+        if(alternativeName.size())
+            return alternativeName.c_str();
+        return "pay alternative cost";
     }
     virtual MTGAlternativeCostRule * clone() const;
 };
@@ -58,12 +120,11 @@ class MTGBuyBackRule: public MTGAlternativeCostRule
 public:
     int isReactingToClick(MTGCardInstance * card, ManaCost * mana = NULL);
     int reactToClick(MTGCardInstance * card);
-    int testDestroy();
     virtual ostream& toString(ostream& out) const;
-    MTGBuyBackRule(int _id);
+    MTGBuyBackRule(GameObserver* observer, int _id);
     const char * getMenuText()
     {
-        return "Cast And Buy Back";
+        return "cast and buy back";
     }
     virtual MTGBuyBackRule * clone() const;
 };
@@ -74,12 +135,11 @@ public:
 
     int isReactingToClick(MTGCardInstance * card, ManaCost * mana = NULL);
     int reactToClick(MTGCardInstance * card);
-    int testDestroy();
     virtual ostream& toString(ostream& out) const;
-    MTGFlashBackRule(int _id);
+    MTGFlashBackRule(GameObserver* observer, int _id);
     const char * getMenuText()
     {
-        return "Flash Back";
+        return "flash back";
     }
     virtual MTGFlashBackRule * clone() const;
 };
@@ -89,28 +149,26 @@ class MTGRetraceRule: public MTGAlternativeCostRule
 public:
     int isReactingToClick(MTGCardInstance * card, ManaCost * mana = NULL);
     int reactToClick(MTGCardInstance * card);
-    int testDestroy();
     virtual ostream& toString(ostream& out) const;
-    MTGRetraceRule(int _id);
+    MTGRetraceRule(GameObserver* observer, int _id);
     const char * getMenuText()
     {
-        return "Retrace";
+        return "retrace";
     }
     virtual MTGRetraceRule * clone() const;
 };
 
-class MTGMorphCostRule: public MTGAbility
+class MTGMorphCostRule: public PermanentAbility
 {
 public:
  
     int isReactingToClick(MTGCardInstance * card, ManaCost * mana = NULL);
     int reactToClick(MTGCardInstance * card);
-    int testDestroy();
     virtual ostream& toString(ostream& out) const;
-    MTGMorphCostRule(int _id);
+    MTGMorphCostRule(GameObserver* observer, int _id);
     const char * getMenuText()
     {
-        return "Play Morphed";
+        return "play morphed";
     }
     virtual MTGMorphCostRule * clone() const;
 };
@@ -121,13 +179,12 @@ public:
     int isReactingToClick(MTGCardInstance * card, ManaCost * mana = NULL);
     int receiveEvent(WEvent *e);
     int reactToClick(MTGCardInstance * card);
-    int testDestroy();
     string suspendmenu;
     virtual ostream& toString(ostream& out) const;
-    MTGSuspendRule(int _id);
+    MTGSuspendRule(GameObserver* observer, int _id);
     const char * getMenuText()
     {
-        suspendmenu = "Suspend";
+        suspendmenu = "suspend";
         //char buffer[20];
         //sprintf(buffer,"-%i",card->suspendedTime);
         //suspendmenu.append(buffer);
@@ -137,7 +194,7 @@ public:
     virtual MTGSuspendRule * clone() const;
 };
 
-class MTGAttackRule: public MTGAbility, public Limitor
+class MTGAttackRule: public PermanentAbility, public Limitor
 {
 public:
  
@@ -145,9 +202,8 @@ public:
     virtual bool greyout(Target*);
     int isReactingToClick(MTGCardInstance * card, ManaCost * mana = NULL);
     int reactToClick(MTGCardInstance * card);
-    int testDestroy();
     virtual ostream& toString(ostream& out) const;
-    MTGAttackRule(int _id);
+    MTGAttackRule(GameObserver* observer, int _id);
     const char * getMenuText()
     {
         return "Attacker";
@@ -157,24 +213,22 @@ public:
 };
 
 /* handles combat trigger send recieve events*/
-class MTGCombatTriggersRule: public MTGAbility
+class MTGCombatTriggersRule: public PermanentAbility
 {
 public:
-    MTGCombatTriggersRule(int _id);
+    MTGCombatTriggersRule(GameObserver* observer, int _id);
     int receiveEvent(WEvent * event);
     virtual ostream& toString(ostream& out) const;
-    int testDestroy();
     virtual MTGCombatTriggersRule * clone() const;
 };
 
-class MTGBlockRule: public MTGAbility
+class MTGBlockRule: public PermanentAbility
 {
 public:
     int isReactingToClick(MTGCardInstance * card, ManaCost * mana = NULL);
     int reactToClick(MTGCardInstance * card);
-    int testDestroy();
     virtual ostream& toString(ostream& out) const;
-    MTGBlockRule(int _id);
+    MTGBlockRule(GameObserver* observer, int _id);
     const char * getMenuText()
     {
         return "Blocker";
@@ -183,32 +237,39 @@ public:
 };
 
 /* Persist Rule */
-class MTGPersistRule: public MTGAbility
+class MTGPersistRule: public PermanentAbility
 {
 public:
-    MTGPersistRule(int _id);
+    MTGPersistRule(GameObserver* observer, int _id);
     int receiveEvent(WEvent * event);
     virtual ostream& toString(ostream& out) const;
-    int testDestroy();
     virtual MTGPersistRule * clone() const;
 };
-//unearths destruction if leaves play effect
-class MTGUnearthRule: public MTGAbility
+/* vampire Rule */
+class MTGVampireRule: public PermanentAbility
 {
 public:
-    MTGUnearthRule(int _id);
+    MTGVampireRule(GameObserver* observer, int _id);
+    map<MTGCardInstance*,vector<MTGCardInstance*> > victims;
+    int receiveEvent(WEvent * event);
+    virtual ostream& toString(ostream& out) const;
+    virtual MTGVampireRule * clone() const;
+};
+//unearths destruction if leaves play effect
+class MTGUnearthRule: public PermanentAbility
+{
+public:
+    MTGUnearthRule(GameObserver* observer, int _id);
     int receiveEvent(WEvent * event);
     virtual ostream& toString(ostream& out) const;
-    int testDestroy();
     virtual MTGUnearthRule * clone() const;
 };
-class MTGTokensCleanup: public MTGAbility
+class MTGTokensCleanup: public PermanentAbility
 {
 public:
     vector<MTGCardInstance *> list;
-    MTGTokensCleanup(int _id);
+    MTGTokensCleanup(GameObserver* observer, int _id);
     int receiveEvent(WEvent * event);
-    int testDestroy();
     virtual MTGTokensCleanup * clone() const;
 };
 
@@ -221,7 +282,7 @@ public:
 class MTGLegendRule: public ListMaintainerAbility
 {
 public:
-    MTGLegendRule(int _id);
+    MTGLegendRule(GameObserver* observer, int _id);
     int canBeInList(MTGCardInstance * card);
     int added(MTGCardInstance * card);
     int removed(MTGCardInstance * card);
@@ -232,7 +293,7 @@ public:
 class MTGPlaneWalkerRule: public ListMaintainerAbility
 {
 public:
-    MTGPlaneWalkerRule(int _id);
+    MTGPlaneWalkerRule(GameObserver* observer, int _id);
     int canBeInList(MTGCardInstance * card);
     int added(MTGCardInstance * card);
     int removed(MTGCardInstance * card);
@@ -241,12 +302,12 @@ public:
     virtual MTGPlaneWalkerRule * clone() const;
 };
 
-class MTGMomirRule: public MTGAbility
+class MTGMomirRule: public PermanentAbility
 {
 private:
     int genRandomCreatureId(int convertedCost);
-    static vector<int> pool[20];
-    static int initialized;
+    vector<int> pool[20];
+    int initialized;
 
     int textAlpha;
     string text;
@@ -255,10 +316,9 @@ public:
     int alreadyplayed;
     MTGAllCards * collection;
     MTGCardInstance * genCreature(int id);
-    int testDestroy();
     void Update(float dt);
     void Render();
-    MTGMomirRule(int _id, MTGAllCards * _collection);
+    MTGMomirRule(GameObserver* observer, int _id, MTGAllCards * _collection);
     int isReactingToClick(MTGCardInstance * card, ManaCost * mana = NULL);
     int reactToClick(MTGCardInstance * card);
     int reactToClick(MTGCardInstance * card, int id);
@@ -270,30 +330,59 @@ public:
     virtual MTGMomirRule * clone() const;
 };
 
-/* LifeLink */
-class MTGLifelinkRule: public MTGAbility
+//stone hewer gaint avatar mode
+class MTGStoneHewerRule: public PermanentAbility
+{
+private:
+    int genRandomEquipId(int convertedCost);
+    vector<int> pool[20];
+    int initialized;
+public:
+	MTGAllCards * collection;
+	MTGCardInstance * genEquip(int id);
+    MTGStoneHewerRule(GameObserver* observer, int _id, MTGAllCards * _collection);
+	int receiveEvent(WEvent * event);
+	const char * getMenuText()
+	{
+		return "Stone Hewer";
+	}
+	virtual ostream& toString(ostream& out) const;
+	virtual MTGStoneHewerRule * clone() const;
+};
+//Hermit Druid avatar mode
+class MTGHermitRule: public PermanentAbility
 {
 public:
-    MTGLifelinkRule(int _id);
+    MTGHermitRule(GameObserver* observer, int _id);
+	int receiveEvent(WEvent * event);
+	const char * getMenuText()
+	{
+		return "Hermit";
+	}
+	virtual MTGHermitRule * clone() const;
+};
+//
+/* LifeLink */
+class MTGLifelinkRule: public PermanentAbility
+{
+public:
+    MTGLifelinkRule(GameObserver* observer, int _id);
 
     int receiveEvent(WEvent * event);
 
-    int testDestroy();
-
     virtual ostream& toString(ostream& out) const;
 
     virtual MTGLifelinkRule * clone() const;
 };
 
 /* Deathtouch */
-class MTGDeathtouchRule: public MTGAbility
+class MTGDeathtouchRule: public PermanentAbility
 {
 public:
-    MTGDeathtouchRule(int _id);
+    MTGDeathtouchRule(GameObserver* observer, int _id);
 
     int receiveEvent(WEvent * event);
 
-    int testDestroy();
     const char * getMenuText()
     {
         return "Deathtouch";
@@ -302,6 +391,15 @@ public:
     virtual MTGDeathtouchRule * clone() const;
 };
 
+/* handling parentchild */
+class ParentChildRule: public PermanentAbility
+{
+public:
+    ParentChildRule(GameObserver* observer, int _id);
+    int receiveEvent(WEvent * event);
+    virtual ostream& toString(ostream& out) const;
+    virtual ParentChildRule * clone() const;
+};
 /* HUD Display */
 
 class HUDString
@@ -318,7 +416,7 @@ public:
     ;
 };
 
-class HUDDisplay: public MTGAbility
+class HUDDisplay: public PermanentAbility
 {
 private:
     list<HUDString *> events;
@@ -328,11 +426,10 @@ private:
     float maxWidth;
     int addEvent(string s);
 public:
-    int testDestroy();
     int receiveEvent(WEvent * event);
     void Update(float dt);
     void Render();
-    HUDDisplay(int _id);
+    HUDDisplay(GameObserver* observer, int _id);
     ~HUDDisplay();
     virtual HUDDisplay * clone() const;
 };
