@@ -15,39 +15,39 @@ class Player: public Damageable
 {
 protected:
     ManaPool * manaPool;
+    JTexture * mAvatarTex;
+    JQuadPtr mAvatar;
+    bool loadAvatar(string file, string resName = "playerAvatar");
+    bool premade;
 
 public:
-    enum ENUM_PLAY_MODE
+    enum Mode
     {
         MODE_TEST_SUITE,
         MODE_HUMAN,
         MODE_AI
     };
 
-    JTexture * mAvatarTex;
-    JQuadPtr mAvatar;
-    int playMode;
+    int deckId;
+    string mAvatarName;
+    Mode playMode;
     bool nomaxhandsize;
-    bool isPoisoned;
     MTGPlayerCards * game;
+    MTGDeck * mDeck;
     string deckFile;
     string deckFileSmall;
     string deckName;
     string phaseRing;
-
-    Player(string deckFile, string deckFileSmall, MTGDeck * deck = NULL);
+    int offerInterruptOnPhase;
+    Player(GameObserver *observer, string deckFile, string deckFileSmall, MTGDeck * deck = NULL);
     virtual ~Player();
-
+    virtual void setObserver(GameObserver*g);
     virtual void End();
     virtual int displayStack()
     {
         return 1;
     }
     const string getDisplayName() const;
-    int typeAsTarget()
-    {
-        return TARGET_PLAYER;
-    }
 
     int afterDamage();
 
@@ -55,6 +55,7 @@ public:
     int loseLife(int value);
     int gainOrLoseLife(int value);
 
+    bool isPoisoned() {return (poisonCount > 0);}
     int poisoned();
     int damaged();
     int prevented();
@@ -87,23 +88,23 @@ public:
     {
     }
 
-    void loadAvatar(string file);
-
     /**
     ** Returns the path to the stats file of currently selected deck. 
     */
     std::string GetCurrentDeckStatsFile();
+    virtual bool parseLine(const string& s);
+    friend ostream& operator<<(ostream&, const Player&);
 };
 
 class HumanPlayer: public Player
 {
 public:
-    HumanPlayer(string deckFile, string deckFileSmall, MTGDeck * deck = NULL);
-    HumanPlayer(string deckFile);
-
+    HumanPlayer(GameObserver *observer, string deckFile, string deckFileSmall, bool premade = false, MTGDeck * deck = NULL);
+    void End(){
+        if(!premade && opponent())
+            DeckStats::GetInstance()->saveStats(this, opponent(), observer);
+    };
+    friend ostream& operator<<(ostream&, const HumanPlayer&);
 };
 
-ostream& operator<<(ostream&, const Player&);
-istream& operator>>(istream& in, Player& p);
-
 #endif
