@@ -23,10 +23,12 @@ private:
     float timeOpen;
     bool mClosed;
 
+    bool mCenterHorizontal;
+    bool mCenterVertical;
+
     static JQuadPtr spadeR, spadeL, jewel, side;
     static JTexture *spadeRTex, *spadeLTex, *jewelTex, *sideTex;
-    static WFont* titleFont;
-    static hgeParticleSystem* stars;
+    hgeParticleSystem* stars;
 
     inline void MogrifyJewel();
     void drawHorzPole(float x, float y, float width);
@@ -34,13 +36,18 @@ private:
 
 public:
     bool autoTranslate;
-    SimpleMenu(int id, JGuiListener* listener, int fontId, float x, float y, const char * _title = "", int _maxItems = 7);
+    bool isMultipleChoice;
+    SimpleMenu(JGE*, int id, JGuiListener* listener, int fontId, float x, float y, const char * _title = "", int _maxItems = 7, bool centerHorizontal = true, bool centerVertical = true);
     virtual ~SimpleMenu();
     void Render();
     void Update(float dt);
     void Add(int id, const char * Text, string desc = "", bool forceFocus = false);
+    int getmCurr(){return mCurr;}
+    float getWidth(){return mWidth; }
     void Close();
 
+    void RecenterMenu();
+
     float selectionTargetY;
     bool isClosed()
     {
