@@ -1,8 +1,6 @@
 #ifndef _TARGETSLIST_H_
 #define _TARGETSLIST_H_
 
-#define MAX_TARGETS 20
-
 class Targetable;
 class MTGCardInstance;
 class Player;
@@ -11,16 +9,21 @@ class Spell;
 class Interruptible;
 class Damage;
 
+#include <vector>
+using std::vector;
+
 class TargetsList
 {
+protected:
+    size_t iterateTarget(Targetable * previous);
+    vector<Targetable*> targets;
 public:
-    int cursor;
     TargetsList();
     TargetsList(Targetable * _targets[], int nbtargets);
-    Targetable* targets[MAX_TARGETS];
     int alreadyHasTarget(Targetable * target);
     int removeTarget(Targetable * _card);
     int toggleTarget(Targetable * _card);
+    size_t getNbTargets() {return targets.size();};
     virtual int addTarget(Targetable * _target);
     MTGCardInstance * getNextCardTarget(MTGCardInstance * previous = 0);
     Player * getNextPlayerTarget(Player * previous = 0);
@@ -28,10 +31,10 @@ public:
     Interruptible * getNextInterruptible(Interruptible * previous, int type);
     Spell * getNextSpellTarget(Spell * previous = 0);
     Damage * getNextDamageTarget(Damage * previous = 0);
-    Targetable * getNextTarget(Targetable * previous = 0, int type = -1);
+    Targetable * getNextTarget(Targetable * previous = 0);
     void initTargets()
     {
-        cursor = 0;
+        targets.clear();
     }
     ;
 };
