@@ -1,5 +1,5 @@
 /*
-  Filter-like system for determining if a card meats certain criteria, for this and thisforeach autos
+  Filter-like system for determining if a card meets certain criteria, for this and thisforeach autos
 */
 
 #ifndef _THISDESCRIPTOR_H_
@@ -17,11 +17,21 @@ class ThisDescriptor{
    virtual int match(MTGCardInstance * card) = 0;
    int matchValue(int value);  
    virtual ~ThisDescriptor();
+   virtual ThisDescriptor * clone() const = 0;
 };
 
 class ThisDescriptorFactory{
 public:
-  ThisDescriptor * createThisDescriptor(string s);
+  ThisDescriptor * createThisDescriptor(GameObserver* observer, string s);
+};
+
+class ThisTargetCompare:public ThisDescriptor{
+ public:
+  TargetChooser * targetComp;
+  virtual int match(MTGCardInstance * card);
+  ThisTargetCompare(TargetChooser * tcc = NULL);
+  ~ThisTargetCompare();
+  ThisTargetCompare * clone() const;
 };
 
 class ThisCounter:public ThisDescriptor{
@@ -32,6 +42,7 @@ class ThisCounter:public ThisDescriptor{
   ThisCounter(Counter * _counter);
   ThisCounter(int power, int toughness, int nb, const char * name);
   ~ThisCounter();
+  ThisCounter * clone() const;
 };
 
 class ThisCounterAny:public ThisDescriptor{
@@ -39,6 +50,7 @@ class ThisCounterAny:public ThisDescriptor{
   virtual int match(MTGCardInstance *card);
 
   ThisCounterAny(int nb);
+  ThisCounterAny * clone() const;
 };
 
 class ThisControllerlife:public ThisDescriptor{
@@ -46,6 +58,7 @@ class ThisControllerlife:public ThisDescriptor{
     virtual int match(MTGCardInstance * card);
   
     ThisControllerlife(int life);
+    ThisControllerlife * clone() const;
 };
 
 class ThisOpponentlife:public ThisDescriptor{
@@ -53,6 +66,7 @@ class ThisOpponentlife:public ThisDescriptor{
     virtual int match(MTGCardInstance * card);
   
     ThisOpponentlife(int olife);
+    ThisOpponentlife * clone() const;
 };
 
 class ThisEquip:public ThisDescriptor{
@@ -60,6 +74,7 @@ class ThisEquip:public ThisDescriptor{
     virtual int match(MTGCardInstance * card);
   
     ThisEquip(int equipment);
+    ThisEquip * clone() const;
 };
 
 class ThisAuras:public ThisDescriptor{
@@ -67,6 +82,7 @@ class ThisAuras:public ThisDescriptor{
     virtual int match(MTGCardInstance * card);
   
     ThisAuras(int auras);
+    ThisAuras * clone() const;
 };
 
 class ThisOpponentDamageAmount:public ThisDescriptor{
@@ -74,6 +90,7 @@ class ThisOpponentDamageAmount:public ThisDescriptor{
     virtual int match(MTGCardInstance * card);
   
     ThisOpponentDamageAmount(int damagecount);
+    ThisOpponentDamageAmount * clone() const;
 };
 
 class ThisUntapped:public ThisDescriptor{
@@ -81,6 +98,7 @@ class ThisUntapped:public ThisDescriptor{
     virtual int match(MTGCardInstance * card);
   
     ThisUntapped(int untapped);
+    ThisUntapped * clone() const;
 };
 
 class ThisTapped:public ThisDescriptor{
@@ -88,6 +106,7 @@ class ThisTapped:public ThisDescriptor{
     virtual int match(MTGCardInstance * card);
   
     ThisTapped(int tapped);
+    ThisTapped * clone() const;
 };
 
 
@@ -96,6 +115,15 @@ class ThisAttacked:public ThisDescriptor{
     virtual int match(MTGCardInstance * card);
 
 	ThisAttacked(int attack);
+    ThisAttacked * clone() const;
+};
+
+class ThisBlocked:public ThisDescriptor{
+ public:
+    virtual int match(MTGCardInstance * card);
+
+	ThisBlocked(int block);
+    ThisBlocked * clone() const;
 };
 
 class ThisNotBlocked:public ThisDescriptor{
@@ -103,6 +131,7 @@ class ThisNotBlocked:public ThisDescriptor{
     virtual int match(MTGCardInstance * card);
 
 	ThisNotBlocked(int unblocked);
+    ThisNotBlocked * clone() const;
 };
 
 class ThisDamaged:public ThisDescriptor{
@@ -110,6 +139,15 @@ class ThisDamaged:public ThisDescriptor{
     virtual int match(MTGCardInstance * card);
 
 	ThisDamaged(int wasDealtDamage);
+    ThisDamaged * clone() const;
+};
+
+class ThisDualWield:public ThisDescriptor{
+ public:
+    virtual int match(MTGCardInstance * card);
+
+	ThisDualWield(int dualWield);
+    ThisDualWield * clone() const;
 };
 
 class ThisPower:public ThisDescriptor{
@@ -117,6 +155,7 @@ class ThisPower:public ThisDescriptor{
     virtual int match(MTGCardInstance * card);
   
     ThisPower(int power);
+    ThisPower * clone() const;
 };
 
 class ThisToughness:public ThisDescriptor{
@@ -124,12 +163,14 @@ class ThisToughness:public ThisDescriptor{
     virtual int match(MTGCardInstance * card);
   
     ThisToughness(int toughness);
+    ThisToughness * clone() const;
 };
 
 class ThisX:public ThisDescriptor{
   public:
     virtual int match(MTGCardInstance * card);
     ThisX(int x);
+    ThisX * clone() const;
 };
 
 #endif
