@@ -2,20 +2,137 @@
 
 #include "Credits.h"
 #include "GameApp.h"
-#include "GameOptions.h"
 #include "PlayerData.h"
 #include "DeckStats.h"
 #include "Translate.h"
 #include "MTGDeck.h"
 #include "GameObserver.h"
 #include "GameStateShop.h"
-#include "PlayerData.h"
+#include "AIPlayer.h"
+
+
+map <string, Unlockable *> Unlockable::unlockables;
+
+Unlockable::Unlockable() 
+{
+}
+
+void  Unlockable::setValue(string k , string v) 
+{
+    mValues[k] = v;
+}
+
+string Unlockable::getValue(string k) 
+{
+    return mValues[k];
+}
+
+bool Unlockable::isUnlocked() {
+    string id = getValue("id");
+    assert(id.size() > 0);
+    return (options[id].number != 0);
+}
+
+bool Unlockable::tryToUnlock(GameObserver * game) {
+    if (isUnlocked())
+        return false;
+
+    string conditions = getValue("unlock_condition");
+    
+    Player * p = game->players[0];
+    if (p->isAI())
+        return false;
+
+    // We need a card belonging to the player in order to call parceCastRestrictions
+    // The goal is usually to create objects such as targetChoosers, that are usually required
+    // for protection, or determining the card's owner
+    //Therefore here any card is ok
+    MTGCardInstance * dummyCard = p->game->battlefield->nb_cards 
+        ? p->game->battlefield->cards[0]
+        :  p->game->hand->nb_cards 
+            ?  p->game->hand->cards[0]
+            : p->game->library->nb_cards
+                ?  p->game->library->cards[0]
+                : NULL;
+
+    AbilityFactory af(game);
+    int meetConditions = conditions.size() 
+        ? dummyCard 
+            ? af.parseCastRestrictions(dummyCard, p, conditions) 
+            : 0
+        : 1;
+
+    if (!meetConditions)
+        return false;
+
+    // Unlock the award and return
+    string id = getValue("id");
+    assert(id.size() > 0);
+
+	GameOptionAward* goa = (GameOptionAward*) &options[id]; 
+    goa->giveAward();
+    return true;
+}
+
+void Unlockable::load()
+{
+    std::string contents;
+    if (! JFileSystem::GetInstance()->readIntoString("rules/awards.dat", contents))
+        return;
+
+    std::stringstream stream(contents);
+    std::string s;
+
+    Unlockable * current = NULL;
+    while (std::getline(stream, s))
+    {
+        if (!s.size()) continue;
+        if (s[s.size() - 1] == '\r') s.erase(s.size() - 1); //Handle DOS files
+        if (!s.size()) continue;
+        if (s[0] == '#') continue;
+
+        if (s == "[award]")
+        {
+            current = NEW Unlockable();
+            continue;
+        }
+
+
+        if (!current)
+            continue;
+
+        if (s == "[/award]")
+        {
+            string id = current->getValue("id");
+            
+            if (id.size())
+                unlockables[id] = current;
+            else 
+                SAFE_DELETE(current);
+
+            continue;
+        }
+
+        vector<string> keyValue = split(s,'=');
+        if (keyValue.size() != 2)
+            continue;
+
+        current->setValue(keyValue[0], keyValue[1]);
+    }
+}
+
+void Unlockable::Destroy()
+{
+    for (map<string, Unlockable *>::iterator it = unlockables.begin(); it != unlockables.end(); ++it) {
+        SAFE_DELETE(it->second);
+    }
+    unlockables.clear();
+}
 
 CreditBonus::CreditBonus(int _value, string _text)
 {
     value = _value;
     text = _text;
-
 }
 
 void CreditBonus::Render(float x, float y, WFont * font)
@@ -30,6 +147,7 @@ Credits::Credits()
     unlocked = -1;
     p1 = NULL;
     p2 = NULL;
+    observer = NULL;
 }
 
 Credits::~Credits()
@@ -40,17 +158,22 @@ Credits::~Credits()
     bonus.clear();
 }
 
-void Credits::compute(Player * _p1, Player * _p2, GameApp * _app)
+void Credits::compute(GameObserver* g, GameApp * _app)
 {
-    p1 = _p1;
-    p2 = _p2;
-    app = _app;
-    showMsg = (WRand() % 3);
-    GameObserver * g = GameObserver::GetInstance();
     if (!g->turn)
         return;
+    p1 = g->players[0];
+    p2 = g->players[1];
+    observer = g;
+    app = _app;
+    showMsg = (WRand() % 3);
+
+    //no credits when the AI plays :)
+    if (p1->isAI())
+        return;
+
     PlayerData * playerdata = NEW PlayerData(MTGCollection());
-    if (!p1->isAI() && p2->isAI() && p1 != g->gameOver)
+    if (p2->isAI() && p1 != g->gameOver)
     {
         gameLength = time(0) - g->startedAt;
         value = 400;
@@ -101,7 +224,7 @@ void Credits::compute(Player * _p1, Player * _p2, GameApp * _app)
         GameOptionAward * goa = NULL;
         // <Tasks handling>
         vector<Task*> finishedTasks;
-        playerdata->taskList->getDoneTasks(_p1, _p2, _app, &finishedTasks);
+        playerdata->taskList->getDoneTasks(g, _app, &finishedTasks);
 
         char buffer[512];
 
@@ -124,41 +247,48 @@ void Credits::compute(Player * _p1, Player * _p2, GameApp * _app)
                 unlockedTextureName = "unlocked.png";
                 goa = (GameOptionAward*) &options[Options::DIFFICULTY_MODE_UNLOCKED];
                 goa->giveAward();
-                options.save();
             }
-            else if ((unlocked = isMomirUnlocked()))
+            else
             {
-                unlockedTextureName = "momir_unlocked.png";
-                goa = (GameOptionAward*) &options[Options::MOMIR_MODE_UNLOCKED];
-                goa->giveAward();
-                options.save();
+                for (map<string, Unlockable *>::iterator it = Unlockable::unlockables.begin(); it !=  Unlockable::unlockables.end(); ++it) {
+                    Unlockable * award = it->second;
+                    if (award->tryToUnlock(g))
+                    {
+                        unlocked = 1;
+                        unlockedString = award->getValue("unlock_text");
+                        unlockedTextureName = award->getValue("unlock_img");
+                        break;
+                    }
+                }
             }
-            else if ((unlocked = isEvilTwinUnlocked()))
+            
+            if (!unlocked)
             {
-                unlockedTextureName = "eviltwin_unlocked.png";
-                goa = (GameOptionAward*) &options[Options::EVILTWIN_MODE_UNLOCKED];
-                goa->giveAward();
-                options.save();
-            }
-            else if ((unlocked = isRandomDeckUnlocked()))
-            {
-                unlockedTextureName = "randomdeck_unlocked.png";
-                goa = (GameOptionAward*) &options[Options::RANDOMDECK_MODE_UNLOCKED];
-                goa->giveAward();
-                options.save();
-            }
-            else if ((unlocked = unlockRandomSet()))
-            {
-                unlockedTextureName = "set_unlocked.png";
-                MTGSetInfo * si = setlist.getInfo(unlocked - 1);
-                if (si)
-                    unlockedString = si->getName(); //Show the set's pretty name for unlocks.
-            }
-            else if ((unlocked = IsMoreAIDecksUnlocked(stats)))
-            {
-                options[Options::AIDECKS_UNLOCKED].number += 10;
-                options.save();
-                unlockedTextureName = "ai_unlocked.png";
+                if ((unlocked = isEvilTwinUnlocked()))
+                {
+                    unlockedTextureName = "eviltwin_unlocked.png";
+                    goa = (GameOptionAward*) &options[Options::EVILTWIN_MODE_UNLOCKED];
+                    goa->giveAward();
+                }
+                else if ((unlocked = isRandomDeckUnlocked()))
+                {
+                    unlockedTextureName = "randomdeck_unlocked.png";
+                    goa = (GameOptionAward*) &options[Options::RANDOMDECK_MODE_UNLOCKED];
+                    goa->giveAward();
+                }
+                else if ((unlocked = unlockRandomSet()))
+                {
+                    unlockedTextureName = "set_unlocked.png";
+                    MTGSetInfo * si = setlist.getInfo(unlocked - 1);
+                    if (si)
+                        unlockedString = si->getName(); //Show the set's pretty name for unlocks.
+                }
+                else if ((unlocked = IsMoreAIDecksUnlocked(stats)))
+                {
+                    options[Options::AIDECKS_UNLOCKED].number += 10;
+                    options.save();
+                    unlockedTextureName = "ai_unlocked.png";
+                }
             }
 
             if (unlocked && options[Options::SFXVOLUME].number > 0)
@@ -208,7 +338,12 @@ JQuadPtr Credits::GetUnlockedQuad(string textureName)
     JTexture * unlockedTex = WResourceManager::Instance()->RetrieveTexture(textureName);
     if (!unlockedTex) return JQuadPtr();
 
-    return WResourceManager::Instance()->RetrieveQuad(unlockedTextureName, 2, 2, unlockedTex->mWidth - 4, unlockedTex->mHeight - 4);
+    return WResourceManager::Instance()->RetrieveQuad(
+        unlockedTextureName,
+        2.0f,
+        2.0f, 
+        static_cast<float>(unlockedTex->mWidth - 4),
+        static_cast<float>(unlockedTex->mHeight - 4));
     
 }
 
@@ -216,7 +351,6 @@ void Credits::Render()
 {
     if (!p1)
         return;
-    GameObserver * g = GameObserver::GetInstance();
     JRenderer * r = JRenderer::GetInstance();
     WFont * f = WResourceManager::Instance()->GetWFont(Fonts::MAIN_FONT);
     WFont * f2 = WResourceManager::Instance()->GetWFont(Fonts::MENU_FONT);
@@ -228,7 +362,8 @@ void Credits::Render()
     f3->SetScale(1);
     f3->SetColor(ARGB(255,255,255,255));
     char buffer[512];
-    if (!g->turn)
+
+    if (!observer->turn)
     {
         sprintf(buffer, "%s", _("Please check your deck (not enough cards?)").c_str());
     }
@@ -236,7 +371,7 @@ void Credits::Render()
     {
         if (!p1->isAI() && p2->isAI())
         {
-            if (g->gameOver != p1)
+            if (observer->gameOver != p1)
             {
                 sprintf(buffer, _("Congratulations! You earn %i credits").c_str(), value);
                 JQuadPtr unlockedQuad = GetUnlockedQuad(unlockedTextureName);
@@ -258,7 +393,7 @@ void Credits::Render()
         else
         {
             int winner = 2;
-            if (g->gameOver != p1)
+            if (observer->gameOver != p1)
             {
                 winner = 1;
             }
@@ -281,9 +416,9 @@ void Credits::Render()
     y += 15;
 
     //!!
-    if (g->gameOver != p1)
+    if (observer->gameOver != p1 && this->gameLength != 0)
     {
-        sprintf(buffer, _("Game length: %i turns (%i seconds)").c_str(), g->turn, this->gameLength);
+        sprintf(buffer, _("Game length: %i turns (%i seconds)").c_str(), observer->turn, this->gameLength);
         f->DrawString(buffer, 10, y);
         y += 10;
         sprintf(buffer, _("Credits per minute: %i").c_str(), (int) (60 * value / this->gameLength));
@@ -305,40 +440,23 @@ int Credits::isDifficultyUnlocked(DeckStats * stats)
 {
     if (options[Options::DIFFICULTY_MODE_UNLOCKED].number)
         return 0;
-    int nbAIDecks = 0;
-    int found = 1;
+    int nbAIDecks = AIPlayer::getTotalAIDecks();
+
     int wins = 0;
 
-    while (found)
+    for (int i = 0; i < nbAIDecks; ++i) 
     {
-        found = 0;
-        char buffer[512];
         char aiSmallDeckName[512];
-        sprintf(buffer, JGE_GET_RES("ai/baka/deck%i.txt").c_str(), nbAIDecks + 1);
-        if (fileExists(buffer))
-        {
-            found = 1;
-            nbAIDecks++;
-            sprintf(aiSmallDeckName, "ai_baka_deck%i", nbAIDecks);
-            int percentVictories = stats->percentVictories(string(aiSmallDeckName));
-            if (percentVictories >= 67)
-                wins++;
-            if (wins >= 10)
-                return 1;
-        }
+        sprintf(aiSmallDeckName, "ai_baka_deck%i", i+1);
+        int percentVictories = stats->percentVictories(string(aiSmallDeckName));
+        if (percentVictories >= 67)
+            wins++;
+        if (wins >= 10)
+            return 1;
     }
     return 0;
 }
 
-int Credits::isMomirUnlocked()
-{
-    if (options[Options::MOMIR_MODE_UNLOCKED].number)
-        return 0;
-    if (p1->game->inPlay->countByType("land") == 8)
-        return 1;
-    return 0;
-}
-
 int Credits::isEvilTwinUnlocked()
 {
     if (options[Options::EVILTWIN_MODE_UNLOCKED].number)
@@ -440,22 +558,8 @@ int Credits::IsMoreAIDecksUnlocked(DeckStats * stats) {
     // the number of currently unlocked decks in order to go through.
     if (stats->nbGames() < currentlyUnlocked * 1.2) return 0;
 
-    int nbdecks = 0;
-    int found = 1;
-    while (found)
-    {
-        found = 0;
-        char buffer[512];
-        sprintf(buffer, JGE_GET_RES("ai/baka/deck%i.txt").c_str(), nbdecks + 1);
-        wagic::ifstream file(buffer);
-        if (file)
-        {
-            found = 1;
-            file.close();
-            nbdecks++;
-            if (nbdecks > currentlyUnlocked)
-                return 1;
-        }
-    }
+    if (AIPlayer::getTotalAIDecks() > currentlyUnlocked)
+        return 1;
+
     return 0;
 }
