@@ -10,9 +10,9 @@
 //Merge this with DeckStats
 //Have this class handle all the Meta Data rather than relying on MTGDeck. Then MTGDeck would have a MetaData object...
 
-DeckMetaData::DeckMetaData(const string& filename)
+DeckMetaData::DeckMetaData(const string& filename, bool isAI)
     : mFilename(filename), mGamesPlayed(0), mVictories(0), mPercentVictories(0), mDifficulty(0),
-      mDeckLoaded(false), mStatsLoaded(false)
+      mDeckLoaded(false), mStatsLoaded(false), mIsAI(isAI)
 {
     // TODO, figure out how we can defer this to later - currently, 
     // there's a catch 22, as we sort the deck list alphabetically, so we need to open the deck file
@@ -21,6 +21,37 @@ DeckMetaData::DeckMetaData(const string& filename)
     LoadDeck();
 }
 
+
+void DeckMetaData::LoadDeck()
+{
+    if (!mDeckLoaded)
+    {
+        MTGDeck deck(mFilename.c_str(), NULL, 1);
+        mName = trim(deck.meta_name);
+        mDescription = trim(deck.meta_desc);
+        mDeckId = atoi((mFilename.substr(mFilename.find("deck") + 4, mFilename.find(".txt"))).c_str());
+
+        vector<string> requirements = split(deck.meta_unlockRequirements, ',');
+        for(size_t i = 0; i < requirements.size(); ++i)
+        {
+            mUnlockRequirements.push_back(Options::getID(requirements[i]));
+        }
+
+        mDeckLoaded = true;
+        if (!mIsAI)
+            mAvatarFilename = "avatar.jpg";
+        else
+        {
+            ostringstream avatarFilename;
+            avatarFilename << "avatar" << getAvatarId() << ".jpg";
+            mAvatarFilename = avatarFilename.str();
+        }        
+    }
+
+
+}
+
+
 void DeckMetaData::LoadStats()
 {
     if (!mStatsLoaded)
@@ -28,20 +59,21 @@ void DeckMetaData::LoadStats()
         DeckStats * stats = DeckStats::GetInstance();
         if (mIsAI)
         {
+            mPercentVictories = 0;
+            mVictories = 0;
+            mGamesPlayed = 0;
+            mColorIndex = "";
+            mDifficulty = 0;
+
             stats->load(mPlayerDeck);
             DeckStat * opponentDeckStats = stats->getDeckStat(mStatsFilename);
             if (opponentDeckStats)
             {
-                mPercentVictories = stats->percentVictories(mStatsFilename);
+                mPercentVictories = opponentDeckStats->percentVictories();
                 mVictories = opponentDeckStats->victories;
                 mGamesPlayed = opponentDeckStats->nbgames;
                 mColorIndex = opponentDeckStats->manaColorIndex;
-                ostringstream oss;
-                int deckFilenameOffset = mStatsFilename.find("deck") + 4;
-                int oppDeckId = atoi(mStatsFilename.substr(deckFilenameOffset, mStatsFilename.find_last_of(".")).c_str());
-                int avatarId = getAvatarId(oppDeckId);
-                oss << "avatar" << avatarId << ".jpg";
-                mAvatarFilename = oss.str();
+ 
                 if (mPercentVictories < 34)
                 {
                     mDifficulty = HARD;
@@ -54,12 +86,7 @@ void DeckMetaData::LoadStats()
                 {
                     mDifficulty = EASY;
                 }
-            }
-            else
-            {
-                ostringstream oss;
-                oss << "avatar" << getAvatarId(mDeckId) << ".jpg";
-                mAvatarFilename = oss.str();
+                mStatsLoaded = true;
             }
         }
         else
@@ -70,36 +97,18 @@ void DeckMetaData::LoadStats()
                 mGamesPlayed = stats->nbGames();               
                 mPercentVictories = stats->percentVictories();
                 mVictories = static_cast<int>(mGamesPlayed * (mPercentVictories / 100.0f));
+                mStatsLoaded = true;
             }
         }
 
-        mStatsLoaded = true;
     }
 
 }
 
 // since we only have 100 stock avatar images, we need to recycle the images for deck numbers > 99
-int DeckMetaData::getAvatarId(int deckId)
+int DeckMetaData::getAvatarId()
 {
-    int avatarId = deckId % 100;
-    if (deckId >= 100 && avatarId == 0)
-        return 100;
-
-    return avatarId;
-}
-
-void DeckMetaData::LoadDeck()
-{
-    if (!mDeckLoaded)
-    {
-        MTGDeck deck(mFilename.c_str(), NULL, 1);
-        mName = trim(deck.meta_name);
-        mDescription = trim(deck.meta_desc);
-        mDeckId = atoi((mFilename.substr(mFilename.find("deck") + 4, mFilename.find(".txt"))).c_str());
-        mDeckLoaded = true;
-    }
-
-
+    return mDeckId % 100;
 }
 
 //Accessors
@@ -119,6 +128,12 @@ int DeckMetaData::getDeckId()
     return mDeckId;
 }
 
+vector<int> DeckMetaData::getUnlockRequirements()
+{
+    return mUnlockRequirements;
+}
+
+
 string DeckMetaData::getAvatarFilename()
 {
     return mAvatarFilename;
