@@ -114,39 +114,45 @@ void DeckStats::load(const std::string& filename)
     {
         return;
     }
-    wagic::ifstream file(filename.c_str());
-    std::string s;
 
-    if (file)
+    std::string contents;
+    if (JFileSystem::GetInstance()->readIntoString(filename, contents))
     {
+        std::stringstream stream(contents);
+        std::string s;
         // get the associated player deck file:
         int deckId = atoi(filename.substr(filename.find("_deck") + 5, filename.find(".txt")).c_str());
         char buffer[512];
         sprintf(buffer, "deck%i.txt", deckId);
-        string playerDeckFilePath= options.profileFile( buffer);
+        string playerDeckFilePath = options.profileFile( buffer);
         DeckMetaData *playerDeckMetaData = DeckManager::GetInstance()->getDeckMetaDataByFilename( playerDeckFilePath, false);
+        if (!playerDeckMetaData)
+        {
+            DebugTrace("DeckStats.cpp:CONSISTENCY ERROR: DeckStats are set, but no deck meta data");
+            return;
+        }
         // check if this player deck has already been profiled for manacolors
-        char next = file.peek();
+        char next = stream.peek();
         string manaColorIndex = "";
         if ( next == 'M')
         {
-            std::getline(file, s );
+            std::getline(stream, s );
             manaColorIndex = s.substr( s.find(":") + 1);
             playerDeckMetaData->setColorIndex( manaColorIndex );
         }
 
-        while (std::getline(file, s))
+        while (std::getline(stream, s))
         {
             string deckfile = s;
-            std::getline(file, s);
+            std::getline(stream, s);
             int games = atoi(s.c_str());
-            std::getline(file, s);
+            std::getline(stream, s);
             int victories = atoi(s.c_str());
-            next = file.peek();
+            next = stream.peek();
             
             if ( next == 'M')
             {
-                std::getline(file, s );
+                std::getline(stream, s );
                 manaColorIndex = s.substr( s.find(":") + 1);
             }
             if ( masterDeckStats[filename].find(deckfile) != masterDeckStats[filename].end())
@@ -156,16 +162,15 @@ void DeckStats::load(const std::string& filename)
             DeckStat * newDeckStat = NEW DeckStat(games, victories, manaColorIndex);
             (masterDeckStats[filename])[deckfile] = newDeckStat;
         }
-        file.close();
     }
 }
 
 void DeckStats::save(const std::string& filename)
 {
-    std::ofstream file(filename.c_str());
-    char writer[512];
-    if (file)
+    std::ofstream file;
+    if (JFileSystem::GetInstance()->openForWrite(file, filename))
     {
+        char writer[512];
         map<string, DeckStat *> stats = masterDeckStats[currentDeck];
         map<string, DeckStat *>::iterator it;
         string manaColorIndex = "";
@@ -175,7 +180,7 @@ void DeckStats::save(const std::string& filename)
         string playerDeckFilePath= options.profileFile( buffer);
         DeckManager *deckManager = DeckManager::GetInstance();
         DeckMetaData *playerDeckMeta = deckManager->getDeckMetaDataByFilename(playerDeckFilePath, false);
-        if ( playerDeckMeta->getColorIndex() == "" )
+        if (playerDeckMeta && playerDeckMeta->getColorIndex() == "" )
         {
             StatsWrapper *stw = deckManager->getExtendedDeckStats( playerDeckMeta, MTGAllCards::getInstance(), false);
             manaColorIndex = stw->getManaColorIndex();
@@ -193,6 +198,8 @@ void DeckStats::save(const std::string& filename)
             file << "MANA:" << it->second->manaColorIndex <<endl;
         }
         file.close();
+        if(playerDeckMeta)
+            playerDeckMeta->Invalidate();
     }
 }
 
@@ -314,19 +321,16 @@ void StatsWrapper::initStatistics(string deckstats)
         gamesPlayed = stats->nbGames();
 
         // Detailed deck statistics against AI
-        int found = 1;
         int nbDecks = 0;
-        found = 0;
         char buffer[512];
         char smallDeckName[512];
         ostringstream oss;
         oss << "deck" << (nbDecks + 1);
-        string bakaDir = JGE_GET_RES("/ai/baka");
+        string bakaDir = "ai/baka/";
         string deckFilename = oss.str();
         sprintf(buffer, "%s/%s.txt", bakaDir.c_str(), deckFilename.c_str());
         if (fileExists(buffer))
         {
-            found = 1;
             nbDecks++;
 
             sprintf(smallDeckName, "%s_deck%i", "ai_baka", nbDecks);
@@ -388,7 +392,7 @@ void StatsWrapper::updateStats(DeckDataWrapper *myDeck)
     this->totalManaCost = 0;
     this->totalCreatureCost = 0;
     this->totalSpellCost = 0;
-    MTGCard * current = myDeck->getCard();
+    MTGCard * current = NULL;
 
     // Clearing arrays
     for (int i = 0; i <= Constants::STATS_MAX_MANA_COST; i++)
@@ -398,7 +402,7 @@ void StatsWrapper::updateStats(DeckDataWrapper *myDeck)
         this->countSpellsPerCost[i] = 0;
     }
 
-    for (int i = 0; i <= Constants::MTG_NB_COLORS; i++)
+    for (int i = 0; i <= Constants::NB_Colors; i++)
     {
         this->totalCostPerColor[i] = 0;
         this->countLandsPerColor[i] = 0;
@@ -408,7 +412,7 @@ void StatsWrapper::updateStats(DeckDataWrapper *myDeck)
 
     for (int i = 0; i <= Constants::STATS_MAX_MANA_COST; i++)
     {
-        for (int k = 0; k <= Constants::MTG_NB_COLORS; k++)
+        for (int k = 0; k <= Constants::NB_Colors; k++)
         {
             this->countCardsPerCostAndColor[i][k] = 0;
             this->countCreaturesPerCostAndColor[i][k] = 0;
@@ -443,6 +447,28 @@ void StatsWrapper::updateStats(DeckDataWrapper *myDeck)
 
         // Lets look for mana producing abilities
 
+        //http://code.google.com/p/wagic/issues/detail?id=650
+        //Basic lands are not producing their mana through regular abilities anymore,
+        //but through a rule that is outside of the primitives. This block is a hack to address this
+        const int colors[] = {Constants::MTG_COLOR_GREEN, Constants::MTG_COLOR_BLUE, Constants::MTG_COLOR_RED, Constants::MTG_COLOR_BLACK, Constants::MTG_COLOR_WHITE};
+        const string lands[] = { "forest", "island", "mountain", "swamp", "plains" };
+        for (unsigned int i = 0; i < sizeof(colors)/sizeof(colors[0]); ++i)
+        {
+            int colorId = colors[i];
+            string type = lands[i];
+            if (current->data->hasType(type.c_str()))
+            {
+                if (current->data->hasType("Basic"))
+                {
+                    this->countBasicLandsPerColor[colorId] += currentCount;
+                }
+                else
+                {
+                    this->countLandsPerColor[colorId] += currentCount;
+                }
+            }
+        }
+
         vector<string> abilitiesVector;
         string thisstring = current->data->magicText;
         abilitiesVector = split(thisstring, '\n');
@@ -455,7 +481,7 @@ void StatsWrapper::updateStats(DeckDataWrapper *myDeck)
             {
                 s = s.substr(t + 3);
                 ManaCost * mc = ManaCost::parseManaCost(s);
-                for (int j = 0; j < Constants::MTG_NB_COLORS; j++)
+                for (int j = 0; j < Constants::NB_Colors; j++)
                 {
                     if (mc->hasColor(j))
                     {
@@ -482,7 +508,7 @@ void StatsWrapper::updateStats(DeckDataWrapper *myDeck)
 
         // Add to the per color counters
         //  a. regular costs
-        for (int j = 0; j < Constants::MTG_NB_COLORS; j++)
+        for (int j = 0; j < Constants::NB_Colors; j++)
         {
             this->totalCostPerColor[j] += currentCost->getCost(j) * currentCount;
             if (current->data->hasColor(j))
@@ -513,7 +539,7 @@ void StatsWrapper::updateStats(DeckDataWrapper *myDeck)
     }
 
     this->totalColoredSymbols = 0;
-    for (int j = 1; j < Constants::MTG_NB_COLORS; j++)
+    for (int j = 1; j < Constants::NB_Colors; j++)
     {
         this->totalColoredSymbols += this->totalCostPerColor[j];
     }
