@@ -7,8 +7,10 @@
 #include "Player.h"
 #include "Counters.h"
 
-ExtraCost::ExtraCost(const std::string& inCostRenderString, TargetChooser *_tc) :
-    tc(_tc), source(NULL), target(NULL), mCostRenderString(inCostRenderString)
+SUPPORT_OBJECT_ANALYTICS(ExtraCost)
+
+ExtraCost::ExtraCost(const std::string& inCostRenderString, TargetChooser *_tc)
+    : tc(_tc), source(NULL), target(NULL), mCostRenderString(inCostRenderString)
 {
     if (tc)
         tc->targetter = NULL;
@@ -25,8 +27,9 @@ int ExtraCost::setSource(MTGCardInstance * _source)
     if (tc)
     {
         tc->source = _source;
-        // "extra cost is not targetting, protections do not apply" this is not cryptic at all :) make an ability you will understand it then. this keeps the target chooser from being unable to select a creature with shroud/protections.
+        // this keeps the target chooser from being unable to select a creature with shroud/protections.
         tc->targetter = NULL;
+        tc->Owner = source->controller();
     }
     else
     {
@@ -69,11 +72,21 @@ LifeCost * LifeCost::clone() const
     return ec;
 }
 
-LifeCost::LifeCost(TargetChooser *_tc) :
-    ExtraCost("Life", _tc)
+LifeCost::LifeCost(TargetChooser *_tc)
+    : ExtraCost("Life", _tc)
 {
 }
 
+int LifeCost::canPay()
+{
+    MTGCardInstance * _target = (MTGCardInstance *) target;
+    if(_target->controller()->life <= 0)
+    {
+        return 0;
+    }
+    return 1;
+}
+
 int LifeCost::doPay()
 {
     if (!target)
@@ -88,6 +101,60 @@ int LifeCost::doPay()
     return 1;
 }
 
+//life or Mana cost
+LifeorManaCost * LifeorManaCost::clone() const
+{
+    LifeorManaCost * ec = NEW LifeorManaCost(*this);
+    if (tc)
+        ec->tc = tc->clone();
+    return ec;
+}
+
+LifeorManaCost::LifeorManaCost(TargetChooser *_tc,string manaType)
+    : ExtraCost("Phyrexian Mana", _tc), manaType(manaType)
+{
+}
+
+int LifeorManaCost::canPay()
+{
+    MTGCardInstance * _target = (MTGCardInstance *) target;
+    string buildType ="{";
+    buildType.append(manaType);
+    buildType.append("}");
+    ManaCost * newCost = ManaCost::parseManaCost(buildType);
+    if(_target->controller()->getManaPool()->canAfford(newCost) || _target->controller()->life > 1)
+    {
+        SAFE_DELETE(newCost);
+        return 1;
+    }
+    SAFE_DELETE(newCost);
+    return 0;
+}
+
+int LifeorManaCost::doPay()
+{
+    if (!target)
+        return 0;
+
+    MTGCardInstance * _target = (MTGCardInstance *) target;
+    string buildType ="{";
+    buildType.append(manaType);
+    buildType.append("}");
+    ManaCost * newCost = ManaCost::parseManaCost(buildType);
+    if(_target->controller()->getManaPool()->canAfford(newCost))
+    {
+        _target->controller()->getManaPool()->pay(newCost);
+    }
+    else
+    {
+        _target->controller()->loseLife(2);
+    }
+    SAFE_DELETE(newCost);
+    target = NULL;
+    if (tc)
+        tc->initTargets();
+    return 1;
+}
 //discard a card at random as a cost
 //DiscardRandom cost
 DiscardRandomCost * DiscardRandomCost::clone() const
@@ -98,8 +165,8 @@ DiscardRandomCost * DiscardRandomCost::clone() const
     return ec;
 }
 
-DiscardRandomCost::DiscardRandomCost(TargetChooser *_tc) :
-    ExtraCost("Discard Random", _tc)
+DiscardRandomCost::DiscardRandomCost(TargetChooser *_tc)
+    : ExtraCost("Discard Random", _tc)
 {
 }
 
@@ -138,7 +205,7 @@ DiscardCost * DiscardCost::clone() const
 }
 
 DiscardCost::DiscardCost(TargetChooser *_tc) :
-    ExtraCost("Choose card to Discard", _tc)
+ExtraCost("Choose card to Discard", _tc)
 {
 }
 
@@ -148,7 +215,7 @@ int DiscardCost::doPay()
     if (target)
     {
         WEvent * e = NEW WEventCardDiscard(target);
-        GameObserver * game = GameObserver::GetInstance();
+        GameObserver * game = target->owner->getObserver();
         game->receiveEvent(e);
         _target->controller()->game->putInGraveyard(_target);
         target = NULL;
@@ -158,8 +225,40 @@ int DiscardCost::doPay()
     }
     return 0;
 }
-//to library cost
 
+//cycling
+CycleCost * CycleCost::clone() const
+{
+    CycleCost * ec = NEW CycleCost(*this);
+    if (tc)
+        ec->tc = tc->clone();
+    return ec;
+}
+
+CycleCost::CycleCost(TargetChooser *_tc) :
+ExtraCost("Cycle", _tc)
+{
+}
+
+int CycleCost::doPay()
+{
+    MTGCardInstance * _source = (MTGCardInstance *) source;
+    if (_source)
+    {
+        WEvent * e = NEW WEventCardDiscard(target);//cycling sends 2 events one for the discard and one for the specific cycle trigger
+        GameObserver * game = _source->owner->getObserver();
+        game->receiveEvent(e);
+        WEvent * e2 = NEW WEventCardCycle(_source);
+        game->receiveEvent(e2);
+        _source->controller()->game->putInGraveyard(_source);
+        if (tc)
+            tc->initTargets();
+        return 1;
+    }
+    return 0;
+}
+
+//to library cost
 ToLibraryCost * ToLibraryCost::clone() const
 {
     ToLibraryCost * ec = NEW ToLibraryCost(*this);
@@ -168,8 +267,8 @@ ToLibraryCost * ToLibraryCost::clone() const
     return ec;
 }
 
-ToLibraryCost::ToLibraryCost(TargetChooser *_tc) :
-    ExtraCost("Put a card on top of Library", _tc)
+ToLibraryCost::ToLibraryCost(TargetChooser *_tc)
+    : ExtraCost("Put a card on top of Library", _tc)
 {
 }
 
@@ -196,14 +295,14 @@ MillCost * MillCost::clone() const
     return ec;
 }
 
-MillCost::MillCost(TargetChooser *_tc) :
-    ExtraCost("Deplete", _tc)
+MillCost::MillCost(TargetChooser *_tc)
+    : ExtraCost("Deplete", _tc)
 {
 }
 
 int MillCost::canPay()
 {
-    MTGGameZone * z = target->controller()->game->library;
+    MTGGameZone * z = target?target->controller()->game->library:source->controller()->game->library;
     int nbcards = z->nb_cards;
     if (nbcards < 1)
         return 0;
@@ -216,8 +315,8 @@ int MillCost::doPay()
     if (target)
     {
         _target->controller()->game->putInZone(
-                        _target->controller()->game->library->cards[_target->controller()->game->library->nb_cards - 1],
-                        _target->controller()->game->library, _target->controller()->game->graveyard);
+            _target->controller()->game->library->cards[_target->controller()->game->library->nb_cards - 1],
+            _target->controller()->game->library, _target->controller()->game->graveyard);
         target = NULL;
         if (tc)
             tc->initTargets();
@@ -226,8 +325,8 @@ int MillCost::doPay()
     return 0;
 }
 
-MillExileCost::MillExileCost(TargetChooser *_tc) :
-    MillCost(_tc)
+MillExileCost::MillExileCost(TargetChooser *_tc)
+    : MillCost(_tc)
 {
     // override the base string here
     mCostRenderString = "Deplete To Exile";
@@ -239,8 +338,8 @@ int MillExileCost::doPay()
     if (target)
     {
         _target->controller()->game->putInZone(
-                        _target->controller()->game->library->cards[_target->controller()->game->library->nb_cards - 1],
-                        _target->controller()->game->library, _target->controller()->game->exile);
+            _target->controller()->game->library->cards[_target->controller()->game->library->nb_cards - 1],
+            _target->controller()->game->library, _target->controller()->game->exile);
         target = NULL;
         if (tc)
             tc->initTargets();
@@ -249,6 +348,79 @@ int MillExileCost::doPay()
     return 0;
 }
 
+//Tap cost
+TapCost * TapCost::clone() const
+{
+    TapCost * ec = NEW TapCost(*this);
+    return ec;
+}
+
+TapCost::TapCost()
+    : ExtraCost("Tap")
+{
+}
+
+int TapCost::isPaymentSet()
+{
+    if (source && (source->isTapped() || source->hasSummoningSickness()))
+    {
+        return 0;
+    }
+    return 1;
+}
+
+int TapCost::canPay()
+{
+    return isPaymentSet();
+}
+
+int TapCost::doPay()
+{
+    MTGCardInstance * _source = (MTGCardInstance *) source;
+    if (_source)
+    {
+        _source->tap();
+        return 1;
+    }
+    return 0;
+}
+//unTap cost
+UnTapCost * UnTapCost::clone() const
+{
+    UnTapCost * ec = NEW UnTapCost(*this);
+    return ec;
+}
+
+UnTapCost::UnTapCost() :
+ExtraCost("UnTap")
+{
+}
+
+int UnTapCost::isPaymentSet()
+{
+    if (source && !source->isTapped())
+    {
+        return 0;
+    }
+    return 1;
+}
+
+int UnTapCost::canPay()
+{
+    return isPaymentSet();
+}
+
+int UnTapCost::doPay()
+{
+    MTGCardInstance * _source = (MTGCardInstance *) source;
+    if (_source)
+    {
+        _source->untap();
+        return 1;
+    }
+    return 0;
+}
+
 //Tap target cost
 TapTargetCost * TapTargetCost::clone() const
 {
@@ -258,8 +430,8 @@ TapTargetCost * TapTargetCost::clone() const
     return ec;
 }
 
-TapTargetCost::TapTargetCost(TargetChooser *_tc) :
-    ExtraCost("Tap Target", _tc)
+TapTargetCost::TapTargetCost(TargetChooser *_tc)
+    : ExtraCost("Tap Target", _tc)
 {
 }
 
@@ -300,8 +472,8 @@ ExileTargetCost * ExileTargetCost::clone() const
     return ec;
 }
 
-ExileTargetCost::ExileTargetCost(TargetChooser *_tc) :
-    ExtraCost("Exile Target", _tc)
+ExileTargetCost::ExileTargetCost(TargetChooser *_tc)
+    : ExtraCost("Exile Target", _tc)
 {
 }
 
@@ -328,8 +500,8 @@ BounceTargetCost * BounceTargetCost::clone() const
     return ec;
 }
 
-BounceTargetCost::BounceTargetCost(TargetChooser *_tc) :
-    ExtraCost("Return Target to Hand", _tc)
+BounceTargetCost::BounceTargetCost(TargetChooser *_tc)
+    : ExtraCost("Return Target to Hand", _tc)
 {
 }
 
@@ -357,16 +529,15 @@ Ninja * Ninja::clone() const
 }
 
 Ninja::Ninja(TargetChooser *_tc) :
-    ExtraCost("Select unblocked attacker", _tc)
+ExtraCost("Select unblocked attacker", _tc)
 {
 }
 
 int Ninja::isPaymentSet()
 {
-    GameObserver * g = GameObserver::GetInstance();
-    int currentPhase = g->getCurrentGamePhase();
-    if (target && ((target->isAttacker() && target->blocked) || target->isAttacker() < 1 || currentPhase
-                    != Constants::MTG_PHASE_COMBATBLOCKERS))
+    if (target && ((target->isAttacker() && target->isBlocked()) ||
+                   target->isAttacker() < 1 ||
+                   target->getObserver()->getCurrentGamePhase() != MTG_PHASE_COMBATBLOCKERS))
     {
         tc->removeTarget(target);
         target = NULL;
@@ -379,7 +550,6 @@ int Ninja::isPaymentSet()
 
 int Ninja::doPay()
 {
-
     if (target)
     {
         target->controller()->game->putInHand(target);
@@ -402,8 +572,8 @@ SacrificeCost * SacrificeCost::clone() const
     return ec;
 }
 
-SacrificeCost::SacrificeCost(TargetChooser *_tc) :
-    ExtraCost("Sacrifice", _tc)
+SacrificeCost::SacrificeCost(TargetChooser *_tc)
+    : ExtraCost("Sacrifice", _tc)
 {
 }
 
@@ -412,7 +582,7 @@ int SacrificeCost::doPay()
     if (target)
     {
         WEvent * e = NEW WEventCardSacrifice(target);
-        GameObserver * game = GameObserver::GetInstance();
+        GameObserver * game = target->owner->getObserver();
         game->receiveEvent(e);
         target->controller()->game->putInGraveyard(target);
         target = NULL;
@@ -430,14 +600,19 @@ CounterCost * CounterCost::clone() const
     CounterCost * ec = NEW CounterCost(*this);
     if (tc)
         ec->tc = tc->clone();
+    
     if (counter)
         ec->counter = NEW Counter(counter->target, counter->name.c_str(), counter->power, counter->toughness);
+
+    //TODO: counter can be NULL at this point, what do we set ec->counter->nb to if it is?
+    if (ec->counter != NULL)
         ec->counter->nb = counter->nb;
+    
     return ec;
 }
 
 CounterCost::CounterCost(Counter * _counter, TargetChooser *_tc) :
-    ExtraCost("Counters", _tc)
+ExtraCost("Counters", _tc)
 {
     counter = _counter;
     hasCounters = 0;
@@ -488,9 +663,20 @@ int CounterCost::canPay()
     if (counter->nb >= 0)
         return 1; //add counters always possible
     // otherwise, move on only if target has enough counters
-    Counter * targetCounter = target->counters->hasCounter(counter->name.c_str(), counter->power, counter->toughness);
+    Counter * targetCounter = NULL;
+    if(target)
+    {
+        targetCounter = target->counters->hasCounter(counter->name.c_str(), counter->power, counter->toughness);
+
+    }
+    else
+    {
+        targetCounter = source->counters->hasCounter(counter->name.c_str(), counter->power, counter->toughness);
+    }
     if (targetCounter && targetCounter->nb >= -counter->nb)
+    {
         return 1;
+    }
     return 0;
 }
 
@@ -505,6 +691,9 @@ int CounterCost::doPay()
         {
             target->counters->addCounter(counter->name.c_str(), counter->power, counter->toughness);
         }
+        if (tc)
+            tc->initTargets();
+        target = NULL;
         return 1;
     }
 
@@ -516,8 +705,14 @@ int CounterCost::doPay()
             target->counters->removeCounter(counter->name.c_str(), counter->power, counter->toughness);
         }
         hasCounters = 0;
+        if (tc)
+            tc->initTargets();
+        target = NULL;
         return 1;
     }
+    if (tc)
+        tc->initTargets();
+    target = NULL;
     return 0;
 }
 
@@ -611,7 +806,8 @@ int ExtraCosts::canPay()
     {
         if (!costs[i]->canPay())
         {
-            costs[i]->target->isExtraCostTarget = false;
+            if(costs[i]->target)
+                costs[i]->target->isExtraCostTarget = false;
             return 0;
         }
     }
@@ -623,8 +819,8 @@ int ExtraCosts::doPay()
     int result = 0;
     for (size_t i = 0; i < costs.size(); i++)
     {
-    if(costs[i]->target)
-        costs[i]->target->isExtraCostTarget = false;
+        if(costs[i]->target)
+            costs[i]->target->isExtraCostTarget = false;
         result += costs[i]->doPay();
     }
     return result;
