@@ -20,9 +20,9 @@
 #include "DeckEditorMenu.h"
 #include "SimpleMenu.h"
 #include "utils.h"
+#include "AIPlayer.h"
+
 
-// This is pending a change by Wil regarding graphics threads
-#define GRAPHICS_NO_THREADING
 
 //!! helper function; this is probably handled somewhere in the code already.
 // If not, should be placed in general library
@@ -40,7 +40,7 @@ void StringExplode(string str, string separator, vector<string>* results)
 }
 
 GameStateDeckViewer::GameStateDeckViewer(GameApp* parent) :
-    GameState(parent)
+    GameState(parent, "deckeditor")
 {
     bgMusic = NULL;
     useFilter = 0;
@@ -219,27 +219,21 @@ void GameStateDeckViewer::Start()
     lastPos = 0;
     lastTotal = 0;
 
-    pricelist = NEW PriceList(JGE_GET_RES("settings/prices.dat").c_str(), MTGCollection());
+    pricelist = NEW PriceList("settings/prices.dat", MTGCollection());
     playerdata = NEW PlayerData(MTGCollection());
     myCollection = NEW DeckDataWrapper(playerdata->collection);
     myCollection->Sort(WSrcCards::SORT_ALPHA);
     displayed_deck = myCollection;
 
     //Icons
-    mIcons[Constants::MTG_COLOR_ARTIFACT] = WResourceManager::Instance()->GetQuad("c_artifact");
-    mIcons[Constants::MTG_COLOR_LAND] = WResourceManager::Instance()->GetQuad("c_land");
-    mIcons[Constants::MTG_COLOR_WHITE] = WResourceManager::Instance()->GetQuad("c_white");
-    mIcons[Constants::MTG_COLOR_RED] = WResourceManager::Instance()->GetQuad("c_red");
-    mIcons[Constants::MTG_COLOR_BLACK] = WResourceManager::Instance()->GetQuad("c_black");
-    mIcons[Constants::MTG_COLOR_BLUE] = WResourceManager::Instance()->GetQuad("c_blue");
-    mIcons[Constants::MTG_COLOR_GREEN] = WResourceManager::Instance()->GetQuad("c_green");
-    for (int i = 0; i < 7; i++)
+    mIcons = manaIcons;
+    for (int i = 0; i < Constants::NB_Colors; i++)
     {
         mIcons[i]->SetHotSpot(16, 16);
     }
 
     //Grab a texture in VRAM.
-    pspIconsTexture = WResourceManager::Instance()->RetrieveTexture("iconspsp.png", RETRIEVE_LOCK);
+    pspIconsTexture = WResourceManager::Instance()->RetrieveTexture("iconspsp.png", RETRIEVE_MANAGE);
 
     char buf[512];
     for (int i = 0; i < 8; i++)
@@ -252,7 +246,7 @@ void GameStateDeckViewer::Start()
     //init welcome menu
     updateDecks();
 
-    GameApp::playMusic("track1.mp3");
+    GameApp::playMusic("Track1.mp3");
 
     loadIndexes();
     mEngine->ResetInput();
@@ -322,24 +316,22 @@ void GameStateDeckViewer::saveDeck()
 void GameStateDeckViewer::saveAsAIDeck(string deckName)
 {
 
-    vector<DeckMetaData *> aiDecks = GameState::BuildDeckList(JGE_GET_RES("ai/baka"), "ai_baka", NULL);
-    int nbAiDecks = aiDecks.size() + 1;
-    aiDecks.clear();
+    int deckId = AIPlayer::getTotalAIDecks() + 1;
 
-    string defaultAiDeckName = "deck";
     std::ostringstream oss;
-    oss << "deck" << nbAiDecks;
-    defaultAiDeckName = oss.str();
+    oss << "deck" <<deckId;
+    string aiDeckName = oss.str();
     oss.str("");
     if (myDeck->parent->meta_desc == "")
-        oss << endl << "Can you beat your own creations?" << endl << "User created AI Deck # " << nbAiDecks;
+        oss << endl << "Can you beat your own creations?" << endl << "User created AI Deck # " << deckId;
     else
         oss << myDeck->parent->meta_desc;
     string deckDesc = oss.str();
-    string filepath = JGE_GET_RES("ai/baka/");
-    filepath.append(defaultAiDeckName).append(".txt");
+    string filepath = "ai/baka/";
+    filepath.append(aiDeckName).append(".txt");
     DebugTrace("saving AI deck " << filepath);
     myDeck->save(filepath, true, deckName, deckDesc);
+    AIPlayer::invalidateTotalAIDecks(); //We added one AI deck, so we need to invalidate the count cache
 }
 
 void GameStateDeckViewer::Update(float dt)
@@ -424,8 +416,30 @@ void GameStateDeckViewer::Update(float dt)
             }
             break;
         case JGE_BTN_OK:
-            last_user_activity = 0;
-            addRemove(cardIndex[2]);
+            if (mEngine->GetLeftClickCoordinates(x, y))
+            {
+              for(int i=0; i < CARDS_DISPLAYED; i++)
+              {
+                distance2 = static_cast<unsigned int>((cardsCoordinates[i].second - y) * (cardsCoordinates[i].second - y) + (cardsCoordinates[i].first - x) * (cardsCoordinates[i].first - x));
+                if (distance2 < minDistance2)
+                {
+                    minDistance2 = distance2;
+                    n = i;
+                }
+              }
+
+              if(n!=2) {
+                mSelected = n;
+                last_user_activity = 0;
+                mStage = STAGE_TRANSITION_SELECTED;
+              }
+              mEngine->LeftClickedProcessed();
+            }
+            if(mStage != STAGE_TRANSITION_SELECTED)
+            {
+                last_user_activity = 0;
+                addRemove(cardIndex[2]);
+            }
             break;
         case JGE_BTN_SEC:
             last_user_activity = 0;
@@ -439,7 +453,7 @@ void GameStateDeckViewer::Update(float dt)
                     sprintf(buffer, "%s : %i %s", _(card->data->getName()).c_str(), price, _("credits").c_str());
                     const float menuXOffset = SCREEN_WIDTH_F - 300;
                     const float menuYOffset = SCREEN_HEIGHT_F / 2;
-                    subMenu = NEW SimpleMenu(MENU_CARD_PURCHASE, this, Fonts::MAIN_FONT, menuXOffset, menuYOffset, buffer);
+                    subMenu = NEW SimpleMenu(JGE::GetInstance(), MENU_CARD_PURCHASE, this, Fonts::MAIN_FONT, menuXOffset, menuYOffset, buffer);
                     subMenu->Add(MENU_ITEM_YES, "Yes");
                     subMenu->Add(MENU_ITEM_NO, "No", "", true);
                 }
@@ -475,32 +489,6 @@ void GameStateDeckViewer::Update(float dt)
             else if ((mStage == STAGE_ONSCREEN_MENU) && (++stw->currentPage > stw->pageCount)) stw->currentPage = 0;
             break;
         default: // no keypress
-            if (mEngine->GetLeftClickCoordinates(x, y))
-            {
-              for(int i=0; i < CARDS_DISPLAYED; i++)
-              {
-                distance2 = (cardsCoordinates[i].second - y) * (cardsCoordinates[i].second - y) + (cardsCoordinates[i].first - x) * (cardsCoordinates[i].first - x);
-                if (distance2 < minDistance2)
-                {
-                    minDistance2 = distance2;
-                    n = i;
-                }
-              }
-
-              if (n < ((CARDS_DISPLAYED/2) - 1))
-              {
-                  last_user_activity = 0;
-                  mStage = STAGE_TRANSITION_RIGHT;
-              }
-              if (n > ((CARDS_DISPLAYED/2) + 1))
-              {
-                last_user_activity = 0;
-                mStage = STAGE_TRANSITION_LEFT;
-              }
-              mEngine->LeftClickedProcessed();
-              break;
-            }
-
             if (last_user_activity > NO_USER_ACTIVITY_HELP_DELAY)
             {
                 if (mStage != STAGE_ONSCREEN_MENU)
@@ -521,6 +509,37 @@ void GameStateDeckViewer::Update(float dt)
         }
 
     }
+    if (mStage == STAGE_TRANSITION_SELECTED)
+    {
+        if (mSelected < 2)
+        {
+            mRotation -= dt * MED_SPEED;
+            if (mRotation < mSelected-2)
+            {
+                do
+                {
+                    rotateCards(STAGE_TRANSITION_RIGHT);
+                    mRotation += 1;
+                } while (mRotation < -1.0f);
+                mStage = STAGE_WAITING;
+                mRotation = 0;
+            }
+        }
+        else if (mSelected > 2)
+        {
+            mRotation += dt * MED_SPEED;
+            if (mRotation > mSelected - 2)
+            {
+                do
+                {
+                    rotateCards(STAGE_TRANSITION_LEFT);
+                    mRotation -= 1;
+                } while (mRotation > 1.0f);
+                mStage = STAGE_WAITING;
+                mRotation = 0;
+            }
+        }
+    }
     if (mStage == STAGE_TRANSITION_RIGHT || mStage == STAGE_TRANSITION_LEFT)
     {
         if (mStage == STAGE_TRANSITION_RIGHT)
@@ -697,7 +716,7 @@ void GameStateDeckViewer::renderDeckBackground()
     int max2 = 0;
     int maxC2 = 4;
 
-    for (int i = 0; i < Constants::MTG_NB_COLORS - 1; i++)
+    for (int i = 0; i < Constants::NB_Colors - 1; i++)
     {
         int value = myDeck->getCount(i);
         if (value > max1)
@@ -790,7 +809,7 @@ void GameStateDeckViewer::renderOnScreenMenu()
         //Your Deck Information
         char buffer[300];
         int nb_letters = 0;
-        for (int j = 0; j < Constants::MTG_NB_COLORS; j++)
+        for (int j = 0; j < Constants::NB_Colors; j++)
         {
             int value = myDeck->getCount(j);
             if (value > 0)
@@ -864,7 +883,7 @@ void GameStateDeckViewer::renderOnScreenMenu()
             posY += 10;
 
             // Counts by color
-            for (int j = 0; j < Constants::MTG_NB_COLORS; j++)
+            for (int j = 0; j < Constants::NB_Colors; j++)
             {
                 int value = myDeck->getCount(j);
                 if (value > 0)
@@ -982,7 +1001,7 @@ void GameStateDeckViewer::renderOnScreenMenu()
             posY = 70;
 
             // Column titles
-            for (int j = 0; j < Constants::MTG_NB_COLORS - 1; j++)
+            for (int j = 0; j < Constants::NB_Colors - 1; j++)
             {
                 r->RenderQuad(mIcons[j].get(), 52 + j * 15 + leftTransition, posY - 10, 0, 0.5, 0.5);
             }
@@ -991,20 +1010,20 @@ void GameStateDeckViewer::renderOnScreenMenu()
             //font->DrawString(_("Ty"), 27 + leftTransition, posY-16);
 
             // Horizontal table lines
-            r->DrawLine(27 + leftTransition, posY - 20, 60 + (Constants::MTG_NB_COLORS - 2) * 15 + leftTransition, posY - 20,
+            r->DrawLine(27 + leftTransition, posY - 20, 60 + (Constants::NB_Colors - 2) * 15 + leftTransition, posY - 20,
                     ARGB(128, 255, 255, 255));
-            r->DrawLine(27 + leftTransition, posY - 1, 60 + (Constants::MTG_NB_COLORS - 2) * 15 + leftTransition, posY - 1,
+            r->DrawLine(27 + leftTransition, posY - 1, 60 + (Constants::NB_Colors - 2) * 15 + leftTransition, posY - 1,
                     ARGB(128, 255, 255, 255));
-            r->DrawLine(27 + leftTransition, 2 * 10 + posY + 12, 60 + (Constants::MTG_NB_COLORS - 2) * 15 + leftTransition, 2 * 10
+            r->DrawLine(27 + leftTransition, 2 * 10 + posY + 12, 60 + (Constants::NB_Colors - 2) * 15 + leftTransition, 2 * 10
                     + posY + 12, ARGB(128, 255, 255, 255));
-            r->DrawLine(27 + leftTransition, 3 * 10 + posY + 14, 60 + (Constants::MTG_NB_COLORS - 2) * 15 + leftTransition, 3 * 10
+            r->DrawLine(27 + leftTransition, 3 * 10 + posY + 14, 60 + (Constants::NB_Colors - 2) * 15 + leftTransition, 3 * 10
                     + posY + 14, ARGB(128, 255, 255, 255));
 
             // Vertical table lines
             r->DrawLine(26 + leftTransition, posY - 20, 26 + leftTransition, 3 * 10 + posY + 14, ARGB(128, 255, 255, 255));
             r->DrawLine(43 + leftTransition, posY - 20, 43 + leftTransition, 3 * 10 + posY + 14, ARGB(128, 255, 255, 255));
-            r->DrawLine(60 + leftTransition + (Constants::MTG_NB_COLORS - 2) * 15, posY - 20, 60 + leftTransition
-                    + (Constants::MTG_NB_COLORS - 2) * 15, 3 * 10 + posY + 14, ARGB(128, 255, 255, 255));
+            r->DrawLine(60 + leftTransition + (Constants::NB_Colors - 2) * 15, posY - 20, 60 + leftTransition
+                    + (Constants::NB_Colors - 2) * 15, 3 * 10 + posY + 14, ARGB(128, 255, 255, 255));
 
             font->DrawString(_("BL"), 27 + leftTransition, posY);
             font->DrawString(_("NB"), 27 + leftTransition, posY + 10);
@@ -1013,7 +1032,7 @@ void GameStateDeckViewer::renderOnScreenMenu()
 
             int curCount;
 
-            for (int j = 0; j < Constants::MTG_NB_COLORS - 1; j++)
+            for (int j = 0; j < Constants::NB_Colors - 1; j++)
             {
                 curCount = stw->countBasicLandsPerColor[j];
                 sprintf(buffer, (curCount == 0 ? "." : "%i"), curCount);
@@ -1050,13 +1069,13 @@ void GameStateDeckViewer::renderOnScreenMenu()
 
             int totalProducedSymbols;
             totalProducedSymbols = 0;
-            for (int i = 1; i < Constants::MTG_NB_COLORS - 1; i++)
+            for (int i = 1; i < Constants::NB_Colors - 1; i++)
             {
                 totalProducedSymbols += stw->countLandsPerColor[i] + stw->countBasicLandsPerColor[i]; //!! Move to updatestats!
             }
 
             posY = 50;
-            for (int i = 1; i < Constants::MTG_NB_COLORS - 1; i++)
+            for (int i = 1; i < Constants::NB_Colors - 1; i++)
             {
                 if (stw->countLandsPerColor[i] + stw->countBasicLandsPerColor[i] > 0)
                 {
@@ -1128,7 +1147,7 @@ void GameStateDeckViewer::renderOnScreenMenu()
             posY = 70;
 
             // Column titles
-            for (int j = 0; j < Constants::MTG_NB_COLORS - 1; j++)
+            for (int j = 0; j < Constants::NB_Colors - 1; j++)
             {
                 r->RenderQuad(mIcons[j].get(), 67 + j * 15 + leftTransition, posY - 10, 0, 0.5, 0.5);
             }
@@ -1137,11 +1156,11 @@ void GameStateDeckViewer::renderOnScreenMenu()
             font->DrawString(_("#"), 45 + leftTransition, posY - 16);
 
             // Horizontal table lines
-            r->DrawLine(27 + leftTransition, posY - 20, 75 + (Constants::MTG_NB_COLORS - 2) * 15 + leftTransition, posY - 20,
+            r->DrawLine(27 + leftTransition, posY - 20, 75 + (Constants::NB_Colors - 2) * 15 + leftTransition, posY - 20,
                     ARGB(128, 255, 255, 255));
-            r->DrawLine(27 + leftTransition, posY - 1, 75 + (Constants::MTG_NB_COLORS - 2) * 15 + leftTransition, posY - 1,
+            r->DrawLine(27 + leftTransition, posY - 1, 75 + (Constants::NB_Colors - 2) * 15 + leftTransition, posY - 1,
                     ARGB(128, 255, 255, 255));
-            r->DrawLine(27 + leftTransition, Constants::STATS_MAX_MANA_COST * 10 + posY + 12, 75 + (Constants::MTG_NB_COLORS - 2)
+            r->DrawLine(27 + leftTransition, Constants::STATS_MAX_MANA_COST * 10 + posY + 12, 75 + (Constants::NB_Colors - 2)
                     * 15 + leftTransition, Constants::STATS_MAX_MANA_COST * 10 + posY + 12, ARGB(128, 255, 255, 255));
 
             // Vertical table lines
@@ -1151,8 +1170,8 @@ void GameStateDeckViewer::renderOnScreenMenu()
                     ARGB(128, 255, 255, 255));
             r->DrawLine(58 + leftTransition, posY - 20, 58 + leftTransition, Constants::STATS_MAX_MANA_COST * 10 + posY + 12,
                     ARGB(128, 255, 255, 255));
-            r->DrawLine(75 + leftTransition + (Constants::MTG_NB_COLORS - 2) * 15, posY - 20, 75 + leftTransition
-                    + (Constants::MTG_NB_COLORS - 2) * 15, Constants::STATS_MAX_MANA_COST * 10 + posY + 12,
+            r->DrawLine(75 + leftTransition + (Constants::NB_Colors - 2) * 15, posY - 20, 75 + leftTransition
+                    + (Constants::NB_Colors - 2) * 15, Constants::STATS_MAX_MANA_COST * 10 + posY + 12,
                     ARGB(128, 255, 255, 255));
 
             for (int i = 0; i <= Constants::STATS_MAX_MANA_COST; i++)
@@ -1161,12 +1180,12 @@ void GameStateDeckViewer::renderOnScreenMenu()
                 font->DrawString(buffer, 30 + leftTransition, posY);
                 sprintf(buffer, ((*countPerCost)[i] > 0) ? _("%i").c_str() : ".", (*countPerCost)[i]);
                 font->DrawString(buffer, 45 + leftTransition, posY);
-                for (int j = 0; j < Constants::MTG_NB_COLORS - 1; j++)
+                for (int j = 0; j < Constants::NB_Colors - 1; j++)
                 {
                     sprintf(buffer, ((*countPerCostAndColor)[i][j] > 0) ? _("%i").c_str() : ".", (*countPerCostAndColor)[i][j]);
                     font->DrawString(buffer, 64 + leftTransition + j * 15, posY);
                 }
-                r->FillRect(77.f + leftTransition + (Constants::MTG_NB_COLORS - 2) * 15.0f, posY + 2.0f, (*countPerCost)[i] * 5.0f,
+                r->FillRect(77.f + leftTransition + (Constants::NB_Colors - 2) * 15.0f, posY + 2.0f, (*countPerCost)[i] * 5.0f,
                         8.0f, graphColor);
                 posY += 10;
             }
@@ -1231,7 +1250,7 @@ void GameStateDeckViewer::renderOnScreenMenu()
             font->DrawString(_("Total colored manasymbols in cards' casting costs:"), 20 + leftTransition, 30);
 
             posY = 50;
-            for (int i = 1; i < Constants::MTG_NB_COLORS - 1; i++)
+            for (int i = 1; i < Constants::NB_Colors - 1; i++)
             {
                 if (stw->totalCostPerColor[i] > 0)
                 {
@@ -1314,63 +1333,57 @@ void GameStateDeckViewer::renderCard(int id, float rotation)
     int alpha = (int) (255 * (scale + 1.0 - max_scale));
 
     if (!card) return;
-
-#ifdef GRAPHICS_NO_THREADING
-
-    JQuadPtr backQuad = WResourceManager::Instance()->GetQuad("back");
-    JQuadPtr quad;
-
-    int cacheError = CACHE_ERROR_NONE;
-
-    if (!options[Options::DISABLECARDS].number)
+    
+    if (!WResourceManager::Instance()->IsThreaded())
     {
-        quad = WResourceManager::Instance()->RetrieveCard(card, RETRIEVE_EXISTING);
-        cacheError = WResourceManager::Instance()->RetrieveError();
-        if (!quad.get() && cacheError != CACHE_ERROR_404)
+        JQuadPtr backQuad = WResourceManager::Instance()->GetQuad(kGenericCardID);
+        JQuadPtr quad;
+
+        int cacheError = CACHE_ERROR_NONE;
+
+        if (!options[Options::DISABLECARDS].number)
         {
-            if (last_user_activity > (abs(2 - id) + 1) * NO_USER_ACTIVITY_SHOWCARD_DELAY)
-                quad = WResourceManager::Instance()->RetrieveCard(card);
-            else
+            quad = WResourceManager::Instance()->RetrieveCard(card, RETRIEVE_EXISTING);
+            cacheError = WResourceManager::Instance()->RetrieveError();
+            if (!quad.get() && cacheError != CACHE_ERROR_404)
             {
-                quad = backQuad;
+                if (last_user_activity > (abs(2 - id) + 1) * NO_USER_ACTIVITY_SHOWCARD_DELAY)
+                    quad = WResourceManager::Instance()->RetrieveCard(card);
+                else
+                {
+                    quad = backQuad;
+                }
             }
         }
-    }
 
-    if (quad.get())
-    {
-        if (quad == backQuad)
+        if (quad.get())
         {
-            quad->SetColor(ARGB(255,255,255,255));
-            float _scale = scale * (285 / quad->mHeight);
-            JRenderer::GetInstance()->RenderQuad(quad.get(), x, y, 0.0f, _scale, _scale);
+            if (quad == backQuad)
+            {
+                quad->SetColor(ARGB(255,255,255,255));
+                float _scale = scale * (285 / quad->mHeight);
+                JRenderer::GetInstance()->RenderQuad(quad.get(), x, y, 0.0f, _scale, _scale);
+            }
+            else
+            {
+                Pos pos = Pos(x, y, scale * 285 / 250, 0.0, 255);
+                CardGui::DrawCard(card, pos);
+            }
         }
         else
         {
             Pos pos = Pos(x, y, scale * 285 / 250, 0.0, 255);
-            CardGui::DrawCard(card, pos);
+            CardGui::DrawCard(card, pos, DrawMode::kText);
         }
     }
     else
     {
+        int mode = !options[Options::DISABLECARDS].number ? DrawMode::kNormal : DrawMode::kText;
+
         Pos pos = Pos(x, y, scale * 285 / 250, 0.0, 255);
-        CardGui::DrawCard(card, pos, DrawMode::kText);
-        if (!options[Options::DISABLECARDS].number) quad = WResourceManager::Instance()->RetrieveCard(card, CACHE_THUMB);
-        if (quad.get())
-        {
-            float _scale = 285 * scale / quad->mHeight;
-            quad->SetColor(ARGB(40,255,255,255));
-            JRenderer::GetInstance()->RenderQuad(quad.get(), x, y, 0, _scale, _scale);
-        }
+        CardGui::DrawCard(card, pos, mode);
     }
 
-#else
-    int mode = !options[Options::DISABLECARDS].number ? DrawMode::kNormal : DrawMode::kText;
-
-    Pos pos = Pos(x, y, scale * 285 / 250, 0.0, 255);
-    CardGui::DrawCard(card, pos, mode);
-#endif
-
     int quadAlpha = alpha;
     if (!displayed_deck->count(card)) quadAlpha /= 2;
     quadAlpha = 255 - quadAlpha;
@@ -1421,6 +1434,19 @@ void GameStateDeckViewer::Render()
         order[2] = 1;
     }
 
+    // even though we want to draw the cards in a particular z order for layering, we want to prefetch them 
+    // in a different order, ie the center card should appear first, then the adjacent ones
+    if (WResourceManager::Instance()->IsThreaded())
+    {
+        WResourceManager::Instance()->RetrieveCard(cardIndex[0]);
+        WResourceManager::Instance()->RetrieveCard(cardIndex[3]);
+        WResourceManager::Instance()->RetrieveCard(cardIndex[4]);
+        WResourceManager::Instance()->RetrieveCard(cardIndex[2]);
+        WResourceManager::Instance()->RetrieveCard(cardIndex[5]);
+        WResourceManager::Instance()->RetrieveCard(cardIndex[1]);
+        WResourceManager::Instance()->RetrieveCard(cardIndex[6]);
+    }
+
     renderCard(6, mRotation);
     renderCard(5, mRotation);
     renderCard(4, mRotation);
@@ -1490,7 +1516,7 @@ int GameStateDeckViewer::loadDeck(int deckid)
         SAFE_DELETE(myDeck->parent);
         SAFE_DELETE(myDeck);
     }
-    myDeck = NEW DeckDataWrapper(NEW MTGDeck(options.profileFile(deckname, "", false, false).c_str(), MTGCollection()));
+    myDeck = NEW DeckDataWrapper(NEW MTGDeck(options.profileFile(deckname, "", false).c_str(), MTGCollection()));
 
     // Check whether the cards in the deck are actually available in the player's collection:
     int cheatmode = options[Options::CHEATMODE].number;
@@ -1662,3 +1688,24 @@ void GameStateDeckViewer::ButtonPressed(int controllerId, int controlId)
     }
 }
 
+void GameStateDeckViewer::OnScroll(int inXVelocity, int inYVelocity)
+{
+    bool flickHorizontal = (abs(inXVelocity) > abs(inYVelocity));
+    bool flickUp = inYVelocity < 0 ? true : false;
+    bool flickRight = inXVelocity > 0 ? true : false;
+
+    if (mStage == STAGE_FILTERS)
+    {
+        if (flickHorizontal)
+            mEngine->HoldKey_NoRepeat( flickRight ? JGE_BTN_RIGHT : JGE_BTN_LEFT);
+        else
+            mEngine->HoldKey_NoRepeat(flickUp ? JGE_BTN_UP : JGE_BTN_DOWN);
+    }
+    else
+    {
+        if (flickHorizontal && (abs(inXVelocity) > 300))
+            mEngine->HoldKey_NoRepeat(JGE_BTN_PRI);
+        else if (!flickHorizontal && (abs(inYVelocity) > 300))
+            mEngine->HoldKey_NoRepeat(flickUp ? JGE_BTN_UP : JGE_BTN_DOWN);
+    }
+}
