@@ -10,7 +10,6 @@
 #include "GameStateMenu.h"
 #include "DeckManager.h"
 #include "MenuItem.h"
-#include "GameOptions.h"
 #include "GameApp.h"
 #include "MTGCard.h"
 #include "Translate.h"
@@ -19,16 +18,16 @@
 #include "utils.h"
 #include "WFont.h"
 #include <JLogger.h>
+#include "Rules.h"
+#include "ModRules.h"
+#include "Credits.h"
+#include "AIPlayer.h"
+
 #ifdef NETWORK_SUPPORT
 #include <JNetwork.h>
 #endif//NETWORK_SUPPORT
 
-static const char* GAME_VERSION = "WTH?! 0.15.1 - wololo.net";
-
-#define DEFAULT_ANGLE_MULTIPLIER 0.4f
-#define MAX_ANGLE_MULTIPLIER (3*M_PI)
-#define MIN_ANGLE_MULTIPLIER 0.4f
-static const float STEP_ANGLE_MULTIPLIER = 0.0002f;
+static const char* GAME_VERSION = "WTH?! " WAGIC_VERSION_STRING " - wololo.net";
 
 enum ENUM_MENU_STATE_MAJOR
 {
@@ -54,39 +53,14 @@ enum ENUM_MENU_STATE_MINOR
     MENU_STATE_MINOR = 0xF00
 };
 
-enum
-{
-    SUBMENUITEM_CANCEL = kCancelMenuID,
-    MENUITEM_PLAY,
-    MENUITEM_DECKEDITOR,
-    MENUITEM_SHOP,
-    MENUITEM_OPTIONS,
-    MENUITEM_EXIT,
-    SUBMENUITEM_1PLAYER,
-#ifdef NETWORK_SUPPORT
-    SUBMENUITEM_2PLAYERS,
-    SUBMENUITEM_HOST_GAME,
-    SUBMENUITEM_JOIN_GAME,
-#endif //NETWORK_SUPPORT
-    SUBMENUITEM_DEMO,
-    SUBMENUITEM_TESTSUITE,
-    SUBMENUITEM_MOMIR,
-    SUBMENUITEM_CLASSIC,
-    SUBMENUITEM_RANDOM1,
-    SUBMENUITEM_RANDOM2,
-    SUBMENUITEM_STORY
-};
-
 GameStateMenu::GameStateMenu(GameApp* parent) :
-    GameState(parent)
+    GameState(parent, "menu")
 {
     mGuiController = NULL;
     subMenuController = NULL;
     gameTypeMenu = NULL;
     //bgMusic = NULL;
     timeIndex = 0;
-    angleMultiplier = MIN_ANGLE_MULTIPLIER;
-    yW = 55;
     mVolume = 0;
     scroller = NULL;
     langChoices = false;
@@ -99,10 +73,8 @@ GameStateMenu::~GameStateMenu()
 
 void GameStateMenu::Create()
 {
-    mDip = NULL;
     mGuiController = NULL;
     mReadConf = 0;
-    mCurrentSetName[0] = 0;
 
     //load all the icon images. Menu icons are managed, so we can do this here.
     int n = 0;
@@ -125,8 +97,10 @@ void GameStateMenu::Create()
     string lang = options[Options::LANG].str;
     if (lang.size())
     {
-        lang = JGE_GET_RES("lang/") + lang + ".txt";
-        if (fileExists(lang.c_str()))
+        string langpath = "lang/";
+        langpath.append(lang);
+        langpath.append(".txt");
+        if (JFileSystem::GetInstance()->FileExists(langpath))
             langChosen = true;
     }
     if (!langChosen)
@@ -135,8 +109,10 @@ void GameStateMenu::Create()
     }
     scroller = NEW TextScroller(Fonts::MAIN_FONT, SCREEN_WIDTH / 2 - 90, SCREEN_HEIGHT - 17, 180);
     scrollerSet = 0;
-
     splashTex = NULL;
+
+    JFileSystem::GetInstance()->scanfolder("sets/", setFolders);
+    mCurrentSetFolderIndex = 0;
 }
 
 void GameStateMenu::Destroy()
@@ -160,10 +136,9 @@ void GameStateMenu::Start()
     hasChosenGameType = false;
     mParent->gameType = GAME_TYPE_CLASSIC;
 
-    /*
-     if (options[Options::MOMIR_MODE_UNLOCKED].number) hasChosenGameType = 0;
-     if (options[Options::RANDOMDECK_MODE_UNLOCKED].number) hasChosenGameType = 0;
-     */
+    //Manual clean up of some cache Data. Ideally those should clean themselves up, so this is kind of a hack for now
+    WResourceManager::Instance()->ClearUnlocked();
+
     bgTexture = WResourceManager::Instance()->RetrieveTexture("menutitle.png", RETRIEVE_LOCK);
     mBg = WResourceManager::Instance()->RetrieveQuad("menutitle.png", 0, 0, 256, 166); // Create background quad for rendering.
 
@@ -174,19 +149,33 @@ void GameStateMenu::Start()
         currentState = currentState | MENU_STATE_MINOR_FADEIN;
 
     wallpaper = "";
+    scrollerSet = 0; // This will force-update the scroller text
+    mPercentComplete = 0;
 }
 
 void GameStateMenu::genNbCardsStr()
 {
     //How many cards total ?
     PlayerData * playerdata = NEW PlayerData(MTGCollection());
-    if (playerdata && !options[Options::ACTIVE_PROFILE].isDefault())
-        sprintf(nbcardsStr, _("%s: %i cards (%i) (%i unique)").c_str(), options[Options::ACTIVE_PROFILE].str.c_str(),
-                        playerdata->collection->totalCards(), MTGCollection()->totalCards(),
-                        MTGCollection()->primitives.size());
+    size_t totalUnique =  MTGCollection()->primitives.size();
+    size_t totalPrints = MTGCollection()->totalCards();
+
+    if (totalUnique != totalPrints)
+    {
+        if (playerdata && !options[Options::ACTIVE_PROFILE].isDefault())
+            sprintf(nbcardsStr, _("%s: %i cards (%i) (%i unique)").c_str(), options[Options::ACTIVE_PROFILE].str.c_str(),
+                            playerdata->collection->totalCards(), totalPrints,totalUnique);
+        else
+            sprintf(nbcardsStr, _("%i cards (%i unique)").c_str(),totalPrints,totalUnique);
+    }
     else
-        sprintf(nbcardsStr, _("%i cards (%i unique)").c_str(), MTGCollection()->totalCards(),
-                        MTGCollection()->primitives.size());
+    {
+        if (playerdata && !options[Options::ACTIVE_PROFILE].isDefault())
+            sprintf(nbcardsStr, _("%s: %i cards (%i)").c_str(), options[Options::ACTIVE_PROFILE].str.c_str(),
+                            playerdata->collection->totalCards(), totalPrints);
+        else
+            sprintf(nbcardsStr, _("%i cards").c_str(),totalPrints);
+    }
 
     SAFE_DELETE(playerdata);
 }
@@ -194,45 +183,20 @@ void GameStateMenu::genNbCardsStr()
 void GameStateMenu::fillScroller()
 {
     scroller->Reset();
-    char buffer[4096];
     char buff2[512];
 
-    DeckStats * stats = DeckStats::GetInstance();
-	vector<DeckMetaData *> playerDecks = BuildDeckList(options.profileFile(), "", NULL, 6);
-    int totalGames = 0;
-	for (size_t j = 0; j < playerDecks.size(); j++)
-    {
-		DeckMetaData* meta = playerDecks[j];
-        if (meta)
-            meta->LoadStats();
-		sprintf(buffer, "stats/player_deck%i.txt", meta->getDeckId());
-        string deckstats = options.profileFile(buffer);
-        if (fileExists(deckstats.c_str()))
-        {
-            stats->load(deckstats.c_str());
-            int percentVictories = stats->percentVictories();
-
-			sprintf(buff2, _("You have a %i%% victory ratio with \"%s\"").c_str(), percentVictories, meta->getName().c_str());
-            scroller->Add(buff2);
-			sprintf(buff2, _("You have played %i games with \"%s\"").c_str(), meta->getGamesPlayed(), meta->getName().c_str());
-            scroller->Add(buff2);
-			totalGames += meta->getGamesPlayed();
-        }
-    }
-    if (totalGames)
-    {
-        sprintf(buff2, _("You have played a total of %i games").c_str(), totalGames);
-        scroller->Add(buff2);
-    }
-
     if (!options[Options::DIFFICULTY_MODE_UNLOCKED].number)
         scroller->Add(_("Unlock the difficult mode for more challenging duels!"));
-    if (!options[Options::MOMIR_MODE_UNLOCKED].number)
-        scroller->Add(_("Interested in playing Momir Basic? You'll have to unlock it first :)"));
-    if (!options[Options::RANDOMDECK_MODE_UNLOCKED].number)
-        scroller->Add(_("You haven't unlocked the random deck mode yet"));
-    if (!options[Options::EVILTWIN_MODE_UNLOCKED].number)
-        scroller->Add(_("You haven't unlocked the evil twin mode yet"));
+
+    for (map<string, Unlockable *>::iterator it = Unlockable::unlockables.begin(); it !=  Unlockable::unlockables.end(); ++it) {
+        Unlockable * award = it->second;
+        if (!award->isUnlocked())
+        {
+            if (award->getValue("teaser").size())
+                scroller->Add(_(award->getValue("teaser")));
+        }
+    }
+
     if (!options[Options::RANDOMDECK_MODE_UNLOCKED].number)
         scroller->Add(_("You haven't unlocked the random deck mode yet"));
     if (!options[Options::EVILTWIN_MODE_UNLOCKED].number)
@@ -248,61 +212,76 @@ void GameStateMenu::fillScroller()
     sprintf(buff2, _("You have unlocked %i expansions out of %i").c_str(), nbunlocked, setlist.size());
     scroller->Add(buff2);
 
-    PlayerData * playerdata = NEW PlayerData(MTGCollection());
-    int totalCards = playerdata->collection->totalCards();
-    if (totalCards)
-    {
-        sprintf(buff2, _("You have a total of %i cards in your collection").c_str(), totalCards);
-        scroller->Add(buff2);
-
-        int estimatedValue = playerdata->collection->totalPrice();
-        sprintf(buff2, _("The shopkeeper would buy your entire collection for around %i credits").c_str(), estimatedValue / 2);
-        scroller->Add(buff2);
-
-        sprintf(buff2, _("The cards in your collection have an average value of %i credits").c_str(), estimatedValue / totalCards);
-        scroller->Add(buff2);
-    }
-
-    sprintf(buff2, _("You currently have %i credits").c_str(), playerdata->credits);
-    SAFE_DELETE(playerdata);
-    scroller->Add(buff2);
-
     scroller->Add(_("More cards and mods at http://wololo.net/wagic"));
 
-    scroller->Add(_("These stats will be updated next time you run Wagic"));
-
     scrollerSet = 1;
     scroller->setRandom();
 }
-void GameStateMenu::resetDirectory()
-{
-    if (mDip != NULL)
-    {
-        closedir(mDip);
-        mDip = NULL;
-    }
-}
-int GameStateMenu::nextDirectory(const char * root, const char * file)
-{
-    int found = 0;
-    if (!mDip)
-    {
-        mDip = opendir(root);
+
+int GameStateMenu::gamePercentComplete() {
+    if (mPercentComplete)
+        return mPercentComplete;
+
+    int done = 0;
+    int total = 0;
+
+    total++;
+    if (options[Options::DIFFICULTY_MODE_UNLOCKED].number)
+        done++;
+
+    for (map<string, Unlockable *>::iterator it = Unlockable::unlockables.begin(); it !=  Unlockable::unlockables.end(); ++it) {
+        total++;
+        if (it->second->isUnlocked())
+            total++;
     }
 
-    while (!found && (mDit = readdir(mDip)))
+    total++;
+    if (options[Options::RANDOMDECK_MODE_UNLOCKED].number)
+        done++;
+
+    total++;
+    if (options[Options::EVILTWIN_MODE_UNLOCKED].number)
+        done++;
+
+    //Unlocked sets
+    total+= setlist.size();
+    for (int i = 0; i < setlist.size(); i++)
     {
-        sprintf(mCurrentSetFileName, "%s/%s/%s", root, mDit->d_name, file);
-        wagic::ifstream file(mCurrentSetFileName);
-        if (file)
-        {
-            sprintf(mCurrentSetName, "%s", mDit->d_name);
-            file.close();
-            found = 1;
-        }
+        if (1 == options[Options::optionSet(i)].number)
+            done++;
     }
-    if (!found)
-        resetDirectory();
+
+    //unlocked AI decks
+    int currentlyUnlocked = options[Options::AIDECKS_UNLOCKED].number;
+    int totalAIDecks = AIPlayer::getTotalAIDecks();
+    int reallyUnlocked = MIN(currentlyUnlocked, totalAIDecks);
+    total+= totalAIDecks / 10;
+    done+= reallyUnlocked / 10;
+
+    mPercentComplete = 100 * done / total;
+    return mPercentComplete;
+}
+
+int GameStateMenu::nextSetFolder(const string & root, const string & file)
+{
+    bool found = false;
+    while (!found && (mCurrentSetFolderIndex < setFolders.size()))
+    {
+        vector<string> folders;
+        folders.push_back(root);
+        folders.push_back(setFolders[mCurrentSetFolderIndex]);
+        mCurrentSetFileName = buildFilePath(folders, file);
+        
+        if (JFileSystem::GetInstance()->FileExists(mCurrentSetFileName))
+        {
+            mCurrentSetName = setFolders[mCurrentSetFolderIndex];
+            if (mCurrentSetName.length() && (mCurrentSetName[mCurrentSetName.length() - 1] == '/'))
+                mCurrentSetName.resize(mCurrentSetName.length() - 1);
+            found = true;
+        }
+        mCurrentSetFolderIndex++;
+    }
+
     return found;
 }
 
@@ -320,9 +299,8 @@ string GameStateMenu::loadRandomWallpaper()
         return wallpaper;
 
     vector<string> wallpapers;
-    wagic::ifstream file(JGE_GET_RES("graphics/wallpapers.txt").c_str());
-
-    if (!file)
+    izfstream file;
+    if (! JFileSystem::GetInstance()->openForRead(file, "graphics/wallpapers.txt"))
         return wallpaper;
 
     string s;
@@ -334,6 +312,7 @@ string GameStateMenu::loadRandomWallpaper()
             s.erase(s.size() - 1); //Handle DOS files
         wallpapers.push_back(s);
     }
+    file.close();
 
     int rnd = rand() % (wallpapers.size());
     wallpaper = wallpapers[rnd];
@@ -365,62 +344,61 @@ void GameStateMenu::setLang(int id)
 void GameStateMenu::loadLangMenu()
 {
     LOG("GameStateMenu::loadLangMenu");
-    subMenuController = NEW SimpleMenu(MENU_LANGUAGE_SELECTION, this, Fonts::MENU_FONT, 150, 60);
+    subMenuController = NEW SimpleMenu(JGE::GetInstance(), MENU_LANGUAGE_SELECTION, this, Fonts::MENU_FONT, 150, 60);
     if (!subMenuController)
         return;
-    resetDirectory();
-    if (!mDip)
-    {
-        mDip = opendir(JGE_GET_RES("lang").c_str());
-    }
 
-    while ((mDit = readdir(mDip)))
+    vector<string> langFiles = JFileSystem::GetInstance()->scanfolder("lang/");
+    for (size_t i = 0; i < langFiles.size(); ++i)
     {
-        string filename = JGE_GET_RES("lang/");
-        filename += mDit->d_name;
-        wagic::ifstream file(filename.c_str());
+        izfstream file;
+        string filePath = "lang/";
+        filePath.append(langFiles[i]);
+        if (! JFileSystem::GetInstance()->openForRead(file, filePath))
+            continue;
+
         string s;
         string lang;
-        if (file)
+
+        if (std::getline(file, s))
         {
-            if (std::getline(file, s))
-            {
-                lang = getLang(s);
-            }
-            file.close();
+            lang = getLang(s);
         }
+
+        file.close();
+
         if (lang.size())
         {
             langChoices = true;
-            string filen = mDit->d_name;
+            string filen = langFiles[i];
             langs.push_back(filen.substr(0, filen.size() - 4));
             subMenuController->Add(langs.size(), lang.c_str());
         }
     }
-    resetDirectory();
     LOG("GameStateMenu::loadLangMenu - Done");
 }
 
 void GameStateMenu::listPrimitives()
 {
     LOG("GameStateMenu::listPrimitives");
-    resetDirectory();
-    if (!mDip)
+    vector<string> primitiveFiles = JFileSystem::GetInstance()->scanfolder("sets/primitives/");
+
+    if (!primitiveFiles.size())
     {
-        mDip = opendir(JGE_GET_RES("sets/primitives/").c_str());
+        DebugTrace("GameStateMenu.cpp:WARNING:Primitives folder is missing");
+        primitivesLoadCounter = 0;
+        return;
     }
 
-    while ((mDit = readdir(mDip)))
+    for (size_t i = 0; i < primitiveFiles.size(); ++i)
     {
-        string filename = JGE_GET_RES("sets/primitives/");
-        filename += mDit->d_name;
-        wagic::ifstream file(filename.c_str());
-        if (!file)
+        string filename = "sets/primitives/";
+        filename.append(primitiveFiles[i]);
+
+        if (! JFileSystem::GetInstance()->FileExists(filename))
             continue;
-        file.close();
         primitives.push_back(filename);
     }
-    resetDirectory();
     primitivesLoadCounter = 0;
     LOG("GameStateMenu::listPrimitives - Done");
 }
@@ -429,21 +407,48 @@ void GameStateMenu::ensureMGuiController()
 {
     if (!mGuiController)
     {
-        mGuiController = NEW JGuiController(100, this);
+        mGuiController = NEW JGuiController(JGE::GetInstance(), 100, this);
         if (mGuiController)
         {
             WFont * mFont = WResourceManager::Instance()->GetWFont(Fonts::MENU_FONT);
             mFont->SetColor(ARGB(255,255,255,255));
-            mGuiController->Add(NEW MenuItem(MENUITEM_PLAY, mFont, "Play", 80, 50 + SCREEN_HEIGHT / 2, mIcons[8].get(), mIcons[9].get(),
-                            "particle1.psi", WResourceManager::Instance()->GetQuad("particles").get(), true));
-            mGuiController->Add(NEW MenuItem(MENUITEM_DECKEDITOR, mFont, "Deck Editor", 160, 50 + SCREEN_HEIGHT / 2, mIcons[2].get(),
-                            mIcons[3].get(), "particle2.psi", WResourceManager::Instance()->GetQuad("particles").get()));
-            mGuiController->Add(NEW MenuItem(MENUITEM_SHOP, mFont, "Shop", 240, 50 + SCREEN_HEIGHT / 2, mIcons[0].get(), mIcons[1].get(),
-                            "particle3.psi", WResourceManager::Instance()->GetQuad("particles").get()));
-            mGuiController->Add(NEW MenuItem(MENUITEM_OPTIONS, mFont, "Options", 320, 50 + SCREEN_HEIGHT / 2, mIcons[6].get(), mIcons[7].get(),
-                            "particle4.psi", WResourceManager::Instance()->GetQuad("particles").get()));
-            mGuiController->Add(NEW MenuItem(MENUITEM_EXIT, mFont, "Exit", 400, 50 + SCREEN_HEIGHT / 2, mIcons[4].get(), mIcons[5].get(),
-                            "particle5.psi", WResourceManager::Instance()->GetQuad("particles").get()));
+            vector<ModRulesMainMenuItem *>items = gModRules.menu.main;
+
+            int numItems = (int)items.size();
+            float startX = 80.f;
+            float totalSize = SCREEN_WIDTH_F - (2 * startX);
+            float space = 0;
+            if (numItems < 2)
+                startX = SCREEN_WIDTH_F/2;
+            else 
+                space = totalSize/(numItems - 1);
+
+            for (size_t i = 0; i < items.size(); ++i) {
+                ModRulesMainMenuItem * item = items[i];
+                int iconId = (item->mIconId - 1) * 2;
+                mGuiController->Add(NEW MenuItem(
+                    item->mActionId, 
+                    mFont, item->mDisplayName, 
+                    startX + (i * space), 40 + SCREEN_HEIGHT / 2, 
+                    mIcons[iconId].get(), mIcons[iconId + 1].get(),
+                    item->mParticleFile.c_str(), WResourceManager::Instance()->GetQuad("particles").get(),
+                    (i == 0)));
+            }
+
+            JQuadPtr jq = WResourceManager::Instance()->RetrieveTempQuad("button_shoulder.png");
+            if (!jq.get()) return;
+            jq->SetHFlip(false);
+            jq->SetColor(ARGB(abs(255),255,255,255));
+            mFont = WResourceManager::Instance()->GetWFont(Fonts::OPTION_FONT);
+            vector<ModRulesOtherMenuItem *>otherItems = gModRules.menu.other;
+            if (otherItems.size()) {
+                mGuiController->Add(NEW OtherMenuItem(
+                                       otherItems[0]->mActionId,
+                                       mFont, otherItems[0]->mDisplayName,
+                                       SCREEN_WIDTH - 64, 2,
+                                       jq.get(), jq.get(), otherItems[0]->mKey, false
+                                       ));
+            }
         }
     }
 }
@@ -471,25 +476,32 @@ void GameStateMenu::Update(float dt)
         if (primitivesLoadCounter == -1)
         {
             listPrimitives();
-            // Move translator init to GameApp::Create().
-            // Translator::GetInstance()->init();
         }
         if (primitivesLoadCounter < (int) (primitives.size()))
         {
+#ifdef _DEBUG
+            int startTime = JGEGetTime();
+#endif
             MTGCollection()->load(primitives[primitivesLoadCounter].c_str());
+#if _DEBUG
+            int endTime = JGEGetTime();
+            int elapsedTime = (endTime - startTime);
+            DebugTrace("Time elapsed while loading " << primitives[primitivesLoadCounter] << " : " << elapsedTime << " ms");
+#endif
+
             primitivesLoadCounter++;
             break;
         }
         primitivesLoadCounter = primitives.size() + 1;
         if (mReadConf)
         {
-            MTGCollection()->load(mCurrentSetFileName, mCurrentSetName);
+            MTGCollection()->load(mCurrentSetFileName.c_str(), mCurrentSetName.c_str());
         }
         else
         {
             mReadConf = 1;
         }
-        if (!nextDirectory(JGE_GET_RES("sets/").c_str(), "_cards.dat"))
+        if (!nextSetFolder("sets/", "_cards.dat"))
         {
             //Remove temporary translations
             Translator::GetInstance()->tempValues.clear();
@@ -507,10 +519,8 @@ void GameStateMenu::Update(float dt)
             splashTex = NULL;
 
             //check for deleted collection / first-timer
-            wagic::ifstream file(options.profileFile(PLAYER_COLLECTION).c_str());
-            if (file)
+            if (JFileSystem::GetInstance()->FileExists(options.profileFile(PLAYER_COLLECTION)))
             {
-                file.close();
                 currentState = MENU_STATE_MAJOR_MAINMENU;
             }
             else
@@ -521,7 +531,6 @@ void GameStateMenu::Update(float dt)
             //Reload list of unlocked sets, now that we know about the sets.
             options.reloadProfile();
             genNbCardsStr();
-            resetDirectory();
             //All major things have been loaded, resize the cache to use it as efficiently as possible
             WResourceManager::Instance()->ResetCacheLimits();
         }
@@ -530,15 +539,23 @@ void GameStateMenu::Update(float dt)
         currentState &= MENU_STATE_MAJOR_MAINMENU;
         options.reloadProfile(); //Handles building a new deck, if needed.
         break;
-    case MENU_STATE_MAJOR_MAINMENU:
-        if (!scrollerSet)
-            fillScroller();
-        ensureMGuiController();
-        if (mGuiController)
-            mGuiController->Update(dt);
-        if (mEngine->GetButtonState(JGE_BTN_NEXT)) //Hook for GameStateAward state
-            mParent->DoTransition(TRANSITION_FADE, GAME_STATE_AWARDS); //TODO: A slide transition would be nice.
-        break;
+    case MENU_STATE_MAJOR_MAINMENU: 
+        {
+            if (!scrollerSet)
+                fillScroller();
+            ensureMGuiController();
+            if (mGuiController)
+                mGuiController->Update(dt);
+
+            //Hook for Top Menu actions
+            vector<ModRulesOtherMenuItem *>items = gModRules.menu.other;
+            for (size_t i = 0; i < items.size(); ++i)
+            {
+                if (mEngine->GetButtonState(items[i]->mKey) && items[i]->getMatchingGameState())
+                     mParent->DoTransition(TRANSITION_FADE, items[i]->getMatchingGameState()); //TODO: Add the transition as a parameter in the rules file
+            }
+            break;
+         }
 #ifdef NETWORK_SUPPORT
     case MENU_STATE_NETWORK_DEFINE:
         currentState = MENU_STATE_MAJOR_SUBMENU;
@@ -589,18 +606,20 @@ void GameStateMenu::Update(float dt)
             if (!hasChosenGameType)
             {
                 currentState = MENU_STATE_MAJOR_SUBMENU;
-                subMenuController = NEW SimpleMenu(MENU_FIRST_DUEL_SUBMENU, this, Fonts::MENU_FONT, 150, 60);
+                subMenuController = NEW SimpleMenu(JGE::GetInstance(), MENU_FIRST_DUEL_SUBMENU, this, Fonts::MENU_FONT, 150, 60);
                 if (subMenuController)
                 {
-                    subMenuController->Add(SUBMENUITEM_CLASSIC, "Classic");
-                    if (options[Options::MOMIR_MODE_UNLOCKED].number)
-                        subMenuController->Add(SUBMENUITEM_MOMIR, "Momir Basic");
-                    if (options[Options::RANDOMDECK_MODE_UNLOCKED].number)
+                    for (size_t i = 0; i < Rules::RulesList.size(); ++i)
                     {
-                        subMenuController->Add(SUBMENUITEM_RANDOM1, "Random 1 Color");
-                        subMenuController->Add(SUBMENUITEM_RANDOM2, "Random 2 Colors");
+                        Rules * rules = Rules::RulesList[i];
+                        bool unlocked = rules->unlockOption == INVALID_OPTION 
+                            ? (rules->mUnlockOptionString.size() == 0 ||  options[rules->mUnlockOptionString].number !=0)
+                            :  options[rules->unlockOption].number != 0;
+                        if (!rules->hidden && (unlocked))
+                        {
+                            subMenuController->Add(SUBMENUITEM_END_OFFSET + i, rules->displayName.c_str());
+                        }
                     }
-                    subMenuController->Add(SUBMENUITEM_STORY, "Story");
                     subMenuController->Add(SUBMENUITEM_CANCEL, "Cancel");
                 }
             }
@@ -635,38 +654,6 @@ void GameStateMenu::Update(float dt)
         ;// Nothing to do.
     }
 
-    if (mEngine->GetButtonState(JGE_BTN_PREV))
-    {
-        //Reset deck of cards
-        angleMultiplier = MIN_ANGLE_MULTIPLIER;
-        yW = 55;
-    }
-
-    if (yW <= 55)
-    {
-        if (mEngine->GetButtonState(JGE_BTN_PRI))
-            angleMultiplier += STEP_ANGLE_MULTIPLIER;
-        else
-            angleMultiplier *= 0.9999f;
-        if (angleMultiplier > MAX_ANGLE_MULTIPLIER)
-            angleMultiplier = MAX_ANGLE_MULTIPLIER;
-        else if (angleMultiplier < MIN_ANGLE_MULTIPLIER)
-            angleMultiplier = MIN_ANGLE_MULTIPLIER;
-
-        if (mEngine->GetButtonState(JGE_BTN_CANCEL) && (dt != 0))
-        {
-            angleMultiplier = (cos(timeIndex) * angleMultiplier - M_PI / 3.0f - 0.1f - angleW) / dt;
-            yW = yW + 5 * dt + (yW - 45) * 5 * dt;
-        }
-        else
-            angleW = cos(timeIndex) * angleMultiplier - M_PI / 3.0f - 0.1f;
-    }
-    else
-    {
-        angleW += angleMultiplier * dt;
-        yW = yW + 5 * dt + (yW - 55) * 5 * dt;
-    }
-
     scroller->Update(dt);
     if ((currentState & MENU_STATE_MINOR) == MENU_STATE_MINOR_FADEIN)
     {
@@ -675,6 +662,44 @@ void GameStateMenu::Update(float dt)
     }
 }
 
+//Renders the "sub" menu with shoulder button links
+void GameStateMenu::RenderTopMenu()
+{
+    float leftTextPos = 10;
+    float rightTextPos = SCREEN_WIDTH - 10;
+    JRenderer * renderer = JRenderer::GetInstance();
+
+    vector<ModRulesOtherMenuItem *>items = gModRules.menu.other;
+    for (size_t i = 0; i < items.size(); ++i)
+    {
+        switch(items[i]->mKey)
+        {
+        case JGE_BTN_PREV:
+            leftTextPos += 64;
+            break;
+        case JGE_BTN_NEXT:
+            rightTextPos -= 64;
+            break;
+        default:
+            DebugTrace("not supported yet!");
+            break;
+        }
+    }
+
+    WFont * mFont = WResourceManager::Instance()->GetWFont(Fonts::MAIN_FONT);
+    mFont->SetScale(DEFAULT_MAIN_FONT_SCALE);
+    mFont->SetColor(ARGB(128,255,255,255));
+    mFont->DrawString(GAME_VERSION, rightTextPos, 5, JGETEXT_RIGHT);
+    mFont->DrawString(nbcardsStr, leftTextPos, 5);
+    renderer->FillRect(leftTextPos, 26, 104, 8, ARGB(255, 100, 90, 60));
+    renderer->FillRect(leftTextPos + 2, 28, (float)(gamePercentComplete()), 4, ARGB(255,220,200, 125));
+    char buf[512];
+    sprintf(buf, _("achieved: %i%%").c_str(), gamePercentComplete());
+    mFont->DrawString(buf, (leftTextPos + 104) / 2, 35, JGETEXT_CENTER);
+    mFont->SetScale(1.f);
+    mFont->SetColor(ARGB(255,255,255,255));
+}
+
 void GameStateMenu::Render()
 {
     if ((currentState & MENU_STATE_MINOR) == MENU_STATE_MINOR_FADEIN)
@@ -699,9 +724,9 @@ void GameStateMenu::Render()
         }
 
         char text[512];
-        if (mCurrentSetName[0])
+        if (mCurrentSetName.size())
         {
-            sprintf(text, _("LOADING SET: %s").c_str(), mCurrentSetName);
+            sprintf(text, _("LOADING SET: %s").c_str(), mCurrentSetName.c_str());
         }
         else
         {
@@ -717,7 +742,6 @@ void GameStateMenu::Render()
     }
     else
     {
-        mFont = WResourceManager::Instance()->GetWFont(Fonts::MAIN_FONT);
         PIXEL_TYPE colors[] = {
 
         ARGB(255,3,3,0), ARGB(255,8,8,0), ARGB(255,21,21,10), ARGB(255,50,50,30), };
@@ -726,38 +750,14 @@ void GameStateMenu::Render()
         if (mGuiController)
             mGuiController->Render();
 
-        mFont->SetScale(DEFAULT_MAIN_FONT_SCALE);
-        mFont->SetColor(ARGB(128,255,255,255));
-        mFont->DrawString(GAME_VERSION, SCREEN_WIDTH - 74, 5, JGETEXT_RIGHT);
-        mFont->DrawString(nbcardsStr, 10, 5);
-        mFont->SetScale(1.f);
-        mFont->SetColor(ARGB(255,255,255,255));
-
         renderer->FillRoundRect(SCREEN_WIDTH / 2 - 100, SCREEN_HEIGHT, 191, 6, 5, ARGB(100,10,5,0));
         scroller->Render();
 
         if (mBg.get())
             renderer->RenderQuad(mBg.get(), SCREEN_WIDTH / 2, 50);
 
-        JQuadPtr jq = WResourceManager::Instance()->RetrieveTempQuad("button_shoulder.png");
-        if (jq.get())
-        {
-            int alp = 255;
-            if (options.newAward())
-                alp = (int) (sin(timeIndex) * 255);
-            float olds = mFont->GetScale();
-            mFont = WResourceManager::Instance()->GetWFont(Fonts::OPTION_FONT);
-            jq->SetColor(ARGB(abs(alp),255,255,255));
-            mFont->SetColor(ARGB(abs(alp),0,0,0));
-            string s = _("Trophy Room");
-            ;
-            mFont->SetScale(1.0f);
-            mFont->SetScale(50.0f / mFont->GetStringWidth(s.c_str()));
-            renderer->RenderQuad(jq.get(), SCREEN_WIDTH - 64, 2);
-            mFont->DrawString(s, SCREEN_WIDTH - 10, 9, JGETEXT_RIGHT);
-            mFont = WResourceManager::Instance()->GetWFont(Fonts::MENU_FONT);
-            mFont->SetScale(olds);
-        }
+        RenderTopMenu();
+
     }
     if (subMenuController)
     {
@@ -786,7 +786,7 @@ void GameStateMenu::ButtonPressed(int controllerId, int controlId)
         switch (controlId)
         {
         case MENUITEM_PLAY:
-            subMenuController = NEW SimpleMenu(MENU_FIRST_DUEL_SUBMENU, this, Fonts::MENU_FONT, 150, 60);
+            subMenuController = NEW SimpleMenu(JGE::GetInstance(), MENU_FIRST_DUEL_SUBMENU, this, Fonts::MENU_FONT, 150, 60);
             if (subMenuController)
             {
                 subMenuController->Add(SUBMENUITEM_1PLAYER, "1 Player");
@@ -798,20 +798,24 @@ void GameStateMenu::ButtonPressed(int controllerId, int controlId)
                 subMenuController->Add(SUBMENUITEM_DEMO, "Demo");
                 subMenuController->Add(SUBMENUITEM_CANCEL, "Cancel");
 #ifdef TESTSUITE
-                subMenuController->Add(SUBMENUITEM_TESTSUITE, "Test Suite");
+                if (Rules::getRulesByFilename("testsuite.txt"))
+                    subMenuController->Add(SUBMENUITEM_TESTSUITE, "Test Suite");
+#endif
+
+#ifdef AI_CHANGE_TESTING
+                subMenuController->Add(SUBMENUITEM_TESTAI, "AI A/B Testing");
 #endif
                 currentState = MENU_STATE_MAJOR_SUBMENU | MENU_STATE_MINOR_NONE;
             }
             break;
+
         case MENUITEM_DECKEDITOR:
-            mParent->DoTransition(TRANSITION_FADE, GAME_STATE_DECK_VIEWER);
-            break;
         case MENUITEM_SHOP:
-            mParent->DoTransition(TRANSITION_FADE, GAME_STATE_SHOP);
-            break;
         case MENUITEM_OPTIONS:
-            mParent->DoTransition(TRANSITION_FADE, GAME_STATE_OPTIONS);
+        case MENUITEM_TROPHIES:
+            mParent->DoTransition(TRANSITION_FADE,  ModRulesMenuItem::getMatchingGameState(controlId));
             break;
+
         case MENUITEM_EXIT:
             mEngine->End();
             break;
@@ -864,6 +868,7 @@ void GameStateMenu::ButtonPressed(int controllerId, int controlId)
             currentState = MENU_STATE_MAJOR_DUEL | MENU_STATE_MINOR_SUBMENU_CLOSING;
             break;
         case SUBMENUITEM_CANCEL:
+        case kInfoMenuID: // Triangle button
             if (subMenuController != NULL)
             {
                 subMenuController->Close();
@@ -876,50 +881,34 @@ void GameStateMenu::ButtonPressed(int controllerId, int controlId)
 #endif //NETWORK_SUPPORT
             currentState = MENU_STATE_MAJOR_MAINMENU | MENU_STATE_MINOR_SUBMENU_CLOSING;
             break;
-
-        case SUBMENUITEM_CLASSIC:
-            this->hasChosenGameType = true;
-            mParent->gameType = GAME_TYPE_CLASSIC;
+#ifdef AI_CHANGE_TESTING
+        case SUBMENUITEM_TESTAI:
+             options[Options::AIDECKS_UNLOCKED].number = 5000; //hack to force-test all decks
+            mParent->players[0] = PLAYER_TYPE_CPU_TEST;
+            mParent->players[1] = PLAYER_TYPE_CPU_TEST;
+            mParent->gameType = GAME_TYPE_DEMO;
             subMenuController->Close();
             currentState = MENU_STATE_MAJOR_DUEL | MENU_STATE_MINOR_SUBMENU_CLOSING;
             break;
-
-        case SUBMENUITEM_MOMIR:
-            this->hasChosenGameType = true;
-            mParent->gameType = GAME_TYPE_MOMIR;
-            subMenuController->Close();
-            currentState = MENU_STATE_MAJOR_DUEL | MENU_STATE_MINOR_SUBMENU_CLOSING;
-            break;
-
-        case SUBMENUITEM_RANDOM1:
-            this->hasChosenGameType = true;
-            mParent->gameType = GAME_TYPE_RANDOM1;
-            subMenuController->Close();
-            currentState = MENU_STATE_MAJOR_DUEL | MENU_STATE_MINOR_SUBMENU_CLOSING;
-            break;
-
-        case SUBMENUITEM_RANDOM2:
-            this->hasChosenGameType = true;
-            mParent->gameType = GAME_TYPE_RANDOM2;
-            subMenuController->Close();
-            currentState = MENU_STATE_MAJOR_DUEL | MENU_STATE_MINOR_SUBMENU_CLOSING;
-            break;
-
-        case SUBMENUITEM_STORY:
-            this->hasChosenGameType = true;
-            mParent->gameType = GAME_TYPE_STORY;
-            subMenuController->Close();
-            currentState = MENU_STATE_MAJOR_DUEL | MENU_STATE_MINOR_SUBMENU_CLOSING;
-            break;
-
+#endif
 #ifdef TESTSUITE
             case SUBMENUITEM_TESTSUITE:
+            mParent->rules = Rules::getRulesByFilename("testsuite.txt");
+            this->hasChosenGameType = true;
+            mParent->gameType = GAME_TYPE_CLASSIC;
             mParent->players[0] = PLAYER_TYPE_TESTSUITE;
             mParent->players[1] = PLAYER_TYPE_TESTSUITE;
             subMenuController->Close();
             currentState = MENU_STATE_MAJOR_DUEL | MENU_STATE_MINOR_SUBMENU_CLOSING;
             break;
 #endif
+            default: //Game modes
+            this->hasChosenGameType = true;
+            mParent->rules = Rules::RulesList[controlId - SUBMENUITEM_END_OFFSET];
+            mParent->gameType = (mParent->rules->gamemode); //TODO can we get rid of gameType in the long run, since it is also stored in the rules object ?
+            subMenuController->Close();
+            currentState = MENU_STATE_MAJOR_DUEL | MENU_STATE_MINOR_SUBMENU_CLOSING;
+            break;
         }
         break;
     }
@@ -940,13 +929,8 @@ ostream& GameStateMenu::toString(ostream& out) const
                  << " ; currentState : " << currentState
                  << " ; mVolume : " << mVolume
                  << " ; nbcardsStr : " << nbcardsStr
-                 << " ; mDip : " << mDip
-                 << " ; mDit : " << mDit
                  << " ; mCurrentSetName : " << mCurrentSetName
                  << " ; mCurrentSetFileName : " << mCurrentSetFileName
                  << " ; mReadConf : " << mReadConf
-                 << " ; timeIndex : " << timeIndex
-                 << " ; angleMultiplier : " << angleMultiplier
-                 << " ; angleW : " << angleW
-                 << " ; yW : " << yW;
+                 << " ; timeIndex : " << timeIndex;
 }
