@@ -1,10 +1,11 @@
 #include "PrecompiledHeader.h"
 
-#include "CardSelectorSingleton.h"
+#include "CardSelector.h"
 #include "GameApp.h"
 #include "GuiPlay.h"
 #include "Subtypes.h"
 #include "Trash.h"
+#include "ModRules.h"
 
 #define CARD_WIDTH (31)
 
@@ -121,15 +122,13 @@ void GuiPlay::BattleField::reset(float x, float y)
 }
 void GuiPlay::BattleField::EnstackAttacker(CardView* card)
 {
-    GameObserver* game = GameObserver::GetInstance();
     card->x = currentAttacker * (HORZWIDTH - 20) / (attackers + 1);
-    card->y = baseY + (game->players[0] == card->card->controller() ? 20 + y : -20 - y);
+    card->y = baseY + (card->card->getObserver()->players[0] == card->card->controller() ? 20 + y : -20 - y);
     ++currentAttacker;
     //  JRenderer::GetInstance()->RenderQuad(WResourceManager::Instance()->GetQuad("BattleIcon"), card->actX, card->actY, 0, 0.5 + 0.1 * sinf(JGE::GetInstance()->GetTime()), 0.5 + 0.1 * sinf(JGE::GetInstance()->GetTime()));
 }
 void GuiPlay::BattleField::EnstackBlocker(CardView* card)
 {
-    GameObserver* game = GameObserver::GetInstance();
     MTGCardInstance * c = card->card;
     if (!c)
         return;
@@ -139,7 +138,7 @@ void GuiPlay::BattleField::EnstackBlocker(CardView* card)
         offset = c->defenser->getDefenserRank(c);
         card->x = c->defenser->view->x + 5 * offset;
     }
-    card->y = baseY + (game->players[0] == card->card->controller() ? 20 + y + 6 * offset : -20 - y + 6 * offset);
+    card->y = baseY + (card->card->getObserver()->players[0] == card->card->controller() ? 20 + y + 6 * offset : -20 - y + 6 * offset);
 }
 void GuiPlay::BattleField::Update(float dt)
 {
@@ -164,7 +163,7 @@ void GuiPlay::BattleField::Render()
 }
 
 GuiPlay::GuiPlay(GameObserver* game) :
-    game(game)
+    GuiLayer(game)
 {
     end_spells = cards.end();
 }
@@ -183,8 +182,8 @@ bool isSpell(CardView* c)
 }
 void GuiPlay::Replace()
 {
-    unsigned opponentSpellsN = 0, selfSpellsN = 0, opponentLandsN = 0, opponentCreaturesN = 0, battleFieldAttackersN = 0,
-                    battleFieldBlockersN = 0, selfCreaturesN = 0, selfLandsN = 0, selfPlaneswalkern = 0,opponentPlaneswalkern = 0;
+    unsigned opponentSpellsN = 0, selfSpellsN = 0, opponentLandsN = 0, opponentCreaturesN = 0, 
+            battleFieldAttackersN = 0, battleFieldBlockersN = 0, selfCreaturesN = 0, selfLandsN = 0;
 
     end_spells = stable_partition(cards.begin(), cards.end(), &isSpell);
 
@@ -193,7 +192,7 @@ void GuiPlay::Replace()
         {
             if(!(*it)->card->hasSubtype(Subtypes::TYPE_AURA) && !(*it)->card->hasType(Subtypes::TYPE_PLANESWALKER))
             {
-                if (game->players[0] == (*it)->card->controller())
+                if (observer->players[0] == (*it)->card->controller())
                     ++selfSpellsN;
                 else
                     ++opponentSpellsN;
@@ -207,14 +206,14 @@ void GuiPlay::Replace()
                 ++battleFieldAttackersN;
             else if ((*it)->card->isDefenser())
                 ++battleFieldBlockersN;
-            else if (game->players[0] == (*it)->card->controller())
+            else if (observer->players[0] == (*it)->card->controller())
                 ++selfCreaturesN;
             else
                 ++opponentCreaturesN;
         }
         else if ((*it)->card->isLand() || (*it)->card->hasType(Subtypes::TYPE_PLANESWALKER))
         {
-            if (game->players[0] == (*it)->card->controller())
+            if (observer->players[0] == (*it)->card->controller())
                 ++selfLandsN;
             else
                 ++opponentLandsN;
@@ -229,19 +228,19 @@ void GuiPlay::Replace()
         {
             if(!(*it)->card->hasSubtype(Subtypes::TYPE_AURA) && !(*it)->card->hasType(Subtypes::TYPE_PLANESWALKER))
             {
-                if (game->players[0] == (*it)->card->controller())
+                if (observer->players[0] == (*it)->card->controller())
                     selfSpells.Enstack(*it);
                 else
                     opponentSpells.Enstack(*it);
             }
         }
     float x = 24 + opponentSpells.nextX();
-    //seperated the varible X into 2 different varibles. There are 2 players here!!
-    //we should not be using a single varible to determine the positioning of cards!!
+    //seperated the variable X into 2 different variables. There are 2 players here!!
+    //we should not be using a single variable to determine the positioning of cards!!
     float myx = 24 + selfSpells.nextX();
     opponentLands.reset(opponentLandsN,x, 50);
     opponentCreatures.reset(opponentCreaturesN, x, 95);
-    battleField.reset(x, 145);//what does this varible do? i can comment it out with no reprocussions...is this being double handled?
+    battleField.reset(x, 145);//what does this variable do? I can comment it out with no repercussions...is this being double handled?
     selfCreatures.reset(selfCreaturesN, myx, 195);
     selfLands.reset(selfLandsN, myx, 240);
 
@@ -253,14 +252,14 @@ void GuiPlay::Replace()
                 battleField.EnstackAttacker(*it);
             else if ((*it)->card->isDefenser())
                 battleField.EnstackBlocker(*it);
-            else if (game->players[0] == (*it)->card->controller())
+            else if (observer->players[0] == (*it)->card->controller())
                 selfCreatures.Enstack(*it);
             else
                 opponentCreatures.Enstack(*it);
         }
         else if ((*it)->card->isLand())
         {
-            if (game->players[0] == (*it)->card->controller())
+            if (observer->players[0] == (*it)->card->controller())
                 selfLands.Enstack(*it);
             else
                 opponentLands.Enstack(*it);
@@ -272,7 +271,7 @@ void GuiPlay::Replace()
     {
         if ((*it)->card->hasType(Subtypes::TYPE_PLANESWALKER))
         {
-            if (game->players[0] == (*it)->card->controller())
+            if (observer->players[0] == (*it)->card->controller())
                 selfLands.Enstack(*it);
             else
                 opponentLands.Enstack(*it);
@@ -287,14 +286,14 @@ void GuiPlay::Render()
     for (iterator it = cards.begin(); it != cards.end(); ++it)
         if ((*it)->card->isLand())
         {
-            if (game->players[0] == (*it)->card->controller())
+            if (observer->players[0] == (*it)->card->controller())
                 selfLands.Render(*it, cards.begin(), end_spells);
             else
                 opponentLands.Render(*it, cards.begin(), end_spells);
         }
         else if ((*it)->card->isCreature())
         {
-            if (game->players[0] == (*it)->card->controller())
+            if (observer->players[0] == (*it)->card->controller())
                 selfCreatures.Render(*it, cards.begin(), end_spells);
             else
                 opponentCreatures.Render(*it, cards.begin(), end_spells);
@@ -303,7 +302,7 @@ void GuiPlay::Render()
         {
             if (!(*it)->card->target)
             {
-                if (game->players[0] == (*it)->card->controller())
+                if (observer->players[0] == (*it)->card->controller())
                     selfSpells.Render(*it, cards.begin(), end_spells);
                 else
                     opponentSpells.Render(*it, cards.begin(), end_spells);
@@ -313,7 +312,7 @@ void GuiPlay::Render()
         {
             if (!(*it)->card->target)
             {
-                if (game->players[0] == (*it)->card->controller())
+                if (observer->players[0] == (*it)->card->controller())
                     selfPlaneswalker.Render(*it, cards.begin(), end_spells);
                 else
                     opponentPlaneswalker.Render(*it, cards.begin(), end_spells);
@@ -332,7 +331,7 @@ int GuiPlay::receiveEventPlus(WEvent * e)
 {
     if (WEventZoneChange *event = dynamic_cast<WEventZoneChange*>(e))
     {
-        if ((game->players[0]->inPlay() == event->to) || (game->players[1]->inPlay() == event->to))
+        if ((observer->players[0]->inPlay() == event->to) || (observer->players[1]->inPlay() == event->to))
         {
             CardView * card;
             if (event->card->view)
@@ -346,13 +345,18 @@ int GuiPlay::receiveEventPlus(WEvent * e)
             else
                 card = NEW CardView(CardView::playZone, event->card, 0, 0);
             cards.push_back(card);
-            card->t = event->card->isTapped() ? M_PI / 2 : 0;
+
+            if (event->card->isTapped())
+                gModRules.cards.activateEffect->doEffect(card);
+            else
+                gModRules.cards.activateEffect->undoEffect(card);
+
             card->alpha = 255;
 
             // Make sure that the card is repositioned before adding it to the CardSelector, as
             // the card's position is a cue for certain CardSelector variants as to what zone the card is placed in
             Replace();
-            CardSelectorSingleton::Instance()->Add(card);
+            observer->getCardSelector()->Add(card);
             return 1;
         }
     }
@@ -371,10 +375,20 @@ int GuiPlay::receiveEventPlus(WEvent * e)
     else if (WEventCardTap* event = dynamic_cast<WEventCardTap*>(e))
     {
         if (CardView* cv = dynamic_cast<CardView*>(event->card->view))
-            cv->t = event->after ? M_PI / 2 : 0;
+        {
+            if (event->after)
+                gModRules.cards.activateEffect->doEffect(cv);
+            else
+                gModRules.cards.activateEffect->undoEffect(cv);
+            //cv->t = event->after ? M_PI / 2 : 0;
+        }
         else if (event->card->view != NULL)
         {
-            event->card->view->actT = event->after ? M_PI / 2 : 0;
+            if (event->after)
+                gModRules.cards.activateEffect->doEffect(event->card->view);
+            else
+                gModRules.cards.activateEffect->undoEffect(event->card->view);
+            //event->card->view->actT = event->after ? M_PI / 2 : 0;
         }
         else
         {
@@ -385,7 +399,7 @@ int GuiPlay::receiveEventPlus(WEvent * e)
     }
     else if (WEventPhaseChange *event = dynamic_cast<WEventPhaseChange*>(e))
     {
-        if (Constants::MTG_PHASE_COMBATEND == event->to->id)
+        if (MTG_PHASE_COMBATEND == event->to->id)
             battleField.colorFlow = -1;
     }
     else if (dynamic_cast<WEventCardChangeType*> (e))
@@ -397,7 +411,7 @@ int GuiPlay::receiveEventMinus(WEvent * e)
 {
     if (WEventZoneChange *event = dynamic_cast<WEventZoneChange*>(e))
     {
-        if ((game->players[0]->inPlay() == event->from) || (game->players[1]->inPlay() == event->from))
+        if ((observer->players[0]->inPlay() == event->from) || (observer->players[1]->inPlay() == event->from))
             for (iterator it = cards.begin(); it != cards.end(); ++it)
                 if (event->card->previous == (*it)->card || event->card == (*it)->card)
                 {
@@ -406,9 +420,9 @@ int GuiPlay::receiveEventMinus(WEvent * e)
                     else if (event->card->attacker)
                         battleField.removeAttacker(event->card);
                     CardView* cv = *it;
-                    CardSelectorSingleton::Instance()->Remove(cv);
+                    observer->getCardSelector()->Remove(cv);
                     cards.erase(it);
-                    trash(cv);
+                    observer->mTrash->trash(cv);
                     Replace();
                     return 1;
                 }
