@@ -14,19 +14,23 @@
 
 using namespace std;
 
+SUPPORT_OBJECT_ANALYTICS(MTGCardInstance)
+
 MTGCardInstance MTGCardInstance::AnyCard = MTGCardInstance();
 MTGCardInstance MTGCardInstance::NoCard = MTGCardInstance();
 
-MTGCardInstance MTGCardInstance::ExtraRules[] = { MTGCardInstance(), MTGCardInstance() };
-
 MTGCardInstance::MTGCardInstance() :
-    CardPrimitive(), MTGCard(), Damageable(0), view(NULL)
+    CardPrimitive(), MTGCard(), Damageable(0, 0), view(NULL)
 {
     initMTGCI();
 }
 MTGCardInstance::MTGCardInstance(MTGCard * card, MTGPlayerCards * arg_belongs_to) :
-    CardPrimitive(card->data), MTGCard(card), Damageable(card->data->getToughness()), view(NULL)
+    CardPrimitive(card->data), MTGCard(card), Damageable(0, card->data->getToughness()), view(NULL)
 {
+    if(arg_belongs_to)
+      if(arg_belongs_to->owner)
+        observer = arg_belongs_to->owner->getObserver();
+
     initMTGCI();
     model = card;
     attacker = 0;
@@ -51,22 +55,18 @@ void MTGCardInstance::copy(MTGCardInstance * card)
 {
     MTGCard * source = card->model;
     CardPrimitive * data = source->data;
-    for (map<int, int>::const_iterator it = data->basicAbilities.begin(); it != data->basicAbilities.end(); ++it)
-    {
-        int i = it->first;
-        basicAbilities[i] = data->basicAbilities[i];
-    }
+
+    basicAbilities = card->basicAbilities;
     for (size_t i = 0; i < data->types.size(); i++)
     {
         types.push_back(data->types[i]);
     }
-    for (int i = 0; i < Constants::MTG_NB_COLORS; i++)
-    {
-        colors[i] = data->colors[i];
-    }
+
+    colors = data->colors;
+
     manaCost.copy(data->getManaCost());
 
-    text = data->text;
+    setText(""); //The text is retrieved from the data anyways
     setName(data->name);
 
     power = data->power;
@@ -79,13 +79,15 @@ void MTGCardInstance::copy(MTGCardInstance * card)
 
     //Now this is dirty...
     int backupid = mtgid;
+    int castMethodBackUP = this->castMethod;
     mtgid = source->getId();
-    Spell * spell = NEW Spell(this);
-    AbilityFactory af;
-    GameObserver * g = GameObserver::GetInstance();
-    af.addAbilities(g->mLayers->actionLayer()->getMaxId(), spell);
+    Spell * spell = NEW Spell(observer, this);
+    observer = card->observer;
+    AbilityFactory af(observer);
+    af.addAbilities(observer->mLayers->actionLayer()->getMaxId(), spell);
     delete spell;
     mtgid = backupid;
+    castMethod = castMethodBackUP;
 }
 
 MTGCardInstance::~MTGCardInstance()
@@ -104,7 +106,6 @@ int MTGCardInstance::init()
     CardPrimitive::init();
     data = this;
     X = 0;
-    XX = 0;
     return 1;
 }
 
@@ -121,11 +122,6 @@ void MTGCardInstance::initMTGCI()
     frozen = 0;
     fresh = 0;
     isMultiColored = 0;
-    isBlackAndWhite = 0;
-    isRedAndBlue = 0;
-    isBlackAndGreen = 0;
-    isBlueAndGreen = 0;
-    isRedAndWhite = 0;
     isLeveler = 0;
     enchanted = false;
     CDenchanted = 0;
@@ -135,6 +131,7 @@ void MTGCardInstance::initMTGCI()
     morphed = false;
     turningOver = false;
     isMorphed = false;
+    isFlipped = false;
     isPhased = false;
     phasedTurn = -1;
     didattacked = 0;
@@ -146,8 +143,11 @@ void MTGCardInstance::initMTGCI()
     damageToOpponent = false;
     damageToController = false;
     wasDealtDamage = false;
+    isDualWielding = false;
     suspended = false;
     castMethod = Constants::NOT_CAST;
+    mPropertiesChangedSinceLastUpdate = false;
+    kicked = 0;
 
 
     for (int i = 0; i < ManaCost::MANA_PAID_WITH_RETRACE +1; i++)
@@ -173,54 +173,36 @@ void MTGCardInstance::initMTGCI()
     currentZone = NULL;
     data = this; //an MTGCardInstance point to itself for data, allows to update it without killing the underlying database item
 
-    if (basicAbilities[Constants::CHANGELING])
-    {//if the card is a changeling.
-        for (int i = Subtypes::LAST_TYPE + 1;; i++)
+    if (basicAbilities[(int)Constants::CHANGELING])
+    {//if the card is a changeling, it gains all creature subtypes
+        const vector<string> values = Subtypes::subtypesList->getValuesById();
+        for (size_t i = 0; i < values.size(); ++i)
         {
-            string s = Subtypes::subtypesList->find(i);
-            if (!hasSubtype(i))
-            {
-                if (s == "")
-                    break;
-                if (s.find(" ") != string::npos)
-                    continue;
-                if (s == "Nothing" || s == "Swamp" || s == "Plains" || s == "Mountain" || s == "Forest" ||
-                    s == "Island" || s == "Shrine" || s == "Basic" || s == "Colony" || s == "Desert" ||
-                    s == "Dismiss" || s == "Equipment" || s == "Everglades" || s == "Grasslands" || s == "Lair" ||
-                    s == "Level" || s == "Levelup" || s == "Mine" || s == "Oasis" || s == "World" || s == "Aura"
-                    || s == "Land"|| s == "Legendary" || s == "Token" || s == "Planeswalker")
-                    continue;
-                addType(i);
-            }
+            if (!Subtypes::subtypesList->isSubtypeOfType(i,Subtypes::TYPE_CREATURE))
+                continue;
+
+            //Don' want to send any event to the gameObserver inside of initMCGI, so calling the parent addType method instead of mine
+            CardPrimitive::addType(i);
         }
     }
+
     int colored = 0;
-    
+
     for (int i = Constants::MTG_COLOR_GREEN; i <= Constants::MTG_COLOR_WHITE; ++i)
     {
         if (this->hasColor(i))
             ++colored;
     }
-    if(colored > 1)
+    if (colored > 1)
     {
         isMultiColored = 1;
     }
-    
-    if(this->hasColor(Constants::MTG_COLOR_WHITE) && this->hasColor(Constants::MTG_COLOR_BLACK))
-        isBlackAndWhite = 1;
-    if(this->hasColor(Constants::MTG_COLOR_RED) && this->hasColor(Constants::MTG_COLOR_BLUE))
-        isRedAndBlue = 1;
-    if(this->hasColor(Constants::MTG_COLOR_GREEN) && this->hasColor(Constants::MTG_COLOR_BLACK))
-        isBlackAndGreen = 1;
-    if(this->hasColor(Constants::MTG_COLOR_BLUE) && this->hasColor(Constants::MTG_COLOR_GREEN))
-        isBlueAndGreen = 1;
-    if(this->hasColor(Constants::MTG_COLOR_RED) && this->hasColor(Constants::MTG_COLOR_WHITE))
-        isRedAndWhite = 1;
- if(previous && previous->morphed && !turningOver)
- {
- morphed = true;
- isMorphed = true;
- }
+
+    if(previous && previous->morphed && !turningOver)
+    {
+        morphed = true;
+        isMorphed = true;
+    }
 }
 
 const string MTGCardInstance::getDisplayName() const
@@ -232,10 +214,19 @@ void MTGCardInstance::addType(int type)
 {
     bool before = hasType(type);
     CardPrimitive::addType(type);
+
+    if (!before)
+        mPropertiesChangedSinceLastUpdate = true;
+
+    // If the card name is not set, set it to the type.
+    //This is a hack for "transform", used in combination with "losesubtypes" on Basic Lands
+    //See removeType below
+    if (!name.length())
+        setName(Subtypes::subtypesList->find(type));
+
     WEvent * e = NEW WEventCardChangeType(this, type, before, true);
-    GameObserver * go = GameObserver::GetInstance();
-    if (go)
-        go->receiveEvent(e);
+    if (observer)
+        observer->receiveEvent(e);
     else
         SAFE_DELETE(e);
 }
@@ -266,18 +257,26 @@ int MTGCardInstance::removeType(int id, int removeAll)
     bool before = hasType(id);
     int result = CardPrimitive::removeType(id, removeAll);
     bool after = hasType(id);
+    if (before != after)
+    {
+        mPropertiesChangedSinceLastUpdate = true;
+        // Basic lands have the same name as their subtypes, and TargetChoosers don't make a distinction between name and type,
+        // so if we remove a subtype "Forest", we also need to remove its name.
+        //This means the card might lose its name, but usually when we force remove a type, we add another one just after that.
+        //see "AddType" above which force sets a name if necessary
+        if (name.compare(Subtypes::subtypesList->find(id)) == 0)
+            setName("");
+    }
     WEvent * e = NEW WEventCardChangeType(this, id, before, after);
-    GameObserver * go = GameObserver::GetInstance();
-    if (go)
-        go->receiveEvent(e);
+    if (observer)
+        observer->receiveEvent(e);
     else
         SAFE_DELETE(e);
     return result;
 }
 
-int MTGCardInstance::isInPlay()
+int MTGCardInstance::isInPlay(GameObserver* game)
 {
-    GameObserver * game = GameObserver::GetInstance();
     for (int i = 0; i < 2; i++)
     {
         MTGGameZone * zone = game->players[i]->game->inPlay;
@@ -294,7 +293,7 @@ int MTGCardInstance::afterDamage()
     doDamageTest = 0;
     if (!isCreature())
         return 0;
-    if (life <= 0 && isInPlay())
+    if (life <= 0 && isInPlay(observer))
     {
         return destroy();
     }
@@ -304,12 +303,12 @@ int MTGCardInstance::afterDamage()
 int MTGCardInstance::bury()
 {
     Player * p = controller();
-    if (basicAbilities[Constants::EXILEDEATH])
+    if (basicAbilities[(int)Constants::EXILEDEATH])
     {
         p->game->putInZone(this, p->game->inPlay, owner->game->exile);
         return 1;
     }
-    if (!basicAbilities[Constants::INDESTRUCTIBLE])
+    if (!basicAbilities[(int)Constants::INDESTRUCTIBLE])
     {
         p->game->putInZone(this, p->game->inPlay, owner->game->graveyard);
         return 1;
@@ -333,21 +332,28 @@ int MTGCardInstance::has(int basicAbility)
     return basicAbilities[basicAbility];
 }
 
+ManaCost* MTGCardInstance::getReducedManaCost()
+{
+    return &reducedCost;
+}
+ManaCost* MTGCardInstance::getIncreasedManaCost()
+{
+    return &increasedCost;
+}
+
 //sets card as attacked and sends events
 void MTGCardInstance::eventattacked()
 {
     didattacked = 1;
     WEvent * e = NEW WEventCardAttacked(this);
-    GameObserver * game = GameObserver::GetInstance();
-    game->receiveEvent(e);
+    observer->receiveEvent(e);
 }
 
 //sets card as attacked alone and sends events
 void MTGCardInstance::eventattackedAlone()
 {
     WEvent * e = NEW WEventCardAttackedAlone(this);
-    GameObserver * game = GameObserver::GetInstance();
-    game->receiveEvent(e);
+    observer->receiveEvent(e);
 }
 
 //sets card as attacked and sends events
@@ -355,26 +361,23 @@ void MTGCardInstance::eventattackednotblocked()
 {
     didattacked = 1;
     WEvent * e = NEW WEventCardAttackedNotBlocked(this);
-    GameObserver * game = GameObserver::GetInstance();
-    game->receiveEvent(e);
+    observer->receiveEvent(e);
 }
 
 //sets card as attacked and sends events
-void MTGCardInstance::eventattackedblocked()
+void MTGCardInstance::eventattackedblocked(MTGCardInstance * opponent)
 {
     didattacked = 1;
-    WEvent * e = NEW WEventCardAttackedBlocked(this);
-    GameObserver * game = GameObserver::GetInstance();
-    game->receiveEvent(e);
+    WEvent * e = NEW WEventCardAttackedBlocked(this,opponent);
+    observer->receiveEvent(e);
 }
 
 //sets card as blocking and sends events
-void MTGCardInstance::eventblocked()
+void MTGCardInstance::eventblocked(MTGCardInstance * opponent)
 {
     didblocked = 1;
-    WEvent * e = NEW WEventCardBlocked(this);
-    GameObserver * game = GameObserver::GetInstance();
-    game->receiveEvent(e);
+    WEvent * e = NEW WEventCardBlocked(this,opponent);
+    observer->receiveEvent(e);
 }
 
 //Taps the card
@@ -384,8 +387,7 @@ void MTGCardInstance::tap()
         return;
     tapped = 1;
     WEvent * e = NEW WEventCardTap(this, 0, 1);
-    GameObserver * game = GameObserver::GetInstance();
-    game->receiveEvent(e);
+    observer->receiveEvent(e);
 }
 
 void MTGCardInstance::untap()
@@ -394,8 +396,7 @@ void MTGCardInstance::untap()
         return;
     tapped = 0;
     WEvent * e = NEW WEventCardTap(this, 1, 0);
-    GameObserver * game = GameObserver::GetInstance();
-    game->receiveEvent(e);
+    observer->receiveEvent(e);
 }
 
 void MTGCardInstance::setUntapping()
@@ -439,10 +440,13 @@ int MTGCardInstance::triggerRegenerate()
         return 0;
     regenerateTokens--;
     tap();
-    life = toughness;
-    initAttackersDefensers();
-    if (life < 1)
-        return 0; //regeneration didn't work (wither ?)
+    if(isCreature())
+    {
+        life = toughness;
+        initAttackersDefensers();
+        if (life < 1)
+            return 0; //regeneration didn't work (wither ?)
+    }
     return 1;
 }
 
@@ -452,19 +456,17 @@ int MTGCardInstance::initAttackersDefensers()
     setDefenser(NULL);
     banding = NULL;
     blockers.clear();
-    blocked = false;
+    blocked = 0;
     didattacked = 0;
     didblocked = 0;
     return 1;
 }
 
-//Function to call to remove all damages, etc to a card (generally at the end of the turn)
+//Function to cleanup flags on a card (generally at the end of the turn)
 int MTGCardInstance::cleanup()
 {
     initAttackersDefensers();
-    life = toughness;
-    GameObserver * game = GameObserver::GetInstance();
-    if (!game || game->currentPlayer == controller())
+    if (!observer || observer->currentPlayer == controller())
     {
         summoningSickness = 0;
     }
@@ -481,11 +483,11 @@ int MTGCardInstance::cleanup()
 
 int MTGCardInstance::stillInUse()
 {
-    GameObserver * game = GameObserver::GetInstance();
-    if (game->mLayers->actionLayer()->stillInUse(this))
+    if (observer->mLayers->actionLayer()->stillInUse(this))
         return 1;
     if (!previous)
         return 0;
+
     return previous->stillInUse();
 }
 
@@ -500,7 +502,7 @@ int MTGCardInstance::hasSummoningSickness()
 {
     if (!summoningSickness)
         return 0;
-    if (basicAbilities[Constants::HASTE])
+    if (basicAbilities[(int)Constants::HASTE])
         return 0;
     if (!isCreature())
         return 0;
@@ -528,11 +530,11 @@ int MTGCardInstance::canAttack()
         return 0;
     if (hasSummoningSickness())
         return 0;
-    if ((basicAbilities[Constants::DEFENSER] || basicAbilities[Constants::CANTATTACK]) && !basicAbilities[Constants::CANATTACK])
+    if ((basicAbilities[(int)Constants::DEFENSER] || basicAbilities[(int)Constants::CANTATTACK]) && !basicAbilities[(int)Constants::CANATTACK])
         return 0;
     if (!isCreature())
         return 0;
-    if (!isInPlay())
+    if (!isInPlay(observer))
         return 0;
     return 1;
 }
@@ -557,11 +559,11 @@ int MTGCardInstance::canBlock()
 {
     if (tapped)
         return 0;
-    if (basicAbilities[Constants::CANTBLOCK])
+    if (basicAbilities[(int)Constants::CANTBLOCK])
         return 0;
     if (!isCreature())
         return 0;
-    if (!isInPlay())
+    if (!isInPlay(observer))
         return 0;
     return 1;
 }
@@ -579,19 +581,19 @@ int MTGCardInstance::canBlock(MTGCardInstance * opponent)
         return 0;
     if (opponent->cantBeBlockedBy(this))
         return 0;
-    if (opponent->basicAbilities[Constants::UNBLOCKABLE])
+    if (opponent->basicAbilities[(int)Constants::UNBLOCKABLE])
         return 0;
-    if (opponent->basicAbilities[Constants::ONEBLOCKER] && opponent->blocked)
+    if (opponent->basicAbilities[(int)Constants::ONEBLOCKER] && opponent->blocked)
         return 0;
-    if(opponent->basicAbilities[Constants::STRONG] && power < opponent->power)
+    if(opponent->basicAbilities[(int)Constants::STRONG] && power < opponent->power)
         return 0;
-    if(this->basicAbilities[Constants::WEAK] && power < opponent->power)
+    if(this->basicAbilities[(int)Constants::WEAK] && power < opponent->power)
         return 0;
-    if (opponent->basicAbilities[Constants::FEAR] && !(this->hasType(Subtypes::TYPE_ARTIFACT) || this->hasColor(Constants::MTG_COLOR_BLACK)))
+    if (opponent->basicAbilities[(int)Constants::FEAR] && !(this->hasType(Subtypes::TYPE_ARTIFACT) || this->hasColor(Constants::MTG_COLOR_BLACK)))
         return 0;
 
     //intimidate
-    if (opponent->basicAbilities[Constants::INTIMIDATE] && !(this->hasType(Subtypes::TYPE_ARTIFACT)))
+    if (opponent->basicAbilities[(int)Constants::INTIMIDATE] && !(this->hasType(Subtypes::TYPE_ARTIFACT)))
     {
         int canblock = 0;
         for (int i = Constants::MTG_COLOR_GREEN; i <= Constants::MTG_COLOR_WHITE; ++i)
@@ -606,47 +608,47 @@ int MTGCardInstance::canBlock(MTGCardInstance * opponent)
             return 0;
     }
 
-    if (opponent->basicAbilities[Constants::FLYING] && !(basicAbilities[Constants::FLYING] || basicAbilities[Constants::REACH]))
+    if (opponent->basicAbilities[(int)Constants::FLYING] && !(basicAbilities[(int)Constants::FLYING] || basicAbilities[(int)Constants::REACH]))
         return 0;
     //Can block only creatures with flying if has cloud
-    if (basicAbilities[Constants::CLOUD] && !(opponent->basicAbilities[Constants::FLYING]))
+    if (basicAbilities[(int)Constants::CLOUD] && !(opponent->basicAbilities[(int)Constants::FLYING]))
         return 0;
     // If opponent has shadow and a creature does not have either shadow or reachshadow it cannot be blocked
-    if (opponent->basicAbilities[Constants::SHADOW] && !(basicAbilities[Constants::SHADOW]
-                    || basicAbilities[Constants::REACHSHADOW]))
+    if (opponent->basicAbilities[(int)Constants::SHADOW] && !(basicAbilities[(int)Constants::SHADOW]
+                    || basicAbilities[(int)Constants::REACHSHADOW]))
         return 0;
     // If opponent does not have shadow and a creature has shadow it cannot be blocked
-    if (!opponent->basicAbilities[Constants::SHADOW] && basicAbilities[Constants::SHADOW])
+    if (!opponent->basicAbilities[(int)Constants::SHADOW] && basicAbilities[(int)Constants::SHADOW])
         return 0;
-    if (opponent->basicAbilities[Constants::HORSEMANSHIP] && !basicAbilities[Constants::HORSEMANSHIP])
+    if (opponent->basicAbilities[(int)Constants::HORSEMANSHIP] && !basicAbilities[(int)Constants::HORSEMANSHIP])
         return 0;
-    if (opponent->basicAbilities[Constants::SWAMPWALK] && controller()->game->inPlay->hasType("swamp"))
+    if (opponent->basicAbilities[(int)Constants::SWAMPWALK] && controller()->game->inPlay->hasType("swamp"))
         return 0;
-    if (opponent->basicAbilities[Constants::FORESTWALK] && controller()->game->inPlay->hasType("forest"))
+    if (opponent->basicAbilities[(int)Constants::FORESTWALK] && controller()->game->inPlay->hasType("forest"))
         return 0;
-    if (opponent->basicAbilities[Constants::ISLANDWALK] && controller()->game->inPlay->hasType("island"))
+    if (opponent->basicAbilities[(int)Constants::ISLANDWALK] && controller()->game->inPlay->hasType("island"))
         return 0;
-    if (opponent->basicAbilities[Constants::MOUNTAINWALK] && controller()->game->inPlay->hasType("mountain"))
+    if (opponent->basicAbilities[(int)Constants::MOUNTAINWALK] && controller()->game->inPlay->hasType("mountain"))
         return 0;
-    if (opponent->basicAbilities[Constants::PLAINSWALK] && controller()->game->inPlay->hasType("plains"))
+    if (opponent->basicAbilities[(int)Constants::PLAINSWALK] && controller()->game->inPlay->hasType("plains"))
         return 0;
-    if (opponent->basicAbilities[Constants::LEGENDARYWALK] && controller()->game->inPlay->hasPrimaryType("legendary","land"))
+    if (opponent->basicAbilities[(int)Constants::LEGENDARYWALK] && controller()->game->inPlay->hasPrimaryType("legendary","land"))
         return 0;
-    if (opponent->basicAbilities[Constants::DESERTWALK] && controller()->game->inPlay->hasSpecificType("land","desert"))
+    if (opponent->basicAbilities[(int)Constants::DESERTWALK] && controller()->game->inPlay->hasSpecificType("land","desert"))
         return 0;
-    if (opponent->basicAbilities[Constants::SNOWSWAMPWALK] && controller()->game->inPlay->hasSpecificType("snow","swamp"))
+    if (opponent->basicAbilities[(int)Constants::SNOWSWAMPWALK] && controller()->game->inPlay->hasSpecificType("snow","swamp"))
         return 0;
-    if (opponent->basicAbilities[Constants::SNOWFORESTWALK] && controller()->game->inPlay->hasSpecificType("snow","forest"))
+    if (opponent->basicAbilities[(int)Constants::SNOWFORESTWALK] && controller()->game->inPlay->hasSpecificType("snow","forest"))
         return 0;
-    if (opponent->basicAbilities[Constants::SNOWISLANDWALK] && controller()->game->inPlay->hasSpecificType("snow","island"))
+    if (opponent->basicAbilities[(int)Constants::SNOWISLANDWALK] && controller()->game->inPlay->hasSpecificType("snow","island"))
         return 0;
-    if (opponent->basicAbilities[Constants::SNOWMOUNTAINWALK] && controller()->game->inPlay->hasSpecificType("snow","mountain"))
+    if (opponent->basicAbilities[(int)Constants::SNOWMOUNTAINWALK] && controller()->game->inPlay->hasSpecificType("snow","mountain"))
         return 0;
-    if (opponent->basicAbilities[Constants::SNOWPLAINSWALK] && controller()->game->inPlay->hasSpecificType("snow","plains"))
+    if (opponent->basicAbilities[(int)Constants::SNOWPLAINSWALK] && controller()->game->inPlay->hasSpecificType("snow","plains"))
         return 0;
-    if (opponent->basicAbilities[Constants::SNOWWALK] && controller()->game->inPlay->hasPrimaryType("snow","land"))
+    if (opponent->basicAbilities[(int)Constants::SNOWWALK] && controller()->game->inPlay->hasPrimaryType("snow","land"))
         return 0;
-    if (opponent->basicAbilities[Constants::NONBASICWALK] && controller()->game->inPlay->hasTypeButNotType("land","basic"))
+    if (opponent->basicAbilities[(int)Constants::NONBASICWALK] && controller()->game->inPlay->hasTypeButNotType("land","basic"))
         return 0;
     return 1;
 }
@@ -662,7 +664,7 @@ MTGCardInstance * MTGCardInstance::getNextPartner()
     MTGCardInstance * bandingPartner = inplay->getNextAttacker(banding);
     while (bandingPartner)
     {
-        if (basicAbilities[Constants::BANDING] || bandingPartner->basicAbilities[Constants::BANDING])
+        if (basicAbilities[(int)Constants::BANDING] || bandingPartner->basicAbilities[(int)Constants::BANDING])
             return bandingPartner;
         bandingPartner = inplay->getNextAttacker(bandingPartner);
     }
@@ -709,9 +711,8 @@ int MTGCardInstance::setAttacker(int value)
         previousTarget = p;
     attacker = value;
     WEvent * e = NEW WEventCreatureAttacker(this, previousTarget, target);
-    GameObserver * go = GameObserver::GetInstance();
-    if (go)
-        go->receiveEvent(e);
+    if (observer)
+        observer->receiveEvent(e);
     else
         SAFE_DELETE(e);
     return 1;
@@ -767,9 +768,8 @@ int MTGCardInstance::raiseBlockerRankOrder(MTGCardInstance * blocker)
 
     std::iter_swap(it1, it2);
     WEvent* e = NEW WEventCreatureBlockerRank(*it1, *it2, this);
-    GameObserver * go = GameObserver::GetInstance();
-    if (go)
-        go->receiveEvent(e);
+    if (observer)
+        observer->receiveEvent(e);
     else
         SAFE_DELETE(e);
     //delete(e);
@@ -792,7 +792,10 @@ int MTGCardInstance::getDefenserRank(MTGCardInstance * blocker)
 int MTGCardInstance::removeBlocker(MTGCardInstance * blocker)
 {
     blockers.remove(blocker);
-    if (!blockers.size())
+    // Blockers can be removed "manually" (by the blocking player) at the Blockers step,
+    // Or "automatically" in the damage phase, when they die and regenerate (see http://code.google.com/p/wagic/issues/detail?id=563 )
+    // In the second case, we still want the card to be marked as "blocked" this turn
+    if (!blockers.size() && observer->currentGamePhase == MTG_PHASE_COMBATBLOCKERS)
     {
         blocked = false;
     }
@@ -809,13 +812,12 @@ int MTGCardInstance::addBlocker(MTGCardInstance * blocker)
 //Returns opponents to this card for this turn. This * should * take into account banding
 MTGCardInstance * MTGCardInstance::getNextOpponent(MTGCardInstance * previous)
 {
-    GameObserver * game = GameObserver::GetInstance();
     int foundprevious = 0;
     if (!previous)
         foundprevious = 1;
     if (attacker)
     {
-        MTGInPlay * inPlay = game->opponent()->game->inPlay;
+        MTGInPlay * inPlay = observer->opponent()->game->inPlay;
         for (int i = 0; i < inPlay->nb_cards; i++)
         {
             MTGCardInstance * current = inPlay->cards[i];
@@ -835,7 +837,7 @@ MTGCardInstance * MTGCardInstance::getNextOpponent(MTGCardInstance * previous)
     }
     else if (defenser)
     {
-        MTGInPlay * inPlay = game->currentPlayer->game->inPlay;
+        MTGInPlay * inPlay = observer->currentPlayer->game->inPlay;
         for (int i = 0; i < inPlay->nb_cards; i++)
         {
             MTGCardInstance * current = inPlay->cards[i];
@@ -857,10 +859,9 @@ MTGCardInstance * MTGCardInstance::getNextOpponent(MTGCardInstance * previous)
 
 int MTGCardInstance::setDefenser(MTGCardInstance * opponent)
 {
-    GameObserver * g = GameObserver::GetInstance();
     if (defenser)
     {
-        if (g->players[0]->game->battlefield->hasCard(defenser) || g->players[1]->game->battlefield->hasCard(defenser))
+        if (observer->players[0]->game->battlefield->hasCard(defenser) || observer->players[1]->game->battlefield->hasCard(defenser))
         {
             defenser->removeBlocker(this);
         }
@@ -872,7 +873,7 @@ int MTGCardInstance::setDefenser(MTGCardInstance * opponent)
     if (defenser)
         defenser->addBlocker(this);
     if (e)
-        g->receiveEvent(e);
+        observer->receiveEvent(e);
     return 1;
 }
 
@@ -1017,7 +1018,7 @@ int MTGCardInstance::cantBeBlockedBy(MTGCardInstance * card)
 /* Choose a sound sample to associate to that card */
 JSample * MTGCardInstance::getSample()
 {
-    JSample * js;
+    JSample * js = 0;
 
     if (sample.size())
         return WResourceManager::Instance()->RetrieveSample(sample);
@@ -1025,7 +1026,47 @@ JSample * MTGCardInstance::getSample()
     for (int i = types.size() - 1; i > 0; i--)
     {
         string type = Subtypes::subtypesList->find(types[i]);
+        std::transform(type.begin(), type.end(), type.begin(), ::tolower);
         type = type + ".wav";
+        if(getObserver() && getObserver()->getResourceManager())
+        {
+            js = WResourceManager::Instance()->RetrieveSample(type);
+            if (js)
+            {
+                sample = string(type);
+                return js;
+            }
+        }
+    }
+
+    if (basicAbilities.any())
+    {
+        for (size_t x = 0; x < basicAbilities.size(); ++x)
+        {
+            if (!basicAbilities.test(x))
+                continue;
+            string type = Constants::MTGBasicAbilities[x];
+            type = type + ".wav";
+            if(getObserver() && getObserver()->getResourceManager())
+            {
+                js = WResourceManager::Instance()->RetrieveSample(type);
+                if (js)
+                {
+                    sample = string(type);
+                    return js;
+                }
+            }
+        }
+    }
+
+    string type = "";
+    if(!types.size())
+        return NULL;   
+    type = Subtypes::subtypesList->find(types[0]);
+    std::transform(type.begin(), type.end(), type.begin(), ::tolower);
+    type.append(".wav");
+    if(getObserver() && getObserver()->getResourceManager())
+    {
         js = WResourceManager::Instance()->RetrieveSample(type);
         if (js)
         {
@@ -1034,30 +1075,6 @@ JSample * MTGCardInstance::getSample()
         }
     }
 
-    for (map<int, int>::const_iterator it = basicAbilities.begin(); it != basicAbilities.end(); ++it)
-    {
-        int i = it->first;
-        if (!basicAbilities[i])
-            continue;
-        string type = Constants::MTGBasicAbilities[i];
-        type = type + ".wav";
-        js = WResourceManager::Instance()->RetrieveSample(type);
-        if (js)
-        {
-            sample = string(type);
-            return js;
-        }
-    }
-
-    string type = Subtypes::subtypesList->find(types[0]);
-    type = type + ".wav";
-    js = WResourceManager::Instance()->RetrieveSample(type);
-    if (js)
-    {
-        sample = string(type);
-        return js;
-    }
-
     return NULL;
 }
 
@@ -1090,3 +1107,8 @@ std::ostream& operator<<(std::ostream& out, const MTGCardInstance& c)
 {
     return c.toString(out);
 }
+
+MTGCardInstance* MTGCardInstance::clone()
+{
+    return new MTGCardInstance(model, owner->game);
+}
