@@ -11,6 +11,7 @@
 #include "utils.h"
 #include "DeckManager.h"
 #include <iomanip>
+#include "AbilityParser.h"
 
 #if defined (WIN32) || defined (LINUX)
 #include <time.h>
@@ -78,25 +79,13 @@ int MTGAllCards::processConfLine(string &s, MTGCard *card, CardPrimitive * primi
             string value = val;
             //Specific Abilities
             std::transform(value.begin(), value.end(), value.begin(), ::tolower);
-            while (value.size())
+            vector<string> values = split(value, ',');
+            for (size_t values_i = 0; values_i < values.size(); ++values_i)
             {
-                string attribute;
-                size_t found2 = value.find(',');
-                if (found2 != string::npos)
-                {
-                    attribute = value.substr(0, found2);
-                    value = value.substr(found2 + 1);
-                }
-                else
-                {
-                    attribute = value;
-                    value = "";
-                }
 
                 for (int j = Constants::NB_BASIC_ABILITIES - 1; j >= 0; --j)
                 {
-                    size_t found = attribute.find(Constants::MTGBasicAbilities[j]);
-                    if (found != string::npos)
+                    if (values[values_i].find(Constants::MTGBasicAbilities[j]) != string::npos)
                     {
                         primitive->basicAbilities[j] = 1;
                         break;
@@ -106,6 +95,16 @@ int MTGAllCards::processConfLine(string &s, MTGCard *card, CardPrimitive * primi
         }
         break;
 
+    case 'b': //buyback
+        if (!primitive) primitive = NEW CardPrimitive();
+        if (ManaCost * cost = primitive->getManaCost())
+        {
+            string value = val;
+            std::transform(value.begin(), value.end(), value.begin(), ::tolower);
+            cost->BuyBack = ManaCost::parseManaCost(value);
+        }
+        break;
+
     case 'c': //color
         if (!primitive) primitive = NEW CardPrimitive();
         {
@@ -121,48 +120,6 @@ int MTGAllCards::processConfLine(string &s, MTGCard *card, CardPrimitive * primi
         }
         break;
 
-    case 'g': //grade
-        if (s.size() - i - 1 > 2) currentGrade = getGrade(val[2]);
-        break;
-
-    case 'k': //kicker
-        if (!primitive) primitive = NEW CardPrimitive();
-        if (ManaCost * cost = primitive->getManaCost())
-        {
-            string value = val;
-            std::transform(value.begin(), value.end(), value.begin(), ::tolower);
-            cost->kicker = ManaCost::parseManaCost(value);
-        }
-        break;
-
-    case 'o': //othercost
-        if (!primitive) primitive = NEW CardPrimitive();
-        if(key[5] == 'r')//otherrestrictions
-        {
-            string value = val;
-            primitive->setOtherRestrictions(value);
-
-        }
-        else
-        {
-            if (ManaCost * cost = primitive->getManaCost())
-            {
-                string value = val;
-                std::transform(value.begin(), value.end(), value.begin(), ::tolower);
-                cost->alternative = ManaCost::parseManaCost(value);
-            }
-        }
-        break;
-
-    case 'b': //buyback
-        if (!primitive) primitive = NEW CardPrimitive();
-        if (ManaCost * cost = primitive->getManaCost())
-        {
-            string value = val;
-            std::transform(value.begin(), value.end(), value.begin(), ::tolower);
-            cost->BuyBack = ManaCost::parseManaCost(value);
-        }
-        break;
     case 'f': //flashback//morph
         {
             if (!primitive) primitive = NEW CardPrimitive();
@@ -183,11 +140,35 @@ int MTGAllCards::processConfLine(string &s, MTGCard *card, CardPrimitive * primi
             }
             break;
         }
+
+    case 'g': //grade
+        if (s.size() - i - 1 > 2) currentGrade = getGrade(val[2]);
+        break;
+
     case 'i': //id
         if (!card) card = NEW MTGCard();
         card->setMTGId(atoi(val));
         break;
 
+    case 'k': //kicker
+        if (!primitive) primitive = NEW CardPrimitive();
+        if (ManaCost * cost = primitive->getManaCost())
+        {
+            string value = val;
+            std::transform(value.begin(), value.end(), value.begin(), ::tolower);
+            size_t multikick = value.find("multi");
+            bool isMultikicker = false;
+            if(multikick != string::npos)
+            {
+                size_t endK = value.find("{",multikick);
+                value.erase(multikick, endK - multikick);
+                isMultikicker = true;
+        }
+            cost->kicker = ManaCost::parseManaCost(value);  
+            cost->kicker->isMulti = isMultikicker;
+        }
+        break;
+
     case 'm': //mana
         if (!primitive) primitive = NEW CardPrimitive();
         {
@@ -199,10 +180,37 @@ int MTGAllCards::processConfLine(string &s, MTGCard *card, CardPrimitive * primi
 
     case 'n': //name
         if (!primitive) primitive = NEW CardPrimitive();
-        if (0 == strcmp("Bloodrock Cyclops", val)) cout << "val" << endl;
         primitive->setName(val);
         break;
 
+    case 'o': //othercost/otherrestriction
+        if (!primitive) primitive = NEW CardPrimitive();
+        if(key[5] == 'r')//otherrestrictions
+        {
+            string value = val;
+            primitive->setOtherRestrictions(value);
+        }
+        else
+        {
+            if (ManaCost * cost = primitive->getManaCost())
+            {
+                string value = val;
+                std::transform(value.begin(), value.end(), value.begin(), ::tolower);
+                size_t name = value.find("name(");
+                string theName = "";
+                    if(name != string::npos)
+                    {
+                        size_t endName = value.find(")",name);
+                        theName = value.substr(name + 5,endName - name - 5);
+                        value.erase(name, endName - name + 1);
+                    }
+                    cost->alternative = ManaCost::parseManaCost(value);
+                    if(theName.size())
+                        cost->alternative->alternativeName.append(theName);
+            }
+        }
+        break;
+
     case 'p':
         if ('r' == key[1])
         { // primitive
@@ -217,60 +225,12 @@ int MTGAllCards::processConfLine(string &s, MTGCard *card, CardPrimitive * primi
         }
         break;
 
-    case 'r': //retrace/rarity
+    case 'r': //retrace/rarity//restrictions
         if('s' == key[2] && 't' == key[3])//restrictions
         {
             if (!primitive) primitive = NEW CardPrimitive();
             string value = val;
-            primitive->setRestrictions(CardPrimitive::NO_RESTRICTION);
-            primitive->hasRestriction = false;
-            if (value.find("control two or more vampires") != string::npos)
-                primitive->setRestrictions(CardPrimitive::VAMPIRES);
-            if (value.find("control less creatures") != string::npos)
-                primitive->setRestrictions(CardPrimitive::LESS_CREATURES);
-            if (value.find("control snow land") != string::npos)
-                primitive->setRestrictions(CardPrimitive::SNOW_LAND_INPLAY);
-            if (value.find("casted a spell") != string::npos)
-                primitive->setRestrictions(CardPrimitive::CASTED_A_SPELL);
-            if (value.find("one of a kind") != string::npos)
-                primitive->setRestrictions(CardPrimitive::ONE_OF_AKIND);
-            if (value.find("fourth turn") != string::npos)
-                primitive->setRestrictions(CardPrimitive::FOURTHTURN);
-            if (value.find("before battle damage") != string::npos)
-                primitive->setRestrictions(CardPrimitive::BEFORECOMBATDAMAGE);
-            if (value.find("after battle") != string::npos)
-                primitive->setRestrictions(CardPrimitive::AFTERCOMBAT);
-            if (value.find("during battle") != string::npos)
-                primitive->setRestrictions(CardPrimitive::DURINGCOMBAT);
-            if (value.find("myturnonly") != string::npos)
-                primitive->setRestrictions(CardPrimitive::PLAYER_TURN_ONLY);
-            if (value.find("opponentturnonly") != string::npos)
-                primitive->setRestrictions(CardPrimitive::OPPONENT_TURN_ONLY);
-            if (value.find("assorcery") != string::npos)
-                primitive->setRestrictions(CardPrimitive::AS_SORCERY);
-            string types[] = { "my", "opponent", "" };
-            int starts[] = { CardPrimitive::MY_BEFORE_BEGIN, CardPrimitive::OPPONENT_BEFORE_BEGIN, CardPrimitive::BEFORE_BEGIN };
-            for (int j = 0; j < 3; ++j)
-            {
-                size_t found = value.find(types[j]);
-                if (found != string::npos)
-                {
-                    for (int i = 0; i < Constants::NB_MTG_PHASES; i++)
-                    {
-                        string toFind = types[j];
-                        toFind.append(Constants::MTGPhaseCodeNames[i]).append("only");
-                        found = value.find(toFind);
-                        if (found != string::npos)
-                        {
-                            if(primitive->hasRestriction == false)
-                            {
-                                primitive->setRestrictions(starts[j] + i);
-                                primitive->hasRestriction = true;
-                            }
-                        }
-                    }
-                }
-            }
+            primitive->setRestrictions(value);
         }
         else if ('e' == key[1] && 't' == key[2])
         { //retrace
@@ -288,7 +248,8 @@ int MTGAllCards::processConfLine(string &s, MTGCard *card, CardPrimitive * primi
             card->setRarity(val[0]);
         }
         break;
-    case 's': //subtype
+
+    case 's': //subtype, suspend
         {
             if (s.find("suspend") != string::npos)
             {
@@ -308,24 +269,13 @@ int MTGAllCards::processConfLine(string &s, MTGCard *card, CardPrimitive * primi
             else
             {
                 if (!primitive) primitive = NEW CardPrimitive();
-                while (true)
-                {
-                    char* found = strchr(val, ' ');
-                    if (found)
-                    {
-                        string value(val, found - val);
-                        primitive->setSubtype(value);
-                        val = found + 1;
-                    }
-                    else
-                    {
-                        primitive->setSubtype(val);
-                        break;
-                    }
-                }
+                vector<string> values = split(val, ' ');
+                for (size_t values_i = 0; values_i < values.size(); ++values_i)
+                    primitive->setSubtype(values[values_i]);
             }
             break;
         }
+
     case 't':
         if (!primitive) primitive = NEW CardPrimitive();
         if (0 == strcmp("target", key))
@@ -338,21 +288,9 @@ int MTGAllCards::processConfLine(string &s, MTGCard *card, CardPrimitive * primi
             primitive->setText(val);
         else if (0 == strcmp("type", key))
         {
-            while (true)
-            {
-                char* found = strchr(val, ' ');
-                if (found)
-                {
-                    string value(val, found - val);
-                    primitive->setType(value);
-                    val = found + 1;
-                }
-                else
-                {
-                    primitive->setType(val);
-                    break;
-                }
-            }
+            vector<string> values = split(val, ' ');
+            for (size_t values_i = 0; values_i < values.size(); ++values_i)
+                    primitive->setType(values[values_i]);
         }
         else if (0 == strcmp("toughness", key)) primitive->setToughness(atoi(val));
         break;
@@ -371,8 +309,9 @@ int MTGAllCards::processConfLine(string &s, MTGCard *card, CardPrimitive * primi
 
 void MTGAllCards::initCounters()
 {
-    for (int i = 0; i < Constants::MTG_NB_COLORS; i++)
-        colorsCount[i] = 0;
+    colorsCount.erase(colorsCount.begin(),colorsCount.end());
+    for (int i = 0; i < Constants::NB_Colors; i++)
+        colorsCount.push_back(0);
 }
 
 void MTGAllCards::init()
@@ -383,41 +322,6 @@ void MTGAllCards::init()
     initCounters();
 }
 
-void MTGAllCards::loadPrimitives(const char *root_directory)
-{
-    DIR *dir = opendir(root_directory);
-    struct dirent *dit;
-
-    while ((dit = readdir(dir)))
-    {
-        char fullname[1000];
-        sprintf(fullname, "%s/%s", root_directory, dit->d_name);
-        wagic::ifstream file(fullname);
-        if (!file)
-            continue;
-        file.close();
-        MTGCollection()->load(fullname);
-    }
-    closedir(dir);
-}
-
-void MTGAllCards::loadSets(const char *root_directory, const char* filename)
-{
-    DIR *dir = opendir(root_directory);
-    struct dirent *dit;
-
-    while ((dit = readdir(dir)))
-    {
-        char fullname[1000];
-        sprintf(fullname, "%s/%s/%s", root_directory, dit->d_name, filename);
-        wagic::ifstream file(fullname);
-        if (!file)
-            continue;
-        file.close();
-        MTGCollection()->load(fullname, dit->d_name);
-    }
-    closedir(dir);}
-
 int MTGAllCards::load(const char * config_file, const char * set_name, int autoload)
 {
     conf_read_mode = 0;
@@ -425,33 +329,42 @@ int MTGAllCards::load(const char * config_file, const char * set_name, int autol
     MTGSetInfo *si = setlist.getInfo(set_id);
 
     int lineNumber = 0;
-    wagic::ifstream setFile(config_file, ios::in | ios::ate);
-    if (!setFile) return total_cards;
-
-    streampos fileSize = setFile.tellg();
-    setFile.seekg(0, ios::beg);
-
     std::string contents;
-    contents.resize((std::string::size_type) fileSize);
-    setFile.read(&contents[0], fileSize);
-    std::stringstream stream(contents);
+    izfstream file;
+    if (!JFileSystem::GetInstance()->openForRead(file, config_file))
+        return total_cards;
 
     string s;
 
-    while (true)
+    while (getline(file,s))
     {
-        if (!std::getline(stream, s)) return total_cards;
         lineNumber++;
         if (!s.size()) continue;
+        if (s[s.size() - 1] == '\r')
+            s.erase(s.size() - 1); //Handle DOS files
+        if (!s.size()) continue;
+
+        if (s.find("#AUTO_DEFINE ") == 0)
+        {
+            string toAdd = s.substr(13);
+            AutoLineMacro::AddMacro(toAdd);
+            continue;
+        }
 
-        if (s[s.size() - 1] == '\r') s.erase(s.size() - 1); // Handle DOS files
         switch (conf_read_mode)
         {
         case MTGAllCards::READ_ANYTHING:
             if (s[0] == '[')
             {
                 currentGrade = Constants::GRADE_SUPPORTED; // Default value
-                conf_read_mode = ('m' == s[1]) ? MTGAllCards::READ_METADATA : MTGAllCards::READ_CARD; // M for metadata.
+                if (s.size() < 2)
+                {
+                    DebugTrace("FATAL: Card file incorrect");
+                }
+                else
+                {
+                    conf_read_mode = ('m' == s[1]) ? MTGAllCards::READ_METADATA : MTGAllCards::READ_CARD; // M for metadata.
+                }
             }
             else
             {
@@ -463,6 +376,7 @@ int MTGAllCards::load(const char * config_file, const char * set_name, int autol
                     if (!maxGrade) maxGrade = Constants::GRADE_BORDERLINE; //Default setting for grade is borderline?
                     if (fileGrade > maxGrade)
                     {
+                        file.close();
                         return total_cards;
                     }
                 }
@@ -494,6 +408,7 @@ int MTGAllCards::load(const char * config_file, const char * set_name, int autol
             continue;
         }
     }
+    file.close();
     return total_cards;
 }
 
@@ -578,7 +493,7 @@ int MTGAllCards::countByColor(int color)
 {
     if (colorsCount[color] == 0)
     {
-        for (int i = 0; i < Constants::MTG_NB_COLORS; i++)
+        for (int i = 0; i < Constants::NB_Colors; i++)
         {
             colorsCount[i] = 0;
         }
@@ -688,39 +603,75 @@ MTGCard * MTGAllCards::_(int index)
     return getCardById(ids[index]);
 }
 
-MTGCard * MTGAllCards::getCardByName(string name)
+#ifdef TESTSUITE
+void MTGAllCards::prefetchCardNameCache()
 {
-    if (!name.size()) return NULL;
-    if (name[0] == '#') return NULL;
+    map<int, MTGCard *>::iterator it;
+    for (it = collection.begin(); it != collection.end(); it++)
+    {
+        MTGCard * c = it->second;
 
-    map<string, MTGCard * >::iterator cached = mtgCardByNameCache.end();
-    if (mtgCardByNameCache.size() > 0)
-        cached = mtgCardByNameCache.find(name);
+        //Name only
+        string cardName = c->data->name;
+        std::transform(cardName.begin(), cardName.end(), cardName.begin(), ::tolower);
+        mtgCardByNameCache[cardName] = c;
+
+        //Name + set
+        int setId = c->setId;
+        MTGSetInfo* setInfo = setlist.getInfo(setId);
+        if (setInfo)
+        {
+            string setName = setInfo->getName();
+            std::transform(setName.begin(), setName.end(), setName.begin(), ::tolower);
+            cardName = cardName + " (" + setName +  ")";
+            mtgCardByNameCache[cardName] = c;
+        }
+
+        // id
+        std::stringstream out;
+        out << c->getMTGId();
+        mtgCardByNameCache[out.str()] = c;
+    }    
+}
+#endif
+
+MTGCard * MTGAllCards::getCardByName(string nameDescriptor)
+{
+    if (!nameDescriptor.size()) return NULL;
+    if (nameDescriptor[0] == '#') return NULL;
+
+    std::transform(nameDescriptor.begin(), nameDescriptor.end(), nameDescriptor.begin(), ::tolower);
+
+    map<string, MTGCard * >::iterator cached = mtgCardByNameCache.find(nameDescriptor);
     
     if (cached!= mtgCardByNameCache.end())
     {
         return cached->second;
     }
 
-    int cardnb = atoi(name.c_str());
+    int cardnb = atoi(nameDescriptor.c_str());
     if (cardnb)
     {
         MTGCard * result = getCardById(cardnb);
-        mtgCardByNameCache[name] = result;
+        mtgCardByNameCache[nameDescriptor] = result;
         return result;
     }
 
-    std::transform(name.begin(), name.end(), name.begin(), ::tolower);
     int setId = -1;
-    size_t found = name.find(" (");
+    size_t found = nameDescriptor.find(" (");
+    string name = nameDescriptor;
     if (found != string::npos)
     {
-        size_t end = name.find(")");
-        string setName = name.substr(found + 2, end - found - 2);
+        size_t end = nameDescriptor.find(")");
+        string setName = nameDescriptor.substr(found + 2, end - found - 2);
         trim(setName);
-        name = name.substr(0, found);
+        name = nameDescriptor.substr(0, found);
         trim(name);
         setId = setlist[setName];
+
+        //Reconstruct a clean string "name (set)" for cache consistency
+        nameDescriptor = name + " (" + setName + ")";
+        
     }
     map<int, MTGCard *>::iterator it;
     for (it = collection.begin(); it != collection.end(); it++)
@@ -730,12 +681,12 @@ MTGCard * MTGAllCards::getCardByName(string name)
         string cardName = c->data->name;
         std::transform(cardName.begin(), cardName.end(), cardName.begin(), ::tolower);
         if (cardName.compare(name) == 0) {
-            mtgCardByNameCache[name] = c;
+            mtgCardByNameCache[nameDescriptor] = c;
             return c;
         }
 
     }
-    mtgCardByNameCache[name] = NULL;
+    mtgCardByNameCache[nameDescriptor] = NULL;
     return NULL;
 }
 
@@ -751,7 +702,7 @@ MTGDeck::MTGDeck(MTGAllCards * _allcards)
 int MTGDeck::totalPrice()
 {
     int total = 0;
-    PriceList * pricelist = NEW PriceList(JGE_GET_RES("settings/prices.dat").c_str(), MTGCollection());
+    PriceList * pricelist = NEW PriceList("settings/prices.dat", MTGCollection());
     map<int, int>::iterator it;
     for (it = cards.begin(); it != cards.end(); it++)
     {
@@ -771,15 +722,16 @@ MTGDeck::MTGDeck(const char * config_file, MTGAllCards * _allcards, int meta_onl
     size_t dot = filename.find(".");
     meta_name = filename.substr(slash + 1, dot - slash - 1);
     meta_id = atoi(meta_name.substr(4).c_str());
-    wagic::ifstream file(config_file);
-    std::string s;
-    
-    if (file)
+    std::string contents;
+    if (JFileSystem::GetInstance()->readIntoString(config_file, contents))
     {
-        while (std::getline(file, s))
+        std::stringstream stream(contents);
+        std::string s;
+        while (std::getline(stream, s))
         {
             if (!s.size()) continue;
             if (s[s.size() - 1] == '\r') s.erase(s.size() - 1); //Handle DOS files
+            if (!s.size()) continue;
             if (s[0] == '#')
             {
                 size_t found = s.find("NAME:");
@@ -795,6 +747,18 @@ MTGDeck::MTGDeck(const char * config_file, MTGAllCards * _allcards, int meta_onl
                     meta_desc.append(s.substr(found + 5));
                     continue;
                 }
+                found = s.find("HINT:");
+                if (found != string::npos)
+                {
+                    meta_AIHints.push_back(s.substr(found + 5));
+                    continue;
+                }
+                found = s.find("UNLOCK:");
+                if (found != string::npos)
+                {
+                    meta_unlockRequirements = s.substr(found + 7);
+                    continue;
+                }
                 continue;
             }
             if (meta_only) break;
@@ -834,11 +798,10 @@ MTGDeck::MTGDeck(const char * config_file, MTGAllCards * _allcards, int meta_onl
                 DebugTrace("could not find Card matching name: " << s);
             }
         }
-        file.close();
     }
     else
     {
-        //TODO Error management
+        DebugTrace("FATAL:MTGDeck.cpp:MTGDeck - can't load deck file");
     }
 }
 
@@ -861,8 +824,9 @@ int MTGDeck::addRandomCards(int howmany, int * setIds, int nbSets, int rarity, c
 {
     if (howmany <= 0) return 1;
 
-    int unallowedColors[Constants::MTG_NB_COLORS + 1];
-    for (int i = 0; i < Constants::MTG_NB_COLORS; ++i)
+    vector<int> unallowedColors;
+    unallowedColors.resize(Constants::NB_Colors + 1);
+    for (int i = 0; i < Constants::NB_Colors; ++i)
     {
         if (nbcolors)
             unallowedColors[i] = 1;
@@ -904,7 +868,7 @@ int MTGDeck::addRandomCards(int howmany, int * setIds, int nbSets, int rarity, c
 
             if (ok)
             {
-                for (int j = 0; j < Constants::MTG_NB_COLORS; ++j)
+                for (int j = 0; j < Constants::NB_Colors; ++j)
                 {
                     if (unallowedColors[j] && card->data->hasColor(j))
                     {
@@ -949,7 +913,8 @@ int MTGDeck::add(MTGDeck * deck)
 
 int MTGDeck::add(int cardid)
 {
-    if (!database->getCardById(cardid)) return 0;
+    if (!database->getCardById(cardid))
+        return 0;
     if (cards.find(cardid) == cards.end())
     {
         cards[cardid] = 1;
@@ -965,7 +930,8 @@ int MTGDeck::add(int cardid)
 
 int MTGDeck::add(MTGCard * card)
 {
-    if (!card) return 0;
+    if (!card) 
+        return 0;
     return (add(card->getId()));
 }
 
@@ -1038,10 +1004,10 @@ int MTGDeck::save(const string& destFileName, bool useExpandedDescriptions, cons
 {
     string tmp = destFileName;
     tmp.append(".tmp"); //not thread safe
-    std::ofstream file(tmp.c_str());
-    char writer[512];
-    if (file)
+    std::ofstream file;
+    if (JFileSystem::GetInstance()->openForWrite(file, tmp))
     {
+        char writer[512];
         DebugTrace("Saving Deck: " << deckTitle << " to " << destFileName );
         if (meta_name.size())
         {
@@ -1084,8 +1050,7 @@ int MTGDeck::save(const string& destFileName, bool useExpandedDescriptions, cons
             }
         }
         file.close();
-        std::remove(destFileName.c_str());
-        rename(tmp.c_str(), destFileName.c_str());
+        JFileSystem::GetInstance()->Rename(tmp, destFileName);
     }
     return 1;
 }
@@ -1111,7 +1076,6 @@ void MTGDeck::printDetailedDeckText(std::ofstream& file )
         MTGSetInfo *setInfo = setlist.getInfo(card->setId);
         string setName = setInfo->id;
         string cardName = card->data->getName();
-        string description = card->data->getText();
 
         currentCard << "#" << nbCards << " x " << cardName << " (" << setName << "), ";
 
@@ -1131,12 +1095,14 @@ void MTGDeck::printDetailedDeckText(std::ofstream& file )
             currentCard << card->data->getPower() << "/" << card->data->getToughness() << ", ";
 
 
-        if ( card->data->hasRestriction )
-            currentCard << ", " << card->data->otherrestriction;
+        if ( card->data->getOtherRestrictions().size() )
+            currentCard << ", " << card->data->getOtherRestrictions();
 
-        map<int,int>::iterator abilityIter;
-        for ( abilityIter = card->data->basicAbilities.begin(); abilityIter != card->data->basicAbilities.end(); ++abilityIter )
-            currentCard << Constants::MTGBasicAbilities[ abilityIter->first ] << "; ";
+        for (size_t x = 0; x < card->data->basicAbilities.size(); ++x)
+        {
+            if ( card->data->basicAbilities[x] == 1 )
+                currentCard <<  Constants::MTGBasicAbilities[x] << "; ";
+        }
         currentCard <<endl;
 
         for ( int i = 0; i < nbCards; i++ )
@@ -1330,7 +1296,7 @@ MTGSetInfo::MTGSetInfo(string _id)
         counts[i] = 0;
 
     char myFilename[4096];
-    sprintf(myFilename, JGE_GET_RES("sets/%s/booster.txt").c_str(), id.c_str());
+    sprintf(myFilename, "sets/%s/booster.txt", id.c_str());
     mPack = NEW MTGPack(myFilename);
     if (!mPack->isValid())
     {
