@@ -9,10 +9,14 @@
 #include "WEvent.h"
 #include "MTGAbility.h"
 
+#include "iterator"
+
+SUPPORT_OBJECT_ANALYTICS(ManaCost)
 
 ManaCost * ManaCost::parseManaCost(string s, ManaCost * _manaCost, MTGCardInstance * c)
 {
     ManaCost * manaCost;
+    GameObserver* g = c?c->getObserver():NULL;
     if (_manaCost)
     {
         manaCost = _manaCost;
@@ -72,9 +76,8 @@ ManaCost * ManaCost::parseManaCost(string s, ManaCost * _manaCost, MTGCardInstan
                 }
                 else
                 {
-
                     //Parse target for extraCosts
-                    TargetChooserFactory tcf;
+                    TargetChooserFactory tcf(g);
                     TargetChooser * tc = NULL;
                     size_t target_start = value.find("(");
                     size_t target_end = value.find(")");
@@ -93,7 +96,7 @@ ManaCost * ManaCost::parseManaCost(string s, ManaCost * _manaCost, MTGCardInstan
                     case 't': //Tap
                         if (value == "t")
                         {
-                            //default Tap is handled outside of Manacost
+                            manaCost->addExtraCost(NEW TapCost);
                         }
                         else
                         {
@@ -140,38 +143,61 @@ ManaCost * ManaCost::parseManaCost(string s, ManaCost * _manaCost, MTGCardInstan
                         manaCost->addExtraCost(NEW MillCost(tc));
                         break;
                     case 'n': //return unblocked attacker cost
-                        TargetChooserFactory tcf;
-                        tc = tcf.createTargetChooser("creature|myBattlefield", c);
-                        manaCost->addExtraCost(NEW Ninja(tc));
-                        break;
-                    case 'c': //Counters
-                    {
-                        size_t counter_start = value.find("(");
-                        size_t counter_end = value.find(")", counter_start);
-                        AbilityFactory abf;
-                        string counterString = value.substr(counter_start + 1, counter_end - counter_start - 1);
-                        Counter * counter = abf.parseCounter(counterString, c);
-                        size_t separator = value.find(",", counter_start);
-                        size_t separator2 = string::npos;
-                        if (separator != string::npos)
                         {
-                            separator2 = value.find(",", separator + 1);
+                            TargetChooserFactory tcf(g);
+                            tc = tcf.createTargetChooser("creature|myBattlefield", c);
+                            manaCost->addExtraCost(NEW Ninja(tc));
+                            break;
                         }
-                        SAFE_DELETE(tc);
-                        size_t target_start = string::npos;
-                        if (separator2 != string::npos)
+                    case 'p' :
                         {
-                            target_start = value.find(",", separator2 + 1);
+                            SAFE_DELETE(tc);
+                            size_t start = value.find("(");
+                            size_t end = value.rfind(")");
+                            string manaType = value.substr(start + 1, end - start - 1);
+                            manaCost->addExtraCost(NEW LifeorManaCost(NULL,manaType));
+                            break;
                         }
-                        size_t target_end = counter_end;
-                        if (target_start != string::npos && target_end != string::npos)
+                    case 'q':
                         {
-                            string target = value.substr(target_start + 1, target_end - 1 - target_start);
-                            tc = tcf.createTargetChooser(target, c);
+                            manaCost->addExtraCost(NEW UnTapCost);
+                            break;
+                        }
+                    case 'c': //Counters or cycle
+                        {
+                            if(value == "cycle")
+                            {
+                                manaCost->addExtraCost(NEW CycleCost(tc));
+                            }
+                            else
+                            {
+                                size_t counter_start = value.find("(");
+                                size_t counter_end = value.find(")", counter_start);
+                                AbilityFactory abf(g);
+                                string counterString = value.substr(counter_start + 1, counter_end - counter_start - 1);
+                                Counter * counter = abf.parseCounter(counterString, c);
+                                size_t separator = value.find(",", counter_start);
+                                size_t separator2 = string::npos;
+                                if (separator != string::npos)
+                                {
+                                    separator2 = value.find(",", counter_end + 1);
+                                }
+                                SAFE_DELETE(tc);
+                                size_t target_start = string::npos;
+                                if (separator2 != string::npos)
+                                {
+                                    target_start = value.find(",", counter_end + 1);
+                                }
+                                size_t target_end = value.length();
+                                if (target_start != string::npos && target_end != string::npos)
+                                {
+                                    string target = value.substr(target_start + 1, target_end - 1 - target_start);
+                                    tc = tcf.createTargetChooser(target, c);
+                                }
+                                manaCost->addExtraCost(NEW CounterCost(counter, tc));
+                            }
+                            break;
                         }
-                        manaCost->addExtraCost(NEW CounterCost(counter, tc));
-                        break;
-                    }
                     default: //uncolored cost and hybrid costs
                     {
                         int intvalue = atoi(value.c_str());
@@ -189,7 +215,7 @@ ManaCost * ManaCost::parseManaCost(string s, ManaCost * _manaCost, MTGCardInstan
                                 }
                                 else
                                 {
-                                    for (int j = 0; j < Constants::MTG_NB_COLORS; j++)
+                                    for (int j = 0; j < Constants::NB_Colors; j++)
                                     {
                                         if (c == Constants::MTGColorChars[j])
                                         {
@@ -226,7 +252,7 @@ ManaCost::ManaCost()
     init();
 }
 
-ManaCost::ManaCost(int _cost[], int nb_elems)
+ManaCost::ManaCost(vector<int8_t>& _cost, int nb_elems)
 {
     init();
     for (int i = 0; i < nb_elems; i++)
@@ -243,16 +269,15 @@ ManaCost::ManaCost(ManaCost * manaCost)
     init();
     if ( !manaCost ) 
         return;
-    for (int i = 0; i <= Constants::MTG_NB_COLORS; i++)
+    for (int i = 0; i <= Constants::NB_Colors; i++)
     {
         cost[i] = manaCost->getCost(i);
     }
-    for (int i = 0 ; i < 10; i++)
-        hybrids[i] = manaCost->hybrids[i];
+    hybrids = manaCost->hybrids;
 
-    nbhybrids = manaCost->nbhybrids;
-    extraCostsIsCopy = manaCost->extraCostsIsCopy;
     kicker = NEW ManaCost( manaCost->kicker );
+    if(kicker)
+    kicker->isMulti = manaCost->isMulti;
     Retrace = NEW ManaCost( manaCost->Retrace );
     BuyBack = NEW ManaCost( manaCost->BuyBack );
     alternative = NEW ManaCost( manaCost->alternative );
@@ -260,26 +285,23 @@ ManaCost::ManaCost(ManaCost * manaCost)
     morph = NEW ManaCost( manaCost->morph );
     suspend = NEW ManaCost( manaCost->suspend );
 
-    // TODO: Need to figure out if a deep copy is necessary
-    extraCosts = manaCost->extraCosts;
+    extraCosts = manaCost->extraCosts ? manaCost->extraCosts->clone() : NULL;
 }
 
 // Copy Constructor 
 
 ManaCost::ManaCost(const ManaCost& manaCost)
+#ifdef TRACK_OBJECT_USAGE
+    : InstanceCounter<ManaCost>(manaCost)
+#endif
 {
-
-    for (int i = 0; i <= Constants::MTG_NB_COLORS; i++)
+    for (int i = 0; i <= Constants::NB_Colors; i++)
     {
-        cost[i] = manaCost.cost[i];      
-    }
-    for (int i = 0 ; i < 10; i++)
-    {
-        hybrids[i] = NEW ManaCostHybrid( manaCost.hybrids[i] );
+        cost.push_back(manaCost.cost[i]);      
     }
+ 
+    hybrids = manaCost.hybrids;
 
-    nbhybrids = manaCost.nbhybrids;
-    extraCostsIsCopy = manaCost.extraCostsIsCopy;
     // make new copies of the pointers for the deep copy
     kicker = NEW ManaCost( manaCost.kicker );
     Retrace = NEW ManaCost( manaCost.Retrace );
@@ -289,8 +311,7 @@ ManaCost::ManaCost(const ManaCost& manaCost)
     morph = NEW ManaCost( manaCost.morph );
     suspend = NEW ManaCost( manaCost.suspend );
     
-    // TODO: Need to figure out if a deep copy is necessary
-    extraCosts = manaCost.extraCosts;
+    extraCosts = manaCost.extraCosts ? manaCost.extraCosts->clone() : NULL;
 }
 
 // operator=
@@ -298,17 +319,11 @@ ManaCost & ManaCost::operator= (const ManaCost & manaCost)
 {
     if ( this != &manaCost )
     {
-        for (int i = 0; i < Constants::MTG_NB_COLORS; i++)
+        for (int i = 0; i < Constants::NB_Colors; i++)
             cost[i] = manaCost.cost[i];
 
-        for (int i = 0 ; i < 10; i++)
-        {
-            SAFE_DELETE( hybrids[i] );
-            hybrids[i] = manaCost.hybrids[i];
-        }
-        nbhybrids = manaCost.nbhybrids;
+        hybrids = manaCost.hybrids;
         extraCosts = manaCost.extraCosts;
-        extraCostsIsCopy = manaCost.extraCostsIsCopy;
         kicker = manaCost.kicker;
         Retrace = manaCost.Retrace;
         BuyBack = manaCost.BuyBack;
@@ -322,11 +337,69 @@ ManaCost & ManaCost::operator= (const ManaCost & manaCost)
 
 ManaCost::~ManaCost()
 {
-    for (unsigned int i = 0; i < 10; i++)
+    SAFE_DELETE(extraCosts);
+    SAFE_DELETE(kicker);
+    SAFE_DELETE(alternative);
+    SAFE_DELETE(BuyBack);
+    SAFE_DELETE(FlashBack);
+    SAFE_DELETE(Retrace);
+    SAFE_DELETE(morph);
+    SAFE_DELETE(suspend);
+
+    cost.erase(cost.begin() ,cost.end());
+}
+
+void ManaCost::x()
+{
+    if (cost.size() <= (size_t)Constants::NB_Colors)
     {
-        SAFE_DELETE(hybrids[i]);
+        DebugTrace("Seems ManaCost was not properly initialized");
+        return;
     }
 
+    cost[Constants::NB_Colors] = 1;
+}
+
+int ManaCost::hasX()
+{
+    if (cost.size() <= (size_t)Constants::NB_Colors)
+    {
+        DebugTrace("Seems ManaCost was not properly initialized");
+        return 0;
+    }
+
+    return cost[Constants::NB_Colors];
+}
+
+void ManaCost::init()
+{
+    int i;
+    cost.erase(cost.begin(),cost.end());
+    for (i = 0; i <= Constants::NB_Colors; i++)
+    {
+        cost.push_back(0);
+    }
+    extraCosts = NULL;
+    kicker = NULL;
+    alternative = NULL;
+    BuyBack = NULL;
+    FlashBack = NULL;
+    Retrace = NULL;
+    morph = NULL;
+    suspend = NULL;
+    isMulti = false;
+}
+
+void ManaCost::reinit()
+{
+    int i;
+    
+    cost.erase(cost.begin() ,cost.end());
+
+    for (i = 0; i <= Constants::NB_Colors; i++)
+    {
+        cost.push_back(0);
+    }
     SAFE_DELETE(extraCosts);
     SAFE_DELETE(kicker);
     SAFE_DELETE(alternative);
@@ -337,56 +410,19 @@ ManaCost::~ManaCost()
     SAFE_DELETE(suspend);
 }
 
-void ManaCost::x()
-{
-    cost[Constants::MTG_NB_COLORS] = 1;
-}
-
-int ManaCost::hasX()
-{
-    return cost[Constants::MTG_NB_COLORS];
-}
-
-void ManaCost::init()
-{
-    int i;
-    for (i = 0; i <= Constants::MTG_NB_COLORS; i++)
-    {
-        cost[i] = 0;
-    }
-    nbhybrids = 0;
-    extraCosts = NULL;
-    extraCostsIsCopy = 0;
-    kicker = NULL;
-    alternative = NULL;
-    BuyBack = NULL;
-    FlashBack = NULL;
-    Retrace = NULL;
-    morph = NULL;
-    suspend = NULL;
-    
-    // why is hybrids hardcoded to 10?
-    for (i = 0; i < 10; i++)
-        hybrids[i] = NULL;
-}
-
 void ManaCost::copy(ManaCost * _manaCost)
 {
     if (!_manaCost)
         return;
-    for (unsigned int i = 0; i <= Constants::MTG_NB_COLORS; i++)
+
+    cost.erase(cost.begin() ,cost.end());
+
+    for (int i = 0; i <= Constants::NB_Colors; i++)
     {
-        cost[i] = _manaCost->getCost(i);
+        cost.push_back(_manaCost->getCost(i));
     }
-    for (unsigned int i = 0; i < nbhybrids; i++)
-    {
-        SAFE_DELETE(hybrids[i]);
-    }
-    for (unsigned int i = 0; i < _manaCost->nbhybrids; i++)
-    {
-        hybrids[i] = NEW ManaCostHybrid((*_manaCost->hybrids[i]));
-    }
-    nbhybrids = _manaCost->nbhybrids;
+
+    hybrids = _manaCost->hybrids;
 
     SAFE_DELETE(extraCosts);
     if (_manaCost->extraCosts)
@@ -399,6 +435,7 @@ void ManaCost::copy(ManaCost * _manaCost)
     {
         kicker = NEW ManaCost();
         kicker->copy(_manaCost->kicker);
+        kicker->isMulti = _manaCost->kicker->isMulti;
     }
     SAFE_DELETE(alternative);
     if (_manaCost->alternative)
@@ -440,23 +477,28 @@ void ManaCost::copy(ManaCost * _manaCost)
 
 int ManaCost::getCost(int color)
 {
+    if (cost.size() <= (size_t)color)
+    {
+        DebugTrace("Seems ManaCost was not properly initialized");
+        return 0;
+    }
     return cost[color];
 }
 
 ManaCostHybrid * ManaCost::getHybridCost(unsigned int i)
 {
-    if (nbhybrids <= i)
+    if (hybrids.size() <= i)
         return NULL;
-    return hybrids[i];
+    return &hybrids[i];
 }
 
 int ManaCost::hasColor(int color)
 {
-    if (cost[color])
+    if (getCost(color))
         return 1;
-    for (unsigned int i = 0; i < nbhybrids; i++)
+    for (size_t i = 0; i < hybrids.size(); i++)
     {
-        if (hybrids[i]->hasColor(color))
+        if (hybrids[i].hasColor(color))
             return 1;
     }
     return 0;
@@ -474,24 +516,32 @@ int ManaCost::isNull()
 int ManaCost::getConvertedCost()
 {
     int result = 0;
-    for (unsigned int i = 0; i < Constants::MTG_NB_COLORS; i++)
+    for ( int i = 0; i < Constants::NB_Colors; i++)
     {
         result += cost[i];
     }
-    for (unsigned int i = 0; i < nbhybrids; i++)
+    for (size_t i = 0; i < hybrids.size(); i++)
     {
-        result += hybrids[i]->getConvertedCost();
+        result += hybrids[i].getConvertedCost();
     }
+	if(extraCosts && extraCosts->costs.size())
+	{
+		for(unsigned int i = 0; i < extraCosts->costs.size();i++)
+		{
+			ExtraCost * pMana = dynamic_cast<LifeorManaCost*>(extraCosts->costs[i]);
+			if(pMana)
+				result++;
+		}
+	}
+
     return result;
 }
 
 int ManaCost::remove(int color, int value)
 {
-    cost[color] -= value;
-    if (cost[color] < 0)
-    {
-        cost[color] = 0;
-    }
+    assert (value >= 0);
+    int8_t toRemove = min(cost[color], (int8_t)value);
+    cost[color] -= toRemove;
     return 1;
 }
 
@@ -507,24 +557,39 @@ int ManaCost::add(ManaCost * _cost)
 {
     if (!_cost)
         return 0;
-    for (unsigned int i = 0; i < Constants::MTG_NB_COLORS; i++)
+    for ( int i = 0; i < Constants::NB_Colors; i++)
     {
         cost[i] += _cost->getCost(i);
     }
-    for (unsigned int i = 0; i < _cost->nbhybrids; i++)
+
+    std::copy(_cost->hybrids.begin(), _cost->hybrids.end(), std::back_inserter(hybrids));
+
+    return 1;
+}
+
+int ManaCost::remove(ManaCost * _cost)
+{
+    if (!_cost)
+        return 0;
+    for ( int i = 0; i < Constants::NB_Colors; i++)
     {
-        hybrids[i] = NEW ManaCostHybrid((*_cost->hybrids[i]));
+        int8_t toRemove = min(cost[i], (int8_t)_cost->getCost(i)); //we don't want to be negative
+        cost[i] -= toRemove;
+        assert(cost[i] >= 0);
     }
-    nbhybrids = _cost->nbhybrids;
+    return 1;
+}
+
+int ManaCost::removeAll(int color)
+{
+    cost[color] = 0;
     return 1;
 }
 
 int ManaCost::addHybrid(int c1, int v1, int c2, int v2)
 {
-    ManaCostHybrid * h = NEW ManaCostHybrid(c1, v1, c2, v2);
-    hybrids[nbhybrids] = h;
-    nbhybrids++;
-    return nbhybrids;
+    hybrids.push_back(ManaCostHybrid(c1, v1, c2, v2));
+    return hybrids.size();
 }
 
 int ManaCost::addExtraCost(ExtraCost * _cost)
@@ -535,6 +600,15 @@ int ManaCost::addExtraCost(ExtraCost * _cost)
     return 1;
 }
 
+int ManaCost::addExtraCosts(ExtraCosts *_ecost)
+{
+    if (!extraCosts)
+        extraCosts = NEW ExtraCosts();
+    for(size_t i = 0; i < _ecost->costs.size(); i++)
+        extraCosts->costs.push_back(_ecost->costs[i]->clone());
+    return 1;
+}
+
 int ManaCost::isExtraPaymentSet()
 {
     if (!extraCosts)
@@ -568,22 +642,8 @@ int ManaCost::pay(ManaCost * _cost)
     int result = MANA_PAID;
     ManaCost * toPay = NEW ManaCost();
     toPay->copy(_cost);
-
-    if (toPay->kicker)
-    {
-        toPay->add(toPay->kicker);
-        if (!canAfford(toPay))
-        {
-            toPay->copy(_cost);
-        }
-        else
-        {
-            result = MANA_PAID_WITH_KICKER;
-        }
-    }
-
     ManaCost * diff = Diff(toPay);
-    for (int i = 0; i < Constants::MTG_NB_COLORS; i++)
+    for (int i = 0; i < Constants::NB_Colors; i++)
     {
         cost[i] = diff->getCost(i);
     }
@@ -608,7 +668,7 @@ int ManaCost::canAfford(ManaCost * _cost)
 
 int ManaCost::isPositive()
 {
-    for (int i = 0; i < Constants::MTG_NB_COLORS; i++)
+    for (int i = 0; i < Constants::NB_Colors; i++)
     {
 
         if (cost[i] < 0)
@@ -620,40 +680,39 @@ int ManaCost::isPositive()
 
 }
 
-void ManaCost::randomDiffHybrids(ManaCost * _cost, int diff[])
+void ManaCost::randomDiffHybrids(ManaCost * _cost, std::vector<int8_t>& diff)
 {
-    int _nbhybrids = _cost->nbhybrids;
-    for (int i = 0; i < _nbhybrids; i++)
+    for (size_t i = 0; i < _cost->hybrids.size(); i++)
     {
-        ManaCostHybrid * h = _cost->hybrids[i];
-        diff[h->color1 * 2 + 1] -= h->value1;
+        ManaCostHybrid& h = _cost->hybrids[i];
+        diff[h.color1 * 2 + 1] -= h.value1;
     }
 }
 
 /**
     starting from the end of the array (diff) 
 */
-int ManaCost::tryToPayHybrids(ManaCostHybrid * _hybrids[], int _nbhybrids, int diff[])
+int ManaCost::tryToPayHybrids(std::vector<ManaCostHybrid>& _hybrids, int _nbhybrids, std::vector<int8_t>& diff)
 {
     if (!_nbhybrids)
         return 1;
     int result = 0;
-    ManaCostHybrid * h = _hybrids[_nbhybrids - 1];
-    if (diff[h->color1 * 2 + 1] >= h->value1)
+    ManaCostHybrid& h = _hybrids[_nbhybrids - 1];
+    if (diff[h.color1 * 2 + 1] >= h.value1)
     {
-        diff[h->color1 * 2 + 1] -= h->value1;
+        diff[h.color1 * 2 + 1] -= h.value1;
         result = tryToPayHybrids(_hybrids, _nbhybrids - 1, diff);
         if (result)
             return 1;
-        diff[h->color1 * 2 + 1] += h->value1;
+        diff[h.color1 * 2 + 1] += h.value1;
     }
-    if (diff[h->color2 * 2 + 1] >= h->value2)
+    if (diff[h.color2 * 2 + 1] >= h.value2)
     {
-        diff[h->color2 * 2 + 1] -= h->value2;
+        diff[h.color2 * 2 + 1] -= h.value2;
         result = tryToPayHybrids(_hybrids, _nbhybrids - 1, diff);
         if (result)
             return 1;
-        diff[h->color2 * 2 + 1] += h->value2;
+        diff[h.color2 * 2 + 1] += h.value2;
     }
     return 0;
 }
@@ -661,14 +720,18 @@ int ManaCost::tryToPayHybrids(ManaCostHybrid * _hybrids[], int _nbhybrids, int d
 //compute the difference between two mana costs
 ManaCost * ManaCost::Diff(ManaCost * _cost)
 {
-    int diff[(Constants::MTG_NB_COLORS + 1) * 2];
-    diff[Constants::MTG_NB_COLORS * 2] = Constants::MTG_NB_COLORS;
-    for (int i = 0; i < Constants::MTG_NB_COLORS; i++)
+    if (!_cost) 
+        return NEW ManaCost(*this); //diff with null is equivalent to diff with 0
+
+    vector<int8_t> diff;
+    diff.resize((Constants::NB_Colors + 1) * 2);
+    diff[Constants::NB_Colors * 2] = Constants::NB_Colors;
+    for (int i = 0; i < Constants::NB_Colors; i++)
     {
         diff[i * 2] = i;
         diff[i * 2 + 1] = cost[i] - _cost->getCost(i);
     }
-    int hybridResult = tryToPayHybrids(_cost->hybrids, _cost->nbhybrids, diff);
+    int hybridResult = tryToPayHybrids(_cost->hybrids, _cost->hybrids.size(), diff);
     if (!hybridResult)
         randomDiffHybrids(_cost, diff);
 
@@ -676,7 +739,7 @@ ManaCost * ManaCost::Diff(ManaCost * _cost)
     int colorless_idx = Constants::MTG_COLOR_ARTIFACT * 2 + 1;
     if (diff[colorless_idx] < 0)
     {
-        for (int i = 0; i < Constants::MTG_NB_COLORS; i++)
+        for (int i = 0; i < Constants::NB_Colors; i++)
         {
             if (diff[i * 2 + 1] > 0)
             {
@@ -698,18 +761,18 @@ ManaCost * ManaCost::Diff(ManaCost * _cost)
     //Cost X
     if (_cost->hasX())
     {
-        diff[Constants::MTG_NB_COLORS * 2 + 1] = 0;
-        for (int i = 0; i < Constants::MTG_NB_COLORS; i++)
+        diff[Constants::NB_Colors * 2 + 1] = 0;
+        for (int i = 0; i < Constants::NB_Colors; i++)
         {
             if (diff[i * 2 + 1] > 0)
             {
-                diff[Constants::MTG_NB_COLORS * 2 + 1] += diff[i * 2 + 1];
+                diff[Constants::NB_Colors * 2 + 1] += diff[i * 2 + 1];
                 diff[i * 2 + 1] = 0;
             }
         }
     }
 
-    ManaCost * result = NEW ManaCost(diff, Constants::MTG_NB_COLORS + 1);
+    ManaCost * result = NEW ManaCost(diff, Constants::NB_Colors + 1);
     return result;
 
 }
@@ -717,22 +780,21 @@ ManaCost * ManaCost::Diff(ManaCost * _cost)
 string ManaCost::toString()
 {
     ostringstream oss;
-    for (int i = 0; i <= Constants::MTG_NB_COLORS; i++)
+    for (int i = 0; i <= Constants::NB_Colors; i++)
     {
         if (cost[i])
         {
             if ( i == Constants::MTG_COLOR_ARTIFACT)
-                oss << "{" << cost[i] << "}";
+                oss << "{" << getCost(i) << "}";
             else
-                for (int colorCount = 0; colorCount < cost[i]; colorCount++ )
+                for (int colorCount = 0; colorCount < getCost(i); colorCount++ )
                     oss << "{" << Constants::MTGColorChars[i] << "}";
         }
     }
 
-    for (unsigned int i = 0; i < nbhybrids; i++)
+    for (size_t i = 0; i < hybrids.size(); i++)
     {
-        if ( hybrids[i] != NULL )
-            oss << hybrids[i];
+        oss << hybrids[i];
     }
     return oss.str();
 }
@@ -740,9 +802,12 @@ string ManaCost::toString()
 #ifdef WIN32
 void ManaCost::Dump()
 {
+    //if(this->getConvertedCost())//uncomment when this is far too loud and clutters your output making other traces pointless.
+    //{
     DebugTrace( "\n===ManaCost===" );
     DebugTrace( this->toString() );
     DebugTrace( "\n=============" );
+    //}
 }
 
 #endif
@@ -766,7 +831,7 @@ void ManaPool::init()
 {
     ManaCost::init();
     WEvent * e = NEW WEventEmptyManaPool(this);
-    GameObserver::GetInstance()->receiveEvent(e);
+    player->getObserver()->receiveEvent(e);
 }
 
 ManaPool::ManaPool(Player * player) :
@@ -785,7 +850,7 @@ int ManaPool::remove(int color, int value)
     for (int i = 0; i < value; ++i)
     {
         WEvent * e = NEW WEventConsumeMana(color, this);
-        GameObserver::GetInstance()->receiveEvent(e);
+        player->getObserver()->receiveEvent(e);
     }
     return result;
 }
@@ -796,7 +861,7 @@ int ManaPool::add(int color, int value, MTGCardInstance * source)
     for (int i = 0; i < value; ++i)
     {
         WEvent * e = NEW WEventEngageMana(color, source, this);
-        GameObserver::GetInstance()->receiveEvent(e);
+        player->getObserver()->receiveEvent(e);
     }
     return result;
 }
@@ -806,12 +871,12 @@ int ManaPool::add(ManaCost * _cost, MTGCardInstance * source)
     if (!_cost)
         return 0;
     int result = ManaCost::add(_cost);
-    for (unsigned int i = 0; i < Constants::MTG_NB_COLORS; i++)
+    for (int i = 0; i < Constants::NB_Colors; i++)
     {
         for (int j = 0; j < _cost->getCost(i); j++)
         {
             WEvent * e = NEW WEventEngageMana(i, source, this);
-            GameObserver::GetInstance()->receiveEvent(e);
+            player->getObserver()->receiveEvent(e);
         }
     }
     return result;
@@ -819,22 +884,23 @@ int ManaPool::add(ManaCost * _cost, MTGCardInstance * source)
 
 int ManaPool::pay(ManaCost * _cost)
 {
-    int current[Constants::MTG_NB_COLORS];
-    for (unsigned int i = 0; i < Constants::MTG_NB_COLORS; i++)
+    vector<int> current;
+    for (int i = 0; i < Constants::NB_Colors; i++)
     {
-        current[i] = cost[i];
+        current.push_back(cost[i]);
     }
 
     int result = ManaCost::pay(_cost);
-    for (unsigned int i = 0; i < Constants::MTG_NB_COLORS; i++)
+    for (int i = 0; i < Constants::NB_Colors; i++)
     {
         int value = current[i] - cost[i];
         for (int j = 0; j < value; j++)
         {
             WEvent * e = NEW WEventConsumeMana(i, this);
-            GameObserver::GetInstance()->receiveEvent(e);
+            player->getObserver()->receiveEvent(e);
 
         }
     }
+    current.erase(current.begin(),current.end());
     return result;
 }
