@@ -27,13 +27,12 @@ JTexture* SimpleMenu::spadeRTex = NULL;
 JTexture* SimpleMenu::spadeLTex = NULL;
 JTexture* SimpleMenu::jewelTex = NULL;
 JTexture* SimpleMenu::sideTex = NULL;
-WFont* SimpleMenu::titleFont = NULL;
-hgeParticleSystem* SimpleMenu::stars = NULL;
 
-SimpleMenu::SimpleMenu(int id, JGuiListener* listener, int fontId, float x, float y, const char * _title, int _maxItems) :
-    JGuiController(id, listener), fontId(fontId)
+SimpleMenu::SimpleMenu(JGE* jge, int id, JGuiListener* listener, int fontId, float x, float y, const char * _title, int _maxItems, bool centerHorizontal, bool centerVertical)
+    : JGuiController(jge, id, listener), fontId(fontId), mCenterHorizontal(centerHorizontal), mCenterVertical(centerVertical)
 {
     autoTranslate = true;
+    isMultipleChoice = false;
     mHeight = 2 * kVerticalMargin;
     mWidth = 0;
     mX = x;
@@ -57,14 +56,14 @@ SimpleMenu::SimpleMenu(int id, JGuiListener* listener, int fontId, float x, floa
     jewel.reset(NEW JQuad(jewelTex, 1, 1, 3, 3));
     side = WResourceManager::Instance()->RetrieveQuad("menuside.png", 1, 1, 1, kPoleWidth, "menuside", RETRIEVE_MANAGE);
 
-    if (NULL == stars)
-        stars = NEW hgeParticleSystem(WResourceManager::Instance()->RetrievePSI("stars.psi", WResourceManager::Instance()->GetQuad("stars").get()));
+    stars = NEW hgeParticleSystem(WResourceManager::Instance()->RetrievePSI("stars.psi", WResourceManager::Instance()->GetQuad("stars").get()));
 
     stars->FireAt(mX, mY);
 }
 
 SimpleMenu::~SimpleMenu()
 {
+    SAFE_DELETE(stars);
 }
 
 void SimpleMenu::drawHorzPole(float x, float y, float width)
@@ -122,7 +121,8 @@ void SimpleMenu::drawVertPole(float x, float y, float height)
 
 void SimpleMenu::Render()
 {
-    WFont * titleFont = WResourceManager::Instance()->GetWFont(Fonts::SMALLFACE_FONT);
+    WFont * titleFont = WResourceManager::Instance()->GetWFont(fontId);
+    titleFont->SetColor(ARGB(250,255,255,255));//reseting color on passes as this is a shared font now.
     WFont * mFont = WResourceManager::Instance()->GetWFont(fontId);
     if (0 == mWidth)
     {
@@ -136,6 +136,13 @@ void SimpleMenu::Render()
         if ((!title.empty()) && (mWidth < titleFont->GetStringWidth(title.c_str()))) 
 			mWidth = titleFont->GetStringWidth(title.c_str());
         mWidth += 2 * kHorizontalMargin;
+
+        if (mCenterHorizontal)
+            mX = (SCREEN_WIDTH_F - mWidth) / 2;
+
+        if (mCenterVertical)
+            mY = (SCREEN_HEIGHT_F - mHeight) / 2;
+
         for (int i = 0; i < mCount; ++i)
         {
             float y = mY + kVerticalMargin + i * kLineHeight;
@@ -170,11 +177,11 @@ void SimpleMenu::Render()
     for (int i = startId; i < startId + maxItems; i++)
     {
         if (i > mCount - 1) break;
-        if ((static_cast<SimpleMenuItem*> (mObjects[i]))->mY - kLineHeight * startId < mY + height - kLineHeight + 7)
+        if ((static_cast<SimpleMenuItem*> (mObjects[i]))->getY() - kLineHeight * startId < mY + height - kLineHeight + 7)
         {
             if (static_cast<SimpleMenuItem*> (mObjects[i])->hasFocus())
             {
-                WResourceManager::Instance()->GetWFont(Fonts::MAIN_FONT)->DrawString(static_cast<SimpleMenuItem*> (mObjects[i])->desc.c_str(), mX
+                WResourceManager::Instance()->GetWFont(Fonts::MAIN_FONT)->DrawString(static_cast<SimpleMenuItem*> (mObjects[i])->getDescription().c_str(), mX
                                 + mWidth + 10, mY + 15);
                 mFont->SetColor(ARGB(255,255,255,0));
             }
@@ -217,7 +224,8 @@ void SimpleMenu::Add(int id, const char * text, string desc, bool forceFocus)
 {
     SimpleMenuItem * smi = NEW SimpleMenuItem(this, id, fontId, text, 0, mY + kVerticalMargin + mCount * kLineHeight,
                     (mCount == 0), autoTranslate);
-    smi->desc = desc;
+
+    smi->setDescription(desc);
     JGuiController::Add(smi);
     if (mCount <= maxItems) mHeight += kLineHeight;
     if (forceFocus)
@@ -237,6 +245,5 @@ void SimpleMenu::Close()
 void SimpleMenu::destroy()
 {
     SimpleMenu::jewel.reset();
-    SAFE_DELETE(SimpleMenu::stars);
     SAFE_DELETE(SimpleMenu::jewelTex);
 }
