@@ -15,12 +15,14 @@ using std::string;
 
 // NULL is sent in place of a MTGDeck since there is no way to create a MTGDeck without a proper deck file.
 // TestSuiteAI will be responsible for managing its own deck state.
-TestSuiteAI::TestSuiteAI(TestSuite * _suite, int playerId) :
-    AIPlayerBaka("testsuite", "testsuite", "baka.jpg", NULL)
+TestSuiteAI::TestSuiteAI(TestSuiteGame *tsGame, int playerId) :
+    AIPlayerBaka(tsGame->observer, "testsuite", "testsuite", "baka.jpg", NULL)
 {
-    this->game = _suite->buildDeck(playerId);
+    SAFE_DELETE(game); //game might have been set in the parent with default values
+    game = tsGame->buildDeck(this, playerId);
     game->setOwner(this);
-    suite = _suite;
+
+    suite = tsGame;
     timer = 0;
     playMode = MODE_TEST_SUITE;
     this->deckName = "Test Suite AI";
@@ -29,14 +31,13 @@ TestSuiteAI::TestSuiteAI(TestSuite * _suite, int playerId) :
 MTGCardInstance * TestSuiteAI::getCard(string action)
 {
     int mtgid = Rules::getMTGId(action);
-    if (mtgid) return Rules::getCardByMTGId(mtgid);
+    if (mtgid) return Rules::getCardByMTGId(observer, mtgid);
 
     //This mostly handles tokens
-    GameObserver * g = GameObserver::GetInstance();
     std::transform(action.begin(), action.end(), action.begin(), ::tolower);
     for (int i = 0; i < 2; i++)
     {
-        Player * p = g->players[i];
+        Player * p = observer->players[i];
         MTGGameZone * zones[] = { p->game->library, p->game->hand, p->game->inPlay, p->game->graveyard };
         for (int j = 0; j < 4; j++)
         {
@@ -54,9 +55,9 @@ MTGCardInstance * TestSuiteAI::getCard(string action)
     return NULL;
 }
 
-Interruptible * TestSuite::getActionByMTGId(int mtgid)
+Interruptible * TestSuiteGame::getActionByMTGId(int mtgid)
 {
-    ActionStack * as = GameObserver::GetInstance()->mLayers->stackLayer();
+    ActionStack * as = observer->mLayers->stackLayer();
     Interruptible * action = NULL;
     while ((action = as->getNext(action, 0, 0, 1)))
     {
@@ -76,17 +77,27 @@ int TestSuiteAI::displayStack()
 
 int TestSuiteAI::Act(float dt)
 {
-    GameObserver * g = GameObserver::GetInstance();
-    g->gameOver = NULL; // Prevent draw rule from losing the game
+    observer->gameOver = NULL; // Prevent draw rule from losing the game
+
+    //Last bits of initialization require to be done here, after the first "update" call of the game
+
+    if (suite->currentAction == 0)
+    {
+        for (int i = 0; i < 2; ++ i)
+            observer->players[i]->getManaPool()->copy(suite->initState.players[i]->getManaPool());
+    }
+
     if (playMode == MODE_AI && suite->aiMaxCalls)
     {
+        float static counter = 1.0f;
         suite->aiMaxCalls--;
         suite->timerLimit = 40; //TODO Remove this limitation when AI is not using a stupid timer anymore...
-        AIPlayerBaka::Act(dt);
+        AIPlayerBaka::Act(counter++);//dt);
     }
     if (playMode == MODE_HUMAN)
     {
-        g->mLayers->CheckUserInput(0);
+        observer->mLayers->CheckUserInput(0);
+        suite->currentAction++; //hack to avoid repeating the initialization of manapool
         return 1;
     }
 
@@ -95,15 +106,14 @@ int TestSuiteAI::Act(float dt)
     timer = 0;
 
     string action = suite->getNextAction();
-    g->mLayers->stackLayer()->Dump();
-    //  DamageResolverLayer * drl = g->mLayers->combatLayer();
+    observer->mLayers->stackLayer()->Dump();
     DebugTrace("TESTSUITE command: " << action);
 
-    if (g->mLayers->stackLayer()->askIfWishesToInterrupt == this)
+    if (observer->mLayers->stackLayer()->askIfWishesToInterrupt == this)
     {
         if (action.compare("no") != 0 && action.compare("yes") != 0)
         {
-            g->mLayers->stackLayer()->cancelInterruptOffer();
+            observer->mLayers->stackLayer()->cancelInterruptOffer();
             suite->currentAction--;
             return 1;
         }
@@ -113,15 +123,15 @@ int TestSuiteAI::Act(float dt)
     {
         //end of game
         suite->assertGame();
-        g->gameOver = g->players[0];
+        observer->gameOver = observer->players[0];
         DebugTrace("================================    END OF TEST   =======================\n");
         return 1;
     }
 
     if (action.compare("eot") == 0)
     {
-        if (g->getCurrentGamePhase() != Constants::MTG_PHASE_CLEANUP) suite->currentAction--;
-        g->userRequestNextGamePhase();
+        if (observer->getCurrentGamePhase() != MTG_PHASE_CLEANUP) suite->currentAction--;
+        observer->userRequestNextGamePhase();
     }
     else if (action.compare("human") == 0)
     {
@@ -142,67 +152,68 @@ int TestSuiteAI::Act(float dt)
             size_t found = action.find("goto ");
             string phase = action.substr(found + 5);
             int phaseToGo = 0;
-            for (int i = 0; i < Constants::NB_MTG_PHASES; i++)
+            for (int i = 0; i < NB_MTG_PHASES; i++)
             {
                 if (phase.find(Constants::MTGPhaseCodeNames[i]) != string::npos)
                 {
                     phaseToGo = i;
                 }
             }
-            if(g->currentGamePhase != phaseToGo)
+            if(observer->currentGamePhase != phaseToGo)
                 suite->currentAction--;
                 else
                 {
                 return 1;
                 }
-            GuiCombat * gc = g->mLayers->combatLayer();
-            if (ORDER == g->combatStep || DAMAGE == g->combatStep)
+            GuiCombat * gc = observer->mLayers->combatLayer();
+            if (ORDER == observer->combatStep || DAMAGE == observer->combatStep)
             {
                 gc->clickOK();
             }
             else
             {
-                g->userRequestNextGamePhase();
+                observer->userRequestNextGamePhase();
             }
         }
         else
         {
-            GuiCombat * gc = g->mLayers->combatLayer();
-            if (ORDER == g->combatStep || DAMAGE == g->combatStep)
+            GuiCombat * gc = observer->mLayers->combatLayer();
+            if (ORDER == observer->combatStep || DAMAGE == observer->combatStep)
                 gc->clickOK();
             else
-                g->userRequestNextGamePhase();
+                observer->userRequestNextGamePhase();
         }
     }
     else if (action.compare("yes") == 0)
-        g->mLayers->stackLayer()->setIsInterrupting(this);
+        observer->mLayers->stackLayer()->setIsInterrupting(this);
     else if (action.compare("endinterruption") == 0)
-        g->mLayers->stackLayer()->endOfInterruption();
+        observer->mLayers->stackLayer()->endOfInterruption();
     else if (action.compare("no") == 0)
     {
-        if (g->mLayers->stackLayer()->askIfWishesToInterrupt == this) g->mLayers->stackLayer()->cancelInterruptOffer();
+        if (observer->mLayers->stackLayer()->askIfWishesToInterrupt == this)
+            observer->mLayers->stackLayer()->cancelInterruptOffer();
     }
     else if (action.find("choice ") != string::npos)
     {
         DebugTrace("TESTSUITE choice !!!");
         int choice = atoi(action.substr(action.find("choice ") + 7).c_str());
-        g->mLayers->actionLayer()->doReactTo(choice);
+        observer->mLayers->actionLayer()->doReactTo(choice);
     }
     else if (action.find(" -momir- ") != string::npos)
     {
         int start = action.find(" -momir- ");
         int cardId = Rules::getMTGId(action.substr(start + 9).c_str());
         int cardIdHand = Rules::getMTGId(action.substr(0, start).c_str());
-        MTGMomirRule * a = ((MTGMomirRule *) g->mLayers->actionLayer()->getAbility(MTGAbility::MOMIR));
-        a->reactToClick(Rules::getCardByMTGId(cardIdHand), cardId);
-        g->mLayers->actionLayer()->stuffHappened = 1;
+        MTGMomirRule * a = ((MTGMomirRule *) observer->mLayers->actionLayer()->getAbility(MTGAbility::MOMIR));
+        a->reactToClick(Rules::getCardByMTGId(observer, cardIdHand), cardId);
+        observer->mLayers->actionLayer()->stuffHappened = 1;
     }
     else if (action.find("p1") != string::npos || action.find("p2") != string::npos)
     {
-        Player * p = g->players[1];
+        Player * p = observer->players[1];
         size_t start = action.find("p1");
-        if (start != string::npos) p = g->players[0];
-        g->cardClick(NULL, p);
+        if (start != string::npos) p = observer->players[0];
+        observer->cardClick(NULL, p);
     }
     else
     {
@@ -216,7 +227,7 @@ int TestSuiteAI::Act(float dt)
 
         if (toInterrupt)
         {
-            g->stackObjectClicked(toInterrupt);
+            observer->stackObjectClicked(toInterrupt);
             return 1;
         }
 
@@ -225,8 +236,8 @@ int TestSuiteAI::Act(float dt)
         {
             DebugTrace("TESTSUITE Clicking ON: " << card->name);
             card->currentZone->needShuffle = true; //mimic library shuffle
-            g->cardClick(card, card);
-            g->forceShuffleLibraries(); //mimic library shuffle
+            observer->cardClick(card, card);
+            observer->forceShuffleLibraries(); //mimic library shuffle
             return 1;
         }
     }
@@ -244,98 +255,30 @@ void TestSuiteActions::add(string s)
     nbitems++;
 }
 
-TestSuitePlayerData::TestSuitePlayerData()
-{
-    life = 20;
-    manapool = NEW ManaCost();
-}
-
-TestSuitePlayerData::~TestSuitePlayerData()
-{
-    SAFE_DELETE(manapool);
-}
-
-TestSuitePlayerZone::TestSuitePlayerZone()
-{
-    nbitems = 0;
-}
-
-void TestSuitePlayerZone::add(int cardId)
-{
-    cards[nbitems] = cardId;
-    nbitems++;
-}
 
 TestSuiteState::TestSuiteState()
 {
-
+    players.clear();
 }
 
+TestSuiteState::~TestSuiteState() 
+{
+    if(players.size())
+    {
+        if(players[0])
+            SAFE_DELETE(players[0]);
+        if(players[1])
+            SAFE_DELETE(players[1]);
+    }
+};
+
 void TestSuiteState::parsePlayerState(int playerId, string s)
 {
-    size_t limiter = s.find(":");
-    string areaS;
-    int area;
-    if (limiter != string::npos)
-    {
-        areaS = s.substr(0, limiter);
-        if (areaS.compare("graveyard") == 0)
-        {
-            area = 0;
-        }
-        else if (areaS.compare("library") == 0)
-        {
-            area = 1;
-        }
-        else if (areaS.compare("hand") == 0)
-        {
-            area = 2;
-        }
-        else if (areaS.compare("inplay") == 0 || areaS.compare("battlefield") == 0)
-        {
-            area = 3;
-        }
-        else if (areaS.compare("life") == 0)
-        {
-            playerData[playerId].life = atoi((s.substr(limiter + 1)).c_str());
-            return;
-        }
-        else if (areaS.compare("manapool") == 0)
-        {
-            SAFE_DELETE(playerData[playerId].manapool);
-            playerData[playerId].manapool = ManaCost::parseManaCost(s.substr(limiter + 1));
-            return;
-        }
-        else
-        {
-            return; // ERROR
-        }
-        s = s.substr(limiter + 1);
-        while (s.size())
-        {
-            unsigned int value;
-            limiter = s.find(",");
-            if (limiter != string::npos)
-            {
-                string ss = s.substr(0, limiter); // ss is needed because trim requires a non-const reference,
-                value = Rules::getMTGId(trim(ss)); // while in g++ functions cannot take non-const references from temporary values
-                s = s.substr(limiter + 1);
-            }
-            else
-            {
-                value = Rules::getMTGId(trim(s));
-                s = "";
-            }
-            if (value) playerData[playerId].zones[area].add(value);
-        }
-    }
-    else
-    {
-        //ERROR
-    }
+    players[playerId]->parseLine(s);
 }
 
-string TestSuite::getNextAction()
+
+string TestSuiteGame::getNextAction()
 {
     currentAction++;
     if (actions.nbitems && currentAction <= actions.nbitems)
@@ -345,25 +288,15 @@ string TestSuite::getNextAction()
     return "";
 }
 
-MTGPlayerCards * TestSuite::buildDeck(int playerId)
+void TestSuiteGame::handleResults(bool wasAI, int error)
 {
-    int list[100];
-    int nbcards = 0;
-    for (int j = 0; j < 4; j++)
-    {
-        for (int k = 0; k < initState.playerData[playerId].zones[j].nbitems; k++)
-        {
-            int cardid = initState.playerData[playerId].zones[j].cards[k];
-            list[nbcards] = cardid;
-            nbcards++;
-        }
-    }
-    MTGPlayerCards * deck = NEW MTGPlayerCards(list, nbcards);
-    return deck;
-}
+    testsuite->handleResults(wasAI, error);
+};
 
-void TestSuite::initGame()
+
+void TestSuite::initGame(GameObserver* g)
 {
+    observer = g;
     //The first test runs slowly, the other ones run faster.
     //This way a human can see what happens when testing a specific file,
     // or go faster when it comes to the whole test suite.
@@ -376,61 +309,14 @@ void TestSuite::initGame()
     {
         timerLimit = 3;
     }
-    //Put the GameObserver in the initial state
-    GameObserver * g = GameObserver::GetInstance();
-    DebugTrace("TESTSUITE Init Game");
-    g->phaseRing->goToPhase(initState.phase, g->players[0]);
-    g->currentGamePhase = initState.phase;
-    for (int i = 0; i < 2; i++)
-    {
-        AIPlayer * p = (AIPlayer *) (g->players[i]);
-        p->forceBestAbilityUse = forceAbility;
-        p->life = initState.playerData[i].life;
-        p->getManaPool()->copy(initState.playerData[i].manapool);
-        MTGGameZone * playerZones[] = { p->game->graveyard, p->game->library, p->game->hand, p->game->inPlay };
-        for (int j = 0; j < 4; j++)
-        {
-            MTGGameZone * zone = playerZones[j];
-            for (int k = 0; k < initState.playerData[i].zones[j].nbitems; k++)
-            {
-                MTGCardInstance * card = Rules::getCardByMTGId(initState.playerData[i].zones[j].cards[k]);
-                if (card && zone != p->game->library)
-                {
-                    if (zone == p->game->inPlay)
-                    {
-                        MTGCardInstance * copy = p->game->putInZone(card, p->game->library, p->game->stack);
-                        Spell * spell = NEW Spell(copy);
-                        spell->resolve();
-                        if (!summoningSickness && p->game->inPlay->nb_cards > k) p->game->inPlay->cards[k]->summoningSickness = 0;
-                        delete spell;
-                    }
-                    else
-                    {
-                        if (!p->game->library->hasCard(card))
-                        {
-                            LOG ("TESTUITE ERROR, CARD NOT FOUND IN LIBRARY\n");
-                        }
-                        p->game->putInZone(card, p->game->library, zone);
-                    }
-                }
-                else
-                {
-                    if (!card)
-                    {
-                        LOG ("TESTUITE ERROR, card is NULL\n");
-                    }
-                }
-            }
-            zone->cardsSeenThisTurn.clear(); //don't consider those cards as having moved in this area during this turn
-        }
-        p->game->stack->cardsSeenThisTurn.clear(); //don't consider those cards as having moved in this area during this turn
-    }
-DebugTrace("TESTUITE Init Game Done !");
+
+    TestSuiteGame::initGame();
 }
-int TestSuite::Log(const char * text)
+
+int TestSuiteGame::Log(const char * text)
 {
-    ofstream file(JGE_GET_RES("test/results.html").c_str(), ios_base::app);
-    if (file)
+    ofstream file;
+    if (JFileSystem::GetInstance()->openForWrite(file, "test/results.html", ios_base::app))
     {
         file << text;
         file << "\n";
@@ -441,74 +327,91 @@ int TestSuite::Log(const char * text)
     return 1;
 
 }
-int TestSuite::assertGame()
+
+void TestSuiteGame::assertGame()
 {
+    mMutex.lock();
     //compare the game state with the results
     char result[4096];
-    sprintf(result, "<h3>%s</h3>", files[currentfile - 1].c_str());
+    sprintf(result, "<h3>%s</h3>", filename.c_str());
     Log(result);
 
     int error = 0;
     bool wasAI = false;
 
-    GameObserver * g = GameObserver::GetInstance();
-    if (g->currentGamePhase != endState.phase)
+    if (observer->currentGamePhase != endState.phase)
     {
         sprintf(result, "<span class=\"error\">==phase problem. Expected [ %s ](%i), got [ %s ](%i)==</span><br />", 
             Constants::MTGPhaseNames[endState.phase],endState.phase,
-            Constants::MTGPhaseNames[g->currentGamePhase], g->currentGamePhase);
+            Constants::MTGPhaseNames[observer->currentGamePhase], observer->currentGamePhase);
         Log(result);
         error++;
     }
     for (int i = 0; i < 2; i++)
     {
-        TestSuiteAI * p = (TestSuiteAI *) (g->players[i]);
+        TestSuiteAI * p = (TestSuiteAI *) (observer->players[i]);
         if (p->playMode == Player::MODE_AI) wasAI = true;
 
-        if (p->life != endState.playerData[i].life)
+        if (p->life != endState.players[i]->life)
         {
             sprintf(result, "<span class=\"error\">==life problem for player %i. Expected %i, got %i==</span><br />", i,
-                            endState.playerData[i].life, p->life);
+                            endState.players[i]->life, p->life);
             Log(result);
             error++;
         }
-        if (!p->getManaPool()->canAfford(endState.playerData[i].manapool))
+        if (p->poisonCount != endState.players[i]->poisonCount)
+        {
+            sprintf(result, "<span class=\"error\">==poison counter problem for player %i. Expected %i, got %i==</span><br />", i,
+                            endState.players[i]->poisonCount, p->poisonCount);
+            Log(result);
+            error++;
+        }        if (!p->getManaPool()->canAfford(endState.players[i]->getManaPool()))
         {
             sprintf(result, "<span class=\"error\">==Mana problem. Was expecting %i but got %i for player %i==</span><br />",
-                            endState.playerData[i].manapool->getConvertedCost(), p->getManaPool()->getConvertedCost(), i);
+                            endState.players[i]->getManaPool()->getConvertedCost(), p->getManaPool()->getConvertedCost(), i);
             Log(result);
             error++;
         }
-        if (!endState.playerData[i].manapool->canAfford(p->getManaPool()))
+        if (!endState.players[i]->getManaPool()->canAfford(p->getManaPool()))
         {
             sprintf(result, "<span class=\"error\">==Mana problem. Was expecting %i but got %i for player %i==</span><br />",
-                            endState.playerData[i].manapool->getConvertedCost(), p->getManaPool()->getConvertedCost(), i);
+                            endState.players[i]->getManaPool()->getConvertedCost(), p->getManaPool()->getConvertedCost(), i);
             Log(result);
+
+            if ( endState.players[i]->getManaPool()->getConvertedCost() == p->getManaPool()->getConvertedCost())
+            {
+                sprintf(result, "<span class=\"error\">====(Apparently Mana Color issues since converted cost is the same)==</span><br />");
+                Log(result);
+            }
             error++;
 
         }
         MTGGameZone * playerZones[] = { p->game->graveyard, p->game->library, p->game->hand, p->game->inPlay };
+        MTGGameZone * endstateZones[] = { endState.players[i]->game->graveyard,
+                                         endState.players[i]->game->library,
+                                         endState.players[i]->game->hand,
+                                         endState.players[i]->game->inPlay };
         for (int j = 0; j < 4; j++)
         {
             MTGGameZone * zone = playerZones[j];
-            if (zone->nb_cards != endState.playerData[i].zones[j].nbitems)
+            if (zone->nb_cards != endstateZones[j]->nb_cards)
             {
                 sprintf(
                                 result,
                                 "<span class=\"error\">==Card number not the same in player %i's %s==, expected %i, got %i</span><br />",
-                                i, zone->getName(), endState.playerData[i].zones[j].nbitems, zone->nb_cards);
+                                i, zone->getName(), endstateZones[j]->nb_cards, zone->nb_cards);
                 Log(result);
                 error++;
             }
-            for (int k = 0; k < endState.playerData[i].zones[j].nbitems; k++)
+            for (size_t k = 0; k < (size_t)endstateZones[j]->nb_cards; k++)
             {
-                int cardid = endState.playerData[i].zones[j].cards[k];
-                if (cardid != -1)
-                {
-                    MTGCardInstance * card = Rules::getCardByMTGId(cardid);
-                    if (!card || !zone->hasCard(card))
+                MTGCardInstance* cardToCheck = (k<endstateZones[j]->cards.size())?endstateZones[j]->cards[k]:0;
+                if(cardToCheck)
+                {   // Can be NULL if used "*" in the testcase.
+                    MTGCardInstance* card = Rules::getCardByMTGId(observer, cardToCheck->getId());
+                    if (card != 0 && !zone->hasCard(card))
                     {
-                        sprintf(result, "<span class=\"error\">==Card ID not the same. Didn't find %i</span><br />", cardid);
+                        sprintf(result, "<span class=\"error\">==Card ID not the same. Didn't find %i</span><br />", card->getId());
                         Log(result);
                         error++;
                     }
@@ -516,13 +419,23 @@ int TestSuite::assertGame()
             }
         }
     }
+    handleResults(wasAI, error);
+
+    if(!error)
+        Log("<span class=\"success\">==Test Succesful !==</span>");
+    else
+        Log("<span class=\"error\">==Test Failed !==</span>");
+    mMutex.unlock();
+}
+
+void TestSuite::handleResults(bool wasAI, int error)
+{
     if (wasAI)
     {
         nbAITests++;
         if (error)
         {
             nbAIFailed++;
-            return 0;
         }
     }
     else
@@ -531,18 +444,29 @@ int TestSuite::assertGame()
         if (error)
         {
             nbFailed++;
-            return 0;
         }
     }
-    Log("<span class=\"success\">==Test Succesful !==</span>");
-    return 1;
 }
 
-TestSuite::TestSuite(const char * filename, MTGAllCards* _collection)
+TestSuite::~TestSuite()
+{
+  mProcessing = false;
+  while(mWorkerThread.size())
+  {
+    mWorkerThread.back()->join();
+    SAFE_DELETE(mWorkerThread.back());
+    mWorkerThread.pop_back();
+  }
+
+  observer = 0;
+}
+
+TestSuite::TestSuite(const char * filename)
+    : TestSuiteGame(0), mRules(0), mProcessing(false)
 {
-    collection = _collection;
     timerLimit = 0;
-    wagic::ifstream file(filename);
+    testsuite = this;
+    
     std::string s;
     nbfiles = 0;
     currentfile = 0;
@@ -554,12 +478,17 @@ TestSuite::TestSuite(const char * filename, MTGAllCards* _collection)
     seed = 0;
     forceAbility = false;
     aiMaxCalls = -1;
-    if (file)
+    startTime = JGEGetTime();
+    endTime = startTime;
+    std::string contents;
+    if (JFileSystem::GetInstance()->readIntoString(filename, contents))
     {
-        while (std::getline(file, s))
+        std::stringstream stream(contents);
+        while (std::getline(stream, s))
         {
             if (!s.size()) continue;
             if (s[s.size() - 1] == '\r') s.erase(s.size() - 1); //Handle DOS files
+            if (!s.size()) continue;
             if (s[0] == '/' && s[1] == '*') comment = 1;
             if (s[0] && s[0] != '#' && !comment)
             {
@@ -568,11 +497,15 @@ TestSuite::TestSuite(const char * filename, MTGAllCards* _collection)
             }
             if (s[0] == '*' && s[1] == '/') comment = 0;
         }
-        file.close();
     }
 
-    ofstream file2(JGE_GET_RES("/test/results.html").c_str());
-    if (file2)
+    //If more than 1 test, prefecth names to make the suite run faster
+    if (nbfiles > 1)
+        MTGCollection()->prefetchCardNameCache();
+
+
+    ofstream file2;
+    if (JFileSystem::GetInstance()->openForWrite(file2, "/test/results.html"))
     {
         file2 << "<html><head>";
 #ifdef WIN32
@@ -585,22 +518,57 @@ TestSuite::TestSuite(const char * filename, MTGAllCards* _collection)
         file2.close();
     }
 
+    SAFE_DELETE(observer);
 }
 
 int TestSuite::loadNext()
 {
+
+    endTime = JGEGetTime();
     summoningSickness = 0;
     seed = 0;
     aiMaxCalls = -1;
     if (!nbfiles) return 0;
-    if (currentfile >= nbfiles) return 0;
-    currentfile++;
-    if (!load(files[currentfile - 1].c_str()))
+
+    filename = getNextFile();
+    if (filename == "") {
+        // we let GameStateDuel delete the latest gameObserver.
+        mProcessing = false;
+        observer = 0;
+        return 0;
+    }
+
+    if (filename == "+pregametests")
+    {
+        pregameTests();
+        return loadNext();
+    }
+
+    if(!mProcessing)
+    {   // "I don't like to wait" mode
+        mProcessing = true;
+        while(mWorkerThread.size())
+        {
+          mWorkerThread.back()->join();
+          SAFE_DELETE(mWorkerThread.back());
+          mWorkerThread.pop_back();
+        }
+
+        size_t thread_count = 1;
+#ifdef QT_CONFIG
+        thread_count = QThread::idealThreadCount();
+#elif defined(IOS)
+        thread_count = 6;
+#endif
+        for(size_t i = 0; i < (thread_count-1); i++)
+            mWorkerThread.push_back(new boost::thread(ThreadProc, this));
+    }
+
+    cleanup();
+    if (!load())
         return loadNext();
     else
         cout << "Starting test : " << files[currentfile - 1] << endl;
-    //load(files[currentfile].c_str());
-    //currentfile++;
     return currentfile;
 }
 
@@ -609,61 +577,47 @@ void TestSuiteActions::cleanup()
     nbitems = 0;
 }
 
-void TestSuitePlayerZone::cleanup()
+void TestSuiteState::cleanup(TestSuiteGame* tsGame)
 {
-    nbitems = 0;
-}
-
-void TestSuitePlayerData::cleanup()
-{
-    if (manapool) delete manapool;
-    manapool = NULL;
-    manapool = NEW ManaCost();
-    for (int i = 0; i < 5; i++)
+    for (size_t i = 0; i < players.size(); i++)
     {
-        zones[i].cleanup();
+        SAFE_DELETE(players[i]);
     }
-    life = 20;
-}
+    players.clear();
 
-void TestSuiteState::cleanup()
-{
-    for (int i = 0; i < 2; i++)
-    {
-        playerData[i].cleanup();
-    }
+    players.push_back(new TestSuiteAI(tsGame, 0));
+    players.push_back(new TestSuiteAI(tsGame, 1));
 }
 
 void TestSuite::cleanup()
 {
     currentAction = 0;
-    initState.cleanup();
-    endState.cleanup();
+    observer = 0;
+    initState.cleanup(this);
+    endState.cleanup(this);
     actions.cleanup();
-    loadRandValues("");
 }
 
-int TestSuite::load(const char * _filename)
+
+bool TestSuiteGame::load()
 {
     summoningSickness = 0;
     forceAbility = false;
     gameType = GAME_TYPE_CLASSIC;
-    char filename[4096];
-    sprintf(filename, JGE_GET_RES("/test/%s").c_str(), _filename);
-    wagic::ifstream file(filename);
+
     std::string s;
-    loadRandValues("");
 
     int state = -1;
 
-    //  std::cout << std::endl << std::endl << "!!!" << file << std::endl << std::endl;
-    if (file)
+    std::string contents;
+    if (JFileSystem::GetInstance()->readIntoString("test/"+filename, contents))
     {
-        cleanup();
-        while (std::getline(file, s))
+        std::stringstream stream(contents);
+        while (std::getline(stream, s))
         {
             if (!s.size()) continue;
             if (s[s.size() - 1] == '\r') s.erase(s.size() - 1); //Handle DOS files
+            if (!s.size()) continue;
             if (s[0] == '#') continue;
             std::transform(s.begin(), s.end(), s.begin(), ::tolower);
             if (s.compare("summoningsickness") == 0)
@@ -681,11 +635,6 @@ int TestSuite::load(const char * _filename)
                 seed = atoi(s.substr(5).c_str());
                 continue;
             }
-            if (s.find("rvalues:") == 0)
-            {
-                loadRandValues(s.substr(8).c_str());
-                continue;
-            }
             if (s.find("aicalls ") == 0)
             {
                 aiMaxCalls = atoi(s.substr(8).c_str());
@@ -773,13 +722,12 @@ int TestSuite::load(const char * _filename)
                 break;
             }
         }
-        file.close();
     }
     else
     {
-        return 0;
+        return false;
     }
-    return 1;
+    return true;
 }
 
 void TestSuite::pregameTests()
@@ -796,4 +744,168 @@ void TestSuite::pregameTests()
         if (!sb.unitTest()) nbFailed++;
     }
 }
+
+void TestSuite::ThreadProc(void* inParam)
+{
+    LOG("Entering TestSuite::ThreadProc");
+    TestSuite* instance = reinterpret_cast<TestSuite*>(inParam);
+    if (instance)
+    {
+        string filename;
+        float counter = 1.0f;
+        while(instance->mProcessing && (filename = instance->getNextFile()) != "")
+        {
+            TestSuiteGame theGame(instance, filename);
+            if(theGame.isOK)
+            {
+                theGame.observer->loadTestSuitePlayer(0, &theGame);
+                theGame.observer->loadTestSuitePlayer(1, &theGame);
+
+                theGame.observer->startGame(theGame.gameType, instance->mRules);
+                theGame.initGame();
+
+                while(!theGame.observer->gameOver)
+                    theGame.observer->Update(counter++);
+
+                if(theGame.observer->mGameType != GAME_TYPE_MOMIR)
+                {
+                    stringstream stream;
+                    stream << *(theGame.observer);
+                    theGame.observer->load(stream.str(), false, &theGame);
+                    theGame.assertGame();
+                }
+            }
+        }
+    }
+    LOG("Leaving TestSuite::ThreadProc");
+}
+
+boost::mutex TestSuiteGame::mMutex;
+
+TestSuiteGame::~TestSuiteGame()
+{
+    SAFE_DELETE(observer);
+}
+
+TestSuiteGame::TestSuiteGame(TestSuite* testsuite)
+    : summoningSickness(0), forceAbility(false), gameType(GAME_TYPE_CLASSIC), timerLimit(0),
+      currentAction(0), observer(0), testsuite(testsuite)
+{
+}
+
+TestSuiteGame::TestSuiteGame(TestSuite* testsuite, string _filename)
+    : summoningSickness(0), forceAbility(false), gameType(GAME_TYPE_CLASSIC), timerLimit(3),
+      currentAction(0), observer(0), testsuite(testsuite)
+{
+    filename = _filename;
+    observer = new GameObserver();
+
+    initState.cleanup(this);
+    endState.cleanup(this);
+
+    isOK = load();
+}
+
+void TestSuiteGame::ResetManapools()
+{
+    for (int i = 0; i < 2; ++ i)
+        observer->players[i]->getManaPool()->copy(initState.players[i]->getManaPool());
+}
+
+
+void TestSuiteGame::initGame()
+{
+    DebugTrace("TESTSUITE Init Game");
+    observer->phaseRing->goToPhase(initState.phase, observer->players[0], false);
+    observer->currentGamePhase = initState.phase;
+
+    observer->resetStartupGame();
+
+    for (int i = 0; i < 2; i++)
+    {        
+        AIPlayerBaka * p = (AIPlayerBaka *) (observer->players[i]);
+        p->forceBestAbilityUse = forceAbility;
+        p->life = initState.players[i]->life;
+        p->poisonCount = initState.players[i]->poisonCount;
+        stringstream stream;
+        stream << initState.players[i]->getRandomGenerator()->saveLoadedRandValues(stream);
+        p->getRandomGenerator()->loadRandValues(stream.str());
+        MTGGameZone * playerZones[] = { p->game->graveyard, p->game->library, p->game->hand, p->game->inPlay };
+        MTGGameZone * loadedPlayerZones[] = { initState.players[i]->game->graveyard,
+                                              initState.players[i]->game->library,
+                                              initState.players[i]->game->hand,
+                                              initState.players[i]->game->inPlay };
+        for (int j = 0; j < 4; j++)
+        {
+            MTGGameZone * zone = playerZones[j];
+            for (size_t k = 0; k < loadedPlayerZones[j]->cards.size(); k++)
+            {
+                MTGCardInstance * card = Rules::getCardByMTGId(observer, loadedPlayerZones[j]->cards[k]->getId());
+                if (card && zone != p->game->library)
+                {
+                    if (zone == p->game->inPlay)
+                    {
+                        MTGCardInstance * copy = p->game->putInZone(card, p->game->library, p->game->stack);
+                        Spell * spell = NEW Spell(observer, copy);
+                        spell->resolve();
+                        if (!summoningSickness && (size_t)p->game->inPlay->nb_cards > k) p->game->inPlay->cards[k]->summoningSickness = 0;
+                        delete spell;
+                    }
+                    else
+                    {
+                        if (!p->game->library->hasCard(card))
+                        {
+                            LOG ("TESTUITE ERROR, CARD NOT FOUND IN LIBRARY\n");
+                        }
+                        p->game->putInZone(card, p->game->library, zone);
+                    }
+                }
+                else
+                {
+                    if (!card)
+                    {
+                        LOG ("TESTUITE ERROR, card is NULL\n");
+                    }
+                }
+            }
+            zone->cardsSeenThisTurn.clear(); //don't consider those cards as having moved in this area during this turn
+        }
+        p->game->stack->cardsSeenThisTurn.clear(); //don't consider those cards as having moved in this area during this turn
+
+    }
+    DebugTrace("TESTUITE Init Game Done !");
+}
+
+MTGPlayerCards * TestSuiteGame::buildDeck(Player* player, int playerId)
+{
+    int list[100];
+    int nbcards = 0;
+    MTGPlayerCards * deck = NULL;
+
+    if(initState.players.size() > (size_t)playerId)
+    {
+        MTGGameZone * loadedPlayerZones[] = { initState.players[playerId]->game->graveyard,
+            initState.players[playerId]->game->library,
+            initState.players[playerId]->game->hand,
+            initState.players[playerId]->game->inPlay };
+
+        for (int j = 0; j < 4; j++)
+        {
+            for (size_t k = 0; k < loadedPlayerZones[j]->cards.size(); k++)
+            {
+                int cardid = loadedPlayerZones[j]->cards[k]->getId();
+                list[nbcards] = cardid;
+                nbcards++;
+            }
+        }
+        deck = NEW MTGPlayerCards(player, list, nbcards);
+    }
+    else
+    {
+        deck = NEW MTGPlayerCards();
+    }
+
+    return deck;
+}
+
 #endif
