@@ -3,14 +3,16 @@
 #include "Translate.h"
 #include "WResourceManager.h"
 #include "TranslateKeys.h"
-
+#ifdef SDL_CONFIG
+#include <SDL.h>
+#endif
 using std::string;
 using std::map;
 
 static map<const LocalKeySym, KeyRep> fattable;
 static map<const JButton, KeyRep> slimtable;
 
-#if defined(LINUX) || defined (IOS)
+#if defined(LINUX) || defined (IOS) || defined (ANDROID) ||  defined (SDL_CONFIG) || defined (QT_CONFIG)
 const KeyRep& translateKey(LocalKeySym key)
 {
     {
@@ -20,21 +22,22 @@ const KeyRep& translateKey(LocalKeySym key)
     }
 
     char* str = NULL;
-#if !defined(QT_CONFIG) && !defined(IOS)
+
+#if !defined(QT_CONFIG) && !defined(IOS) && !defined (SDL_CONFIG)
     str = XKeysymToString(key);
-#endif // QT_CONFIG
+#elif defined (SDL_CONFIG)
+    str = (char*)SDL_GetKeyName(key);
+#endif
     if (!str)
     {
         str = NEW char[11];
-        sprintf(str, "%lu", key);
+        sprintf(str, "%lu", (long unsigned int)key);  //TODO: Wagic is not supposed to know that a key actually is an unsingned long, so this part should probably be platform specific (move to JGE ?)
     }
     const KeyRep k = make_pair(str, static_cast<JQuad*>(NULL));
     fattable[key] = k;
     return fattable[key];
 }
-#else
-#ifdef WIN32
-
+#elif defined(WIN32)
 const KeyRep& translateKey(LocalKeySym key)
 {
     {
@@ -130,7 +133,6 @@ const KeyRep& translateKey(LocalKeySym key)
     return k;
 }
 #endif
-#endif
 
 const KeyRep& translateKey(JButton key) {
   /*
