@@ -2,13 +2,15 @@
 
 #include "utils.h"
 #include <JFileSystem.h>
-#include "GameOptions.h"
 #include "WResourceManager.h"
+#include "WCachedResource.h"
 #include <hge/hgeparticle.h>
 
 #ifdef WITH_FMOD
 #endif
 
+const std::string kPlaceholderID("placeholder");
+
 //WResource
 WResource::~WResource()
 {
@@ -39,6 +41,9 @@ void WResource::deadbolt()
 void WResource::lock()
 {
     if (locks < WRES_MAX_LOCK) locks++;
+    // watch out for this - if the lock count increases beyond a reasonable threshold, 
+    // someone is probably not releasing the texture correctly
+    assert(locks < 50);
 }
 
 void WResource::unlock(bool force)
@@ -97,7 +102,18 @@ JQuadPtr WCachedTexture::GetQuad(float offX, float offY, float width, float heig
     if (width == 0.0f || width > static_cast<float> (texture->mWidth)) width = static_cast<float> (texture->mWidth);
     if (height == 0.0f || height > static_cast<float> (texture->mHeight)) height = static_cast<float> (texture->mHeight);
 
-	std::map<string, JQuadPtr>::iterator iter = mTrackedQuads.find(resname);
+    // If we're fetching a card resource, but it's not available yet, we'll be attempting to get the Quad from the temporary back image.
+    // If that's the case, don't stash a separate tracked quad entry for each card name in the the Back/BackThumbnail's resource
+    string resource(resname);
+    if (mFilename == kGenericCard || mFilename == kGenericThumbCard)
+    {
+        // if we're the back or thumb_back file, but we've been asked for a card ID, then assign it
+        // a placeholder ID.  Reason being, hotspots on quads are different (ie centered) for card images, so we'll store a separate quad for cards 
+        if (resname != kGenericCardID && resname != kGenericCardThumbnailID)
+            resource = kPlaceholderID;
+    }
+
+	std::map<string, JQuadPtr>::iterator iter = mTrackedQuads.find(resource);
 	if (iter != mTrackedQuads.end())
 		return iter->second;
 
@@ -105,7 +121,7 @@ JQuadPtr WCachedTexture::GetQuad(float offX, float offY, float width, float heig
 
     //Update JQ's values to what we called this with.
     quad->SetTextureRect(offX, offY, width, height);
-	mTrackedQuads.insert(std::pair<string, JQuadPtr>(resname, quad));
+	mTrackedQuads.insert(std::pair<string, JQuadPtr>(resource, quad));
     return quad;
 
 }
@@ -134,8 +150,7 @@ unsigned long WCachedTexture::size()
     if (!texture) return 0;
 
     unsigned int pixel_size = 4;
-#if defined WIN32 || defined LINUX || defined IOS
-#else
+#if defined(PSP)
     pixel_size = JRenderer::GetInstance()->PixelSize(texture->mTextureFormat);
 #endif
     return texture->mTexHeight * texture->mTexWidth * pixel_size;
@@ -334,7 +349,7 @@ bool WCachedParticles::Attempt(const string& filename, int submode, int & error)
     // we're actually trying to read more than the file size now, but it's no problem.
     // Note that this fix is only to avoid the largest problems, filling a structure
     // by directly reading a file, is really a bad idea ...
-    fileSys->ReadFile(&(particles->nEmission), sizeof(hgeParticleSystemInfo));
+    fileSys->ReadFile(&(particles->nEmission), sizeof(hgeParticleSystemInfo) - sizeof(void*));
     fileSys->CloseFile();
 
     particles->sprite = NULL;
