@@ -273,7 +273,7 @@ bool WGuiMenu::yieldFocus()
 
 //WGuiList
 WGuiList::WGuiList(string name, WSyncable * syncme) :
-    WGuiMenu(JGE_BTN_DOWN, JGE_BTN_UP, false, syncme)
+    WGuiMenu(JGE_BTN_DOWN, JGE_BTN_UP, false, syncme), startWindow(-1), endWindow(-1)
 {
     failMsg = "NO OPTIONS AVAILABLE";
     width = SCREEN_WIDTH - 10;
@@ -360,10 +360,11 @@ void WGuiList::Render()
     //Render items.
     if (start >= 0)
     {
+        int pos;
         //Render current underlay.
         if (currentItem >= 0 && currentItem < nbitems && items[currentItem]->Visible()) items[currentItem]->Underlay();
 
-        for (int pos = 0; pos < nbitems; pos++)
+        for (pos = 0; pos < nbitems; pos++)
         {
             if (!items[pos]->Visible()) continue;
 
@@ -386,6 +387,9 @@ void WGuiList::Render()
             break;
         }
 
+        startWindow = start;
+        endWindow = pos;
+
         //Draw scrollbar
         if (listHeight > SCREEN_HEIGHT && listSelectable > 1)
         {
@@ -418,6 +422,43 @@ void WGuiList::ButtonPressed(int controllerId, int controlId)
     it->ButtonPressed(controllerId, controlId);
 }
 
+bool WGuiList::CheckUserInput(JButton key)
+{
+    JGE * mEngine = JGE::GetInstance();
+    int i, j;
+
+    if ((key == JGE_BTN_OK) && mEngine->GetLeftClickCoordinates(i, j))
+    {   // a dude clicked somwhere, we're gonna select the closest object from where he clicked
+        int n = currentItem;
+        unsigned int distance2;
+        unsigned int minDistance2 = -1;
+        int begin = (startWindow == -1) ? 0 : startWindow;
+        int end = (endWindow == -1) ? items.size() : endWindow;
+        for(int k = begin; k < end; k++)
+        {
+          WGuiBase* pItem = (items[k]);
+          distance2 = static_cast<unsigned int>((pItem->getY() - j) * (pItem->getY() - j) + (pItem->getX() - i) * (pItem->getX() - i));
+          if (distance2 < minDistance2 && pItem->Selectable())
+          {
+              minDistance2 = distance2;
+              n = k;
+          }
+        }
+
+        if (n != currentItem && items[n]->Selectable())
+        {
+            setSelected(n);
+            mEngine->LeftClickedProcessed();
+            if (sync) syncMove();
+            return true;
+        }
+    }
+
+//    mEngine->LeftClickedProcessed();
+    return WGuiMenu::CheckUserInput(key);
+}
+
+
 string WDecoEnum::lookupVal(int value)
 {
 
@@ -503,7 +544,7 @@ void WDecoConfirm::Entering(JButton key)
 
     SAFE_DELETE(confirmMenu);
     mState = OP_CONFIRMED;
-    confirmMenu = NEW SimpleMenu(444, listener, Fonts::MENU_FONT, 50, 170);
+    confirmMenu = NEW SimpleMenu(JGE::GetInstance(), 444, listener, Fonts::MENU_FONT, 50, 170);
     confirmMenu->Add(1, confirm.c_str());
     confirmMenu->Add(2, cancel.c_str());
 }
@@ -713,31 +754,66 @@ void WGuiSplit::setModal(bool val)
 
 bool WGuiSplit::CheckUserInput(JButton key)
 {
+    bool result = false;
+
     if (hasFocus())
     {
+        int i,j;
+
+        if (key == JGE_BTN_NONE && JGE::GetInstance()->GetLeftClickCoordinates(i, j))
+        {   // a dude clicked somwhere, we're gonna select the closest object from where he clicked
+            unsigned int distanceLeft, distanceRight;
+
+            distanceLeft = static_cast<unsigned int>((left->getY() - j) * (left->getY() - j) + (left->getX() - i) * (left->getX() - i));
+            distanceRight = static_cast<unsigned int>((right->getY() - j) * (right->getY() - j) + (right->getX() - i) * (right->getX() - i));
+
+            if (distanceLeft < distanceRight && bRight)
+            {
+              key = JGE_BTN_LEFT;
+            }
+            else if(!bRight && distanceLeft > distanceRight)
+            {
+              key = JGE_BTN_RIGHT;
+            }
+        }
+
         if (!bRight)
         {
-            if (left->CheckUserInput(key)) return true;
+            if (left->CheckUserInput(key))  {
+                result = true;
+                goto done;
+            }
+
             if (key == JGE_BTN_RIGHT && !isModal() && right->Selectable() && left->Leaving(JGE_BTN_RIGHT))
             {
                 bRight = !bRight;
                 right->Entering(JGE_BTN_RIGHT);
-                return true;
+                result = true;
+                goto done;
             }
         }
         else
         {
-            if (right->CheckUserInput(key)) return true;
+            if (right->CheckUserInput(key)) {
+                result = true;
+                goto done;
+            }
             if (key == JGE_BTN_LEFT && !isModal() && left->Selectable() && right->Leaving(JGE_BTN_LEFT))
             {
                 bRight = !bRight;
                 left->Entering(JGE_BTN_LEFT);
-                return true;
+                result = true;
+                goto done;
             }
         }
-    }
 
-    return false;
+    }
+done:
+    if(result == true)
+    {
+        JGE::GetInstance()->LeftClickedProcessed();
+    }
+    return result;
 }
 
 void WGuiSplit::Update(float dt)
@@ -825,13 +901,12 @@ bool WGuiSplit::yieldFocus()
 }
 
 //WGuiMenu
-WGuiMenu::WGuiMenu(JButton next = JGE_BTN_RIGHT, JButton prev = JGE_BTN_LEFT, bool m, WSyncable * syncme) :
-    WGuiItem("")
+WGuiMenu::WGuiMenu(JButton next, JButton prev, bool dPad, WSyncable * syncme) : WGuiItem("")
 {
     buttonNext = next;
     buttonPrev = prev;
     currentItem = -1;
-    mDPad = m;
+    mDPad = dPad;
     sync = syncme;
     held = JGE_BTN_NONE;
 }
@@ -897,18 +972,18 @@ bool WGuiMenu::CheckUserInput(JButton key)
         int n = currentItem;
         unsigned int distance2;
         unsigned int minDistance2 = -1;
-        for(size_t k=0; k < items.size(); k++)
+        for(size_t k = 0; k < items.size(); k++)
         {
-          WGuiItem* pItem = (WGuiItem*)items[k];
-          distance2 = (pItem-> getY() - j) * (pItem-> getY() - j) + (pItem-> getX() - i) * (pItem-> getX() - i);
-          if (distance2 < minDistance2)
+          WGuiBase* pItem = items[k];
+          distance2 = static_cast<unsigned int>((pItem->getY() - j) * (pItem->getY() - j) + (pItem->getX() - i) * (pItem->getX() - i));
+          if (distance2 < minDistance2 && pItem->Selectable())
           {
               minDistance2 = distance2;
               n = k;
           }
         }
 
-        if (n != currentItem)
+        if (n != currentItem && items[n]->Selectable())
         {
             setSelected(n);
             mEngine->LeftClickedProcessed();
@@ -936,7 +1011,7 @@ bool WGuiMenu::CheckUserInput(JButton key)
         {
             held = buttonNext;
             duration = 0;
-            if (nextItem()) return true;
+             if (nextItem()) return true;
         }
         else if (held == buttonNext && duration > 1)
         {
@@ -947,6 +1022,8 @@ bool WGuiMenu::CheckUserInput(JButton key)
 
     if (currentItem >= 0 && currentItem < nbitems) result = items[currentItem]->CheckUserInput(key);
 
+    mEngine->LeftClickedProcessed();
+
     return result;
 }
 void WGuiMenu::syncMove()
@@ -1024,9 +1101,7 @@ bool WGuiMenu::nextItem()
 
     while (potential < nbitems - 1 && items[potential]->Selectable() == false)
         potential++;
-    if (potential == nbitems || !items[potential]->Selectable())
-        potential = -1;
-    else if (potential != currentItem && (!now || now->Leaving(buttonNext)))
+    if (potential != currentItem && (!now || now->Leaving(buttonNext)))
     {
         currentItem = potential;
         items[currentItem]->Entering(buttonNext);
@@ -1055,9 +1130,9 @@ bool WGuiMenu::prevItem()
 
     while (potential > 0 && items[potential]->Selectable() == false)
         potential--;
-    if (potential < 0 || !items[potential]->Selectable())
-        potential = -1;
-    else if (potential != currentItem && (!now || now->Leaving(buttonNext)))
+
+    if ( (!(potential < 0 || !items[potential]->Selectable()))
+          && (potential != currentItem && (!now || now->Leaving(buttonNext))))
     {
         currentItem = potential;
         items[currentItem]->Entering(buttonPrev);
@@ -1118,7 +1193,6 @@ bool WGuiTabMenu::CheckUserInput(JButton key)
 {
     bool result = false;
     bool kidModal = false;
-    bool handledInput = false;
     int nbitems = (int) items.size();
     JGE * mEngine = JGE::GetInstance();
     int i, j;
@@ -1218,7 +1292,7 @@ void WGuiAward::Underlay()
     char buf[1024];
     JQuadPtr trophy;
 
-    string n = Options::getName(id);
+    string n = id ? Options::getName(id) : textId;
     if (n.size())
     {
         sprintf(buf, "trophy_%s.png", n.c_str()); //Trophy specific to the award
@@ -1241,7 +1315,7 @@ void WGuiAward::Underlay()
 }
 void WGuiAward::Render()
 {
-    GameOptionAward * goa = dynamic_cast<GameOptionAward*> (&options[id]);
+    GameOptionAward * goa = id ? dynamic_cast<GameOptionAward*> (&options[id]) : dynamic_cast<GameOptionAward*> (&options[textId]);
 
     if (!goa) return;
 
@@ -1284,6 +1358,17 @@ WGuiAward::WGuiAward(int _id, string name, string _text, string _details) :
     height = 60;
     details = _details;
 }
+
+WGuiAward::WGuiAward(string _id, string name, string _text, string _details) :
+    WGuiItem(name)
+{
+    id = 0;
+    textId = _id;
+    text = _text;
+    height = 60;
+    details = _details;
+}
+
 WGuiAward::~WGuiAward()
 {
     GameOptionAward * goa = dynamic_cast<GameOptionAward*> (&options[id]);
@@ -1292,7 +1377,7 @@ WGuiAward::~WGuiAward()
 bool WGuiAward::Visible()
 {
     //WGuiAward is only visible when it's tied to an already achieved award.
-    GameOptionAward * goa = dynamic_cast<GameOptionAward*> (&options[id]);
+    GameOptionAward * goa = id ? dynamic_cast<GameOptionAward*> (&options[id]) : dynamic_cast<GameOptionAward*> (&options[textId]);
     if (!goa || !goa->number) return false;
     return true;
 }
@@ -1359,14 +1444,14 @@ void WGuiCardImage::Render()
         JQuadPtr q;
         if (bThumb)
         {
-            q = WResourceManager::Instance()->GetQuad("back_thumb");
+            q = WResourceManager::Instance()->GetQuad(kGenericCardThumbnailID);
 #if defined WIN32 || defined LINUX
             if(!q)
-            q = WResourceManager::Instance()->GetQuad("back");
+            q = WResourceManager::Instance()->GetQuad(kGenericCardID);
 #endif
         }
         else
-            q = WResourceManager::Instance()->GetQuad("back");
+            q = WResourceManager::Instance()->GetQuad(kGenericCardID);
         float scale = p.actZ * 257.f / q->mHeight;
         q->SetColor(ARGB(255,255,255,255));
         renderer->RenderQuad(q.get(), p.x, p.y, 0, scale, scale);
@@ -1429,14 +1514,14 @@ void WGuiCardDistort::Render()
         //Default to back.
         if (bThumb)
         {
-            q = WResourceManager::Instance()->GetQuad("back_thumb");
+            q = WResourceManager::Instance()->GetQuad(kGenericCardThumbnailID);
 #if defined WIN32 || defined LINUX
             if(!q)
-            q = WResourceManager::Instance()->GetQuad("back");
+            q = WResourceManager::Instance()->GetQuad(kGenericCardID);
 #endif
         }
         else
-            q = WResourceManager::Instance()->GetQuad("back");
+            q = WResourceManager::Instance()->GetQuad(kGenericCardID);
     }
     else
     {
@@ -1514,7 +1599,7 @@ WDistort::WDistort(float x1, float y1, float x2, float y2, float x3, float y3, f
 
 void WGuiListRow::Render()
 {
-    int start = 0, nowPos = 0, vHeight = 0;
+    int start = 0, nowPos = 0;
     int nbitems = (int) items.size();
 
     //List is empty.
@@ -1540,7 +1625,6 @@ void WGuiListRow::Render()
         }
     }
 
-    vHeight = 0;
     nowPos = 4;
     float nowVPos = 4;
     float tallestRow = 0;
@@ -1604,7 +1688,7 @@ bool WGuiFilters::Finish(bool emptyset)
         {
             WCFilterFactory * wc = WCFilterFactory::GetInstance();
             WCardFilter * f = wc->Construct(src);
-            if (recolorTo > -1 && recolorTo < Constants::MTG_NB_COLORS)
+            if (recolorTo > -1 && recolorTo < Constants::NB_Colors)
             {
                 f = NEW WCFilterAND(f, NEW WCFilterColor(recolorTo));
             }
@@ -1612,7 +1696,7 @@ bool WGuiFilters::Finish(bool emptyset)
         }
         else
         {
-            if (recolorTo > -1 && recolorTo < Constants::MTG_NB_COLORS)
+            if (recolorTo > -1 && recolorTo < Constants::NB_Colors)
             {
                 WCardFilter * f = NEW WCFilterColor(recolorTo);
                 source->addFilter(f);
@@ -1883,7 +1967,7 @@ void WGuiFilterItem::updateValue()
         SAFE_DELETE(mParent->subMenu);
         mState = STATE_CHOOSE_TYPE;
         SAFE_DELETE(mParent->subMenu);
-        mParent->subMenu = NEW SimpleMenu(-1234, this, Fonts::MENU_FONT, 20, 20, "Filter By...", 10);
+        mParent->subMenu = NEW SimpleMenu(JGE::GetInstance(), -1234, this, Fonts::MENU_FONT, 20, 20, "Filter By...", 10);
         if (mParent->isAvailable(FILTER_SET))
         {
             mParent->subMenu->Add(FILTER_SET, "Set");
@@ -1951,17 +2035,22 @@ void WGuiFilterItem::updateValue()
         SAFE_DELETE(mParent->subMenu);
         mParent->clearArgs();
         mState = STATE_CHOOSE_VAL;
-        mParent->subMenu = NEW SimpleMenu(-1234, this, Fonts::MENU_FONT, 20, 20, "Filter:");
+        mParent->subMenu = NEW SimpleMenu(JGE::GetInstance(), -1234, this, Fonts::MENU_FONT, 20, 20, "Filter:");
         if (filterType == FILTER_TYPE)
         {
             mParent->addArg("Artifact", "t:Artifact;");
             mParent->addArg("Artifact Creature", "t:Artifact;&t:Creature;");
+            mParent->addArg("Aura", "t:Aura;");
+            mParent->addArg("Basic", "t:Basic;");
             mParent->addArg("Creature", "t:Creature;");
             mParent->addArg("Enchantment", "t:Enchantment;");
+            mParent->addArg("Equipment", "t:Equipment;");
             mParent->addArg("Instant", "t:Instant;");
             mParent->addArg("Land", "t:Land;");
             mParent->addArg("Legendary", "t:Legendary;");
             mParent->addArg("Sorcery", "t:Sorcery;");
+            mParent->addArg("Tribal", "t:Tribal;");
+
         }
         else if (filterType == FILTER_SUBTYPE)
         {
@@ -2254,7 +2343,7 @@ WGuiBase::CONFIRM_TYPE WGuiKeyBinder::needsConfirm()
             confirmationString = ss.str();
 
             // Then create the menu.
-            confirmMenu = NEW SimpleMenu(0, this, Fonts::MENU_FONT, 40, 130, "Conflict");
+            confirmMenu = NEW SimpleMenu(JGE::GetInstance(), 0, this, Fonts::MENU_FONT, 40, 130, "Conflict");
             confirmMenu->Add(1, _("Cancel and return to the options menu").c_str());
             confirmMenu->Add(2, _("This is okay, validate and save").c_str());
             return CONFIRM_NEED;
@@ -2262,6 +2351,7 @@ WGuiBase::CONFIRM_TYPE WGuiKeyBinder::needsConfirm()
     }
 
     // Check whether any button has no key associated to it.
+#if (!defined IOS)
     confirmingButton = JGE_BTN_NONE;
     for (signed int i = (sizeof(btnToCheck) / sizeof(btnToCheck[0])) - 1; i >= 0; --i)
     {
@@ -2281,12 +2371,13 @@ WGuiBase::CONFIRM_TYPE WGuiKeyBinder::needsConfirm()
         confirmationString = s;
 
         confirmingButton = btnToCheck[i];
-        confirmMenu = NEW SimpleMenu(1, this, Fonts::MENU_FONT, 40, 130, "Binding missing");
+        confirmMenu = NEW SimpleMenu(JGE::GetInstance(), 1, this, Fonts::MENU_FONT, 40, 130, "Binding missing");
         confirmMenu->Add(1, _("Cancel and return to the options menu").c_str());
         confirmMenu->Add(2, _("This is okay, validate and save").c_str());
         return CONFIRM_NEED;
     }
-
+#endif // IOS
+    
     return CONFIRM_OK;
 }
 
